#!/bin/sh

# Licensed Materials - Property of IBM
# IBM Cognos Products: XQE
# (C) Copyright IBM Corp. 2009, 2010
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
JavaHomeError() {
	echo "Could not find path to java."
	echo "The environment variable JAVA_HOME is not defined or incorrectly defined."
	echo "Please define JAVA_HOME first."
	exit 1;
}

invalidInput() {
	echo "ERROR Encountered!! Invalid input value passed to utility QueryServiceAdminTask. For help use command [ queryServiceAdminTask.sh -help ] ."
}
help() {
	echo "."
	echo "Use the QueryServiceAdminTask to specify query service operations. This utility expects two input parameters:  TaskType(String Input) and TaskParameter(String Input)."
	echo "."
	echo "The first (mandatory) parameter, TaskType(String Input), identifies the type of operation. Use one of the following values. For TaskType 2, a file called SALDump.xml is created in the <c10_location>/logs/XQE folder."
	echo "."
	echo "."
	echo "------------------------------------------------------------------------"
	echo "1 - Clear Secure Cache."
	echo "2 - Write Secure Cache State."
	echo "------------------------------------------------------------------------"
	echo "."
	echo "."
	echo "The second (optional) parameter, TaskParameter(String Input), identifies the values used to perform the operation specified by the first parameter."
	echo "."
	echo "Use the format, \"<datasource_name>/<catalog_name>/<cube_name>\". You can use the wild card character \"*\" for any datasource, catalog, or cube name. For example, \"\\*/\\*/\\*\" or \"DS_TEMP/\\*/\\*\". When no input is used for this parameter, it uses the value \"*/*/*\" by default."
	echo "."
	echo "In this example, [ queryServiceAdminTask.sh 1 \"\\*/\\*/\\*\" ] the secured cache is cleared for all cubes in all catalogs under all datasources."
}
gotJava() {	
	J_OPTS="-Dlog4j.configuration=INFO"
	RUNJAVA="$java_home/bin/java"	
       
        JAVA_64BIT_OPTS=""
        if [ -f "$RUNJAVA" ]
        then 
             IS_DUAL_MODE_JRE='${RUNJAVA} -help 2>&1 | grep d64'
             if [ "$IS_DUAL_MODE_JRE" != "" ]
             then
                  JAVA_64BIT_OPTS="-d64" 
             fi
        fi 

	CLASSPATH="../webapps/p2pd/WEB-INF/classes:../webapps/p2pd/WEB-INF/lib/cclcfgapi.jar:../webapps/p2pd/WEB-INF/lib/jcam_crypto.jar:../webapps/p2pd/WEB-INF/lib/CognosIPF.jar:../webapps/p2pd/WEB-INF/lib/log4j-1.2.8.jar:../tomcat/lib/jcam_jni.jar:../webapps/p2pd/WEB-INF/lib/xqeService.jar:../webapps/p2pd/WEB-INF/lib/hessian-3.0.8.jar:../webapps/p2pd/WEB-INF/lib/mx4j-tools.jar:../webapps/p2pd/WEB-INF/lib/CognosCCL4J.jar:../webapps/p2pd/WEB-INF/lib/commons-lang-2.3.jar:../webapps/p2pd/WEB-INF/lib/commons-configuration-1.5.jar:../webapps/p2pd/WEB-INF/lib/commons-logging-1.1.jar:../webapps/p2pd/WEB-INF/lib/commons-collections-3.2.1.jar"
	echo "Using CLASSPATH: $CLASSPATH"
	echo "Using J_HOME:    $java_home"
	echo "Using J_OPTS:    $J_OPTS"	
	param1=$1
	param2=$2
	echo "\nCommand:" "$RUNJAVA" "$JAVA_64BIT_OPTS" "$J_OPTS" -cp "$CLASSPATH"  com.cognos.xqe.adminconsole.commandlineutility.QueryServiceAdminTask "$temp"
	"$RUNJAVA" "$JAVA_64BIT_OPTS" "$J_OPTS" -cp "$CLASSPATH"  com.cognos.xqe.adminconsole.commandlineutility.QueryServiceAdminTask $1 $2
	exit $?


}
findJava() {	
	java_home="jre/6.0"
	if [ -f "$java_home/bin/java" ]
	then
		gotJava $*;	
	else 	
		if [ -z "$JAVA_HOME" ]
		then
			JavaHomeError;
		else
			java_home="$JAVA_HOME"
			echo Looking for JRE in: "$java_home/bin..."
			if [ -f "$java_home/bin/java" ]
			then			
				gotJava $*;			
			else 
				java_home="$JAVA_HOME/jre"
				echo Looking for JRE in: "$java_home/bin..."
				if [ -f "$java_home/bin/java" ]
				then				
					gotJava $*;	
				else 
					JavaHomeError;
				fi
			fi
		fi
	fi		
}
OS_NAME=`uname`
CWD=$PWD
	case "$OS_NAME" in
	"Linux"|"SunOS") 
        LD_LIBRARY_PATH=$CWD:$CWD/bind:$CWD/lib
        export LD_LIBRARY_PATH 
        ;;  
        "AIX") export LIBPATH=$CWD:$CWD/bind:$CWD/lib
	  ;;
	   "HP-UX") export SHLIB_PATH=$CWD:$CWD/bind:$CWD/lib
	;;   
	esac
if [ -z "$1" ]
then
 invalidInput;
else
	case $1 in 
	"1") findJava $*;;
		
	"2") findJava $*;;	
	"-help") help;;
	*) invalidInput;;	
	esac   
fi


