<?xml version="1.0" encoding="UTF-8"?>
<!--
  Licensed Materials - Property of IBM
  IBM Cognos Products: Viewer
  (C) Copyright IBM Corp. 2001, 2010

  US Government Users Restricted Rights - Use, duplication or
  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:rsext="xalan://com.cognos.reportserver.ext.RSExt" xmlns:bus="http://developer.cognos.com/schemas/bibus/3/" version="1.0" exclude-result-prefixes="xsl rsext bus">
	<xsl:include href="qrsvpCommon.xsl"/>
	<xsl:include href="V5drillthru.xsl"/>
	<xsl:param name="CVDisableSelection" select="''"/>
	<xsl:param name="CVDisableDrill" select="''"/>
	<xsl:param name="genIntermediateXML" select="''"/>
	<xsl:param name="CVObjectRef" select="'window.oCV'"/>
	<xsl:param name="CVSCObjectRef" select="'window.oCVSC'"/>
	<xsl:param name="restoreOldDrillChartBehavior" select="'false'"/>

	<xsl:variable name="CVJSObjectRef" select="concat($CVObjectRef, $CognosViewerID)" />
	<xsl:variable name="CVSCJSObjectRef" select="concat($CVSCObjectRef, $CognosViewerID)" />

	<xsl:output method="html" version="1.0" encoding="UTF-8" indent="no"/>
	<!-- only run these in Query Studio -->
	<xsl:template name="startPage"/>
	<xsl:template name="dropIndicators"/>
	<xsl:template name="drillTargetsSetup">
		<xsl:if test="$drillSupport!='no'">
			<xsl:call-template name="addDrillThroughContext">
				<xsl:with-param name="closeScript" select="'no'"/>
			</xsl:call-template>
			<xsl:value-of select="$CVJSObjectRef"/>.rvMainWnd.addDrillTargets(drillTargets);
			<xsl:text disable-output-escaping="yes">&lt;/script&gt;</xsl:text>
		</xsl:if>
	</xsl:template>
	<!-- used in selection -->
	<xsl:template name="GenEventHandlers">
		<xsl:if test="./drillTargets or /Document/META-DATA">
			<xsl:if test="@layoutClass='listColumnBody' or @layoutClass='listColumnTitle' or @layoutClass='rowCell' or @layoutClass='crosstabFactCell' or @layoutClass='crosstabNodeMember' or @layoutClass='crosstabCorner' or @layoutClass='repeaterTableCell' or name()='OCHART'">
				<xsl:if test="@cid != ''">
					<xsl:attribute name="cid"><xsl:value-of select="@cid"/></xsl:attribute>
				</xsl:if>
				<xsl:if test="@uid != ''">
					<xsl:attribute name="uid"><xsl:value-of select="@uid"/></xsl:attribute>
					<xsl:attribute name="name">c<xsl:value-of select="$CognosViewerID"/><xsl:value-of select="@uid"/></xsl:attribute>
				</xsl:if>
				<xsl:if test="@type != ''">
					<xsl:attribute name="type"><xsl:value-of select="@type"/></xsl:attribute>
				</xsl:if>
				<xsl:if test="@ctx != ''">
					<xsl:attribute name="ctx"><xsl:value-of select="@ctx"/></xsl:attribute>
				</xsl:if>
				<xsl:if test="@class">
					<xsl:attribute name="class">
						<xsl:if test="$CVDisableSelection != 'true'">
							<xsl:text>c</xsl:text><xsl:value-of select="$CognosViewerID"/><xsl:value-of select="@uid"/><xsl:text> </xsl:text>
						</xsl:if>
						<xsl:value-of select="@class" disable-output-escaping="yes"/>
						<xsl:if test="@styleId and $vUseStyleId='true'">
							<xsl:text> </xsl:text><xsl:value-of select="@styleId" disable-output-escaping="yes"/>
						</xsl:if>
					</xsl:attribute>
				</xsl:if>
			</xsl:if>
		</xsl:if>
		<xsl:if test="name() = 'OCHART'">
			<xsl:attribute name="rsvpChart">true</xsl:attribute>
			<xsl:if test="$isIE and $CVDisableDrill != 'true'">
				<xsl:attribute name="onmouseover"><xsl:value-of select="$CVJSObjectRef"/>.goDrillManager.ddc(event);</xsl:attribute>
				<xsl:attribute name="onmouseout"><xsl:value-of select="$CVJSObjectRef"/>.goDrillManager.hdc(event);</xsl:attribute>
			</xsl:if>
		</xsl:if>
	</xsl:template>
	<!-- used to add events to the main page for selection -->
	<xsl:template match="*[@layoutClass='page']" priority="1">
		<xsl:if test="string($genIntermediateXML) = 'true'">
			<textarea id="intermediateXML{$CognosViewerID}" style="visibility:hidden; position:absolute;"><htmlFragment><xsl:copy-of select="/"/></htmlFragment></textarea>
		</xsl:if>

		<table id="rt{$CognosViewerID}" cellpadding="0" cellspacing="0" border="0">
		<!--textarea><htmlFragment><xsl:copy-of select="/"/></htmlFragment></textarea-->
			<xsl:if test="$accessible='true'">
				<xsl:attribute name="role">presentation</xsl:attribute>
			</xsl:if>
			<xsl:if test="$CVDisableSelection != 'true'">
				<xsl:attribute name="onmousemove">if (typeof <xsl:value-of select="$CVSCJSObjectRef"/> != 'undefined') <xsl:value-of select="$CVSCJSObjectRef"/>.pageHover(event);</xsl:attribute>
				<xsl:attribute name="onclick"><xsl:value-of select="$CVJSObjectRef"/>.rvMainWnd.pageClicked(event);</xsl:attribute>
				<xsl:attribute name="onkeydown"><xsl:value-of select="$CVJSObjectRef"/>.onKeyDown(event);</xsl:attribute>
				<xsl:attribute name="onDblClick">if (typeof <xsl:value-of select="$CVSCJSObjectRef"/> != 'undefined') <xsl:value-of select="$CVSCJSObjectRef"/>.pageDoubleClicked(event);</xsl:attribute>
				<xsl:attribute name="onContextMenu"><xsl:value-of select="$CVJSObjectRef"/>.dcm(event, true)</xsl:attribute>
			</xsl:if>
			<xsl:call-template name="GenCommonAttributes"/>
			<xsl:apply-templates select="*"/>
		</table>
	</xsl:template>

	<xsl:template match="*[local-name()='OCHART']" priority="1">
		<xsl:variable name="uniqueId"><xsl:value-of select="concat($CognosViewerID, generate-id(.))"/></xsl:variable>
		<span>
			<xsl:attribute name="tabIndex">0</xsl:attribute>
			<xsl:attribute name="chartContainer">true</xsl:attribute>
			<xsl:if test="$accessible='true'">
				<xsl:attribute name="role">presentation</xsl:attribute>
				<xsl:attribute name="aria-labelledby"><xsl:value-of select="$uniqueId"/></xsl:attribute>
				<span style="visibility:hidden; display:none" id="{$uniqueId}"><xsl:value-of select="@altText"/></span>
			</xsl:if>
			<xsl:call-template name="GenChartImage"/>
		</span>
	</xsl:template>

	<xsl:template match="*[@layoutClass='textItem']" priority="1">
		<span>
			<xsl:call-template name="GenTabIndexAttribute"/>

			<xsl:call-template name="GenCommonAttributes"/>

			<xsl:call-template name="AddAccessibleTagsForSpan"/>

			<!--
				First cell in a list or crosstab has a tabIndex of 0. We need the onfocus for those cells
				to fix up the aria-labelled before JAWS kicks in.
			 -->
			<xsl:if test="$accessible='true' and (parent::node()/@layoutClass = 'crosstabCorner' or (parent::node()/@layoutClass = 'listColumnTitle' and count(parent::node()/preceding-sibling::node())=0))">
				<xsl:attribute name="onfocus"><xsl:value-of select="$CVJSObjectRef"/>.onFocus(event);</xsl:attribute>
			</xsl:if>

			<xsl:if test="$CVDisableDrill != 'true'">
				<xsl:choose>
					<xsl:when test="name(parent::node())='drillTargets'">
						<xsl:if test="parent::node()/../@isolated='true' or parent::node()/@isolated='true'">
							<xsl:attribute name="isolated">true</xsl:attribute>
						</xsl:if>
					</xsl:when>
					<xsl:otherwise>
						<xsl:if test="parent::node()/@drillUp = 'true' or parent::node()/@drillDown = 'true' or ./@drillDown='true' or ./@drillUp='true' or parent::node()/../@drillUp = 'true' or parent::node()/../@drillDown = 'true'">
							<xsl:variable name="isIsolated">
								<xsl:choose>
									<xsl:when test="parent::node()/@isolated='true' or ./@isolated='true'">true</xsl:when>
									<xsl:otherwise>false</xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
							<xsl:if test="(parent::node()/@drillDown = 'true' or ./@drillDown='true' or parent::node()/../@drillDown = 'true') and $isIsolated='false'">
								<xsl:attribute name="class"><xsl:text>dl </xsl:text><xsl:value-of select="@layoutClass" disable-output-escaping="yes"/></xsl:attribute>
							</xsl:if>
							<xsl:if test="$isIsolated='true'">
								<xsl:attribute name="isolated">true</xsl:attribute>
							</xsl:if>
						</xsl:if>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
			<xsl:if test="@ctx != ''">
				<xsl:attribute name="ctx"><xsl:value-of select="@ctx"/></xsl:attribute>
			</xsl:if>

			<xsl:value-of select="rsext:htmlencode(string(.))" disable-output-escaping="yes"/>
		</span>
	</xsl:template>
	<xsl:template match="OAREA" priority="2">
		<xsl:if test="./MEMBER or ./drillTargets or ./MEASURE">
			<xsl:if test="./MEMBER or ./MEASURE">
			<xsl:choose>
				<xsl:when test="string($restoreOldDrillChartBehavior) = 'true'">
					<xsl:attribute name="ctx">
						<xsl:for-each select="./*[name()='MEASURE']">
							<xsl:value-of select="./@ctx"/>
							<xsl:if test="position()!=last()">
								<xsl:text>:</xsl:text>
							</xsl:if>
						</xsl:for-each>

						<xsl:if test="./*[name()='MEMBER'] and ./*[name()='MEASURE']">
							<xsl:text>::</xsl:text>
						</xsl:if>

						<xsl:for-each select="./*[name()='MEMBER']">
								<xsl:value-of select="./@ctx"/>
								<xsl:if test="position()!=last()">
									<xsl:text>::</xsl:text>
								</xsl:if>
						</xsl:for-each>
					</xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="ctx"><xsl:value-of select="./@ctx"/></xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:attribute name="class"><xsl:text>dl </xsl:text><xsl:value-of select="@layoutClass" disable-output-escaping="yes"/></xsl:attribute>
			</xsl:if>
			<xsl:if test="./MEASURE and string(./MEASURE) != '' and $CognosViewerID = 'RS'">
				<xsl:attribute name="display"><xsl:value-of select="rsext:htmlencode(string(./MEASURE))" disable-output-escaping="yes"/></xsl:attribute>
			</xsl:if>
			<xsl:choose>
				<xsl:when test="./@type='legendTitle'">
					<xsl:attribute name="isChartTitle">true</xsl:attribute>
				</xsl:when>
			</xsl:choose>
			<xsl:choose>
				<xsl:when test="./@type='ordinalAxisTitle'">
					<xsl:attribute name="isChartTitle">true</xsl:attribute>
				</xsl:when>
			</xsl:choose>
			<xsl:if test="./drillTargets">
				<xsl:attribute name="dtTargets">
					<xsl:apply-templates select="./drillTargets/*" mode="drillTargets"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$isIE">
				<xsl:attribute name="onmouseup"><xsl:value-of select="$CVJSObjectRef"/>.pcc(event)</xsl:attribute>
			</xsl:if>
			<xsl:attribute name="href">#</xsl:attribute>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
