## Licensed Materials - Property of IBM
## IBM Cognos Products: disp
## (C) Copyright IBM Corp. 2005 2010
## US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

##
## disp.py
##
## This sample script provides maintenance and troubleshooting procedures for 
## the Dispatcher via its Jython interface.
##
## This script must be named disp.py and be located in the <jython>/lib directory.
##
## Use a telnet client to access this script:
##   telnet localhost 9999
##
## then load the script: 
##   import disp
##   reload(disp)
##

def help():
  print "Dispatcher commands: "
  print "   disp.info()        - shows general information about the Dispatcher"
  print "   disp.props()       - lists all system properties as seen by the Dispatcher"
  print "   disp.loadProps()   - re-loads the properties defined in p2pd_deploy_defaults.properties"
  print "   disp.services()    - lists all Dispatcher services"
  print "   disp.uptime()      - shows how long the Dispatcher has been running for"
  print "   disp.version()     - displays the version of the Dispatcher"
  print ""
  print "   disp.check()       - checks for known memory leaks"
  print "   disp.fixit()       - fixes known memory leaks"
  print ""
  print "   disp.stacktraces() - Dumps all thread stack traces to the logs"
  print ""
  print "Jython commands: "
  print "   reload(disp)       - Reloads the disp.py script from disk (after being edited)"
  print "   dir(disp)          - Lists all commands available in this script"
  print ""

def info():
  _showVersion()
  _showWorkingDirectory()

def props():
  _showProperties()
  
def loadProps():
  _loadProperties()
  
def uptime():
  _showUptime()
  
def services():
  _showServices()
  
def version():
  _showVersion()
  _showCopyright()

def check():
 _checkForMemoryLeaks(0)
 
def fixit():
 _checkForMemoryLeaks(1)
 
def stacktraces():
 _dumpStackTraces()
 
 

def _showVersion():
  from com.cognos.pogo.util import Version
  ver = Version.getVersionStr()
  print "Dispatcher version is " + ver

def _showCopyright():
  print "(C) Copyright IBM Corp. 2009, 2010"

def _showWorkingDirectory():
  from java.io import File
  pwd = File(".")
  print "Current working directory is " + pwd.getAbsolutePath()
  
def _showProperties():
  from com.cognos.p2pd.util import PropertyInserter
  pnames = PropertyInserter.properties.propertyNames();
  print "Properties:"
  while pnames.hasMoreElements():
    pname = pnames.nextElement()
    value = PropertyInserter.getProperty(pname)
    print "   " + pname + "=" + value   
    
def _loadProperties():
  from com.cognos.p2pd.util import PropertyInserter
  from java.io import FileInputStream
  from com.cognos.cclcfgapi import CCLConfigurationFactory
  from com.cognos.cclcfgapi import ICCLConfiguration
  cclconfig = CCLConfigurationFactory.createConfiguration();
  cclconfig.init();
  fileName = cclconfig.resolveEffectivePath("../webapps/p2pd/WEB-INF/p2pd_deploy_defaults.properties");
  print "p2pd properties file:  " + fileName
  PropertyInserter.properties.load(FileInputStream(cclconfig.resolveEffectivePath(fileName)));     

def _showUptime():
  from com.cognos.pogo.transport import PogoServlet
  print "The dispatcher has been running since " + PogoServlet.starttime.toString()
  
def _showServices():
  from com.cognos.pogo.transport import PogoServlet
  servlet = PogoServlet.getInstance()
  engine = servlet.pogo_engine
  dispServices = engine.dispatcherServices
  serviceLoadMap = dispServices.serviceLoadMap
  for service in serviceLoadMap:
    print "  " + service.getName()
    
def _showHandlerPool():
  from com.cognos.pogo.transport import PogoServlet
  servlet = PogoServlet.getInstance()
  engine = servlet.pogo_engine
  handlerPool = engine.handler_pool
  handlers = handlerPool.handlers.keySet()
  for handler in handlers:
    print "  " + handler

def _checkForMemoryLeaks(fixit):
  from com.cognos.pogo.transport import PogoServlet
  servlet = PogoServlet.getInstance()
  engine = servlet.pogo_engine
  h = engine.handler_pool.find("reportService.warpmta")
  if h != None:
    ipr = h.inProgressRequests
    n = ipr.inProgressRequests.size()
    if n > 0:
      print "Possible memory leak detected, ReportServerRequests: %d objects found." % n
      if fixit == 1:
        ipr.inProgressRequests.clear()
        print "Memory leak removed."
    else:
      print "No known memory leaks detected."

def _dumpStackTraces():
  from java.lang import Thread
  from java.util import Map
  from java.util import TreeMap
  from java.lang import StackTraceElement

  ## Pick a log to send the stack traces to (stack traces can be *very* long)
  log = _getLog()
  log.warn("The following stack traces were generated via the Maintenance Console:");
  
  ## Retrieve all stack traces and sort them by thread name  
  traceMap = Thread.getAllStackTraces()
  sortedMap = TreeMap()
  for thread in traceMap.keySet():
    sortedMap.put(thread.getName(), thread)

  ## Dump all stack traces    
  for threadName in sortedMap.keySet():
    log.warn("Thread " + threadName + ":")
    thread = sortedMap.get(threadName)
    stackTrace = traceMap.get(thread)
    for element in stackTrace:
      log.warn("   " + element.toString())

  print "Stack traces for %d threads dumped to the dispatcher log." % traceMap.size()

def _getLog():
  from org.apache.log import Hierarchy
  return Hierarchy.getDefaultHierarchy().getLoggerFor("com.cognos.pogo.jython.JythonServer")

   