#! /usr/bin/env python

## Licensed Materials - Property of IBM
## IBM Cognos Products: disp
## (C) Copyright IBM Corp. 2005 2010
## US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

## IBMCopyrightScanTag1: This file contains both IBM and NON IBM code.
## Copyright (c) 2002-2006 Python Software Foundation.  All rights reserved.

"""

A multi-threaded telnet-like server that provides a Jython prompt.

For security reasons, it only accepts requests from the localhost.
This can still be insecure, but restricts violations to people that 
can log in on the server. As a safety precaution, this Jython server 
is disabled by default and can only be enabled by system administrators.

"""

import sys, os, string, getopt, thread, socket, traceback
from code import compile_command
from com.cognos.pogo.jython import JythonServer

log = JythonServer.log.getNativeLogger()

def main(port):
    main_thread(port)

def main_thread(port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.bind(("", port))
    sock.listen(5)
    log.info("Listening on port %d" % port);
    while 1:
        (conn, addr) = sock.accept()
        if isRemoteConnection(conn, addr):
            conn.close()
            log.info("Refusing connection from non-local host " + str(addr[0]))
            continue
        thread.start_new_thread(service_thread, (conn, addr))
        del conn, addr

def isRemoteConnection(conn, addr):
    return addr[0] != conn.getsockname()[0] and addr[0] != "localhost" and addr[0] != "loopback"
    
def service_thread(conn, addr):
    (caddr, cport) = addr
    log.info("Accepting connection from " + str(caddr) + " with thread " + str(thread.get_ident()))
    stdin = conn.makefile("r")
    stdout = conn.makefile("w", 0)
    run_interpreter(stdin, stdout)
    log.info("Thread " + str(thread.get_ident()) + " has finished")

def remove_backspaces(line):
    from java.lang import Character
    result = ""
    for i in range(0, len(line)):
       ch = line[i]
       if ch == '\x08':
         result = result[:-1] # Backspace, discard the last character
       else:
         result = result + ch
    return result
    
def run_interpreter(stdin, stdout):
    globals = {}

    stdout.write("\nIBM Cognos Jython Interface\n\n")

    try:
        str(sys.ps1)
    except:
        sys.ps1 = ">>> "
    source = ""

    while 1:
        stdout.write(sys.ps1)
        line = stdin.readline()
        line = remove_backspaces(line)
        if line[:2] == '\377\354':
            line = ""
        if not line and not source:
            break
        if line[-2:] == '\r\n':
            line = line[:-2] + '\n'
        source = source + line
        try:
            code = compile_command(source)
        except SyntaxError, err:
            source = ""
            traceback.print_exception(SyntaxError, err, None, file=stdout)
            continue
        if not code:
            continue
        source = ""
        try:
            run_command(code, stdin, stdout, globals)
        except SystemExit, how:
            if how:
                try:
                    how = str(how)
                except:
                    how = ""
                stdout.write("Exit %s\n" % how)
            break
    stdout.write("\nGoodbye.\n")

def run_command(code, stdin, stdout, globals):
    save = sys.stdin, sys.stdout, sys.stderr
    try:
        sys.stdout = sys.stderr = stdout
        sys.stdin = stdin
        try:
            exec code in globals
        except SystemExit, how:
            raise SystemExit, how, sys.exc_info()[2]
        except:
            type, value, tb = sys.exc_info()
            if tb: tb = tb.tb_next
            traceback.print_exception(type, value, tb)
            del tb
    finally:
        sys.stdin, sys.stdout, sys.stderr = save
