/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */
/**
CCDManager -- Report Viewer class which manages Context Data supplied by RSVP in JSON format
*/

// Constructor
function CCDManager(cv) {
	this.m_cd = null;
	this.m_md = null;
	this.m_oCV = null;
}


// Set functions
CCDManager.prototype.SetContextData = function(CD) {
	if (this.m_cd)
	{
		this.m_cd = null;
	}

	this.m_cd = CD;
};

CCDManager.prototype.SetMetadata = function(MD) {
	if (this.m_md)
	{
		this.m_md = null;
	}

	this.m_md = MD;
};

CCDManager.prototype.AddContextData = function(CD) {
	if (!this.m_cd) {
		this.m_cd = CD;
	} else {
		// Add additional context data
		for (var i in CD) {
			this.m_cd[i] = CD[i];
		}
	}
};

CCDManager.prototype.AddMetadata = function(MD) {
	if (!this.m_md) {
		this.m_md = MD;
	} else {
		// Add additional metadata
		for (var j in MD) {
			this.m_md[j] = MD[j];
		}
	}
};


CCDManager.prototype.SetCognosViewer = function(viewer) {
	if (viewer) {
		this.m_oCV = viewer;
	}
};


CCDManager.prototype.onComplete_GetCDRequest = function(asynchJSONResponse) {
	if (asynchJSONResponse) {
		var jsonResponse = asynchJSONResponse.getResult();
		var xmlResponse = XMLBuilderLoadXMLFromString(xml_decode(jsonResponse.xml));
		if (xmlResponse) {
			var allBlocks = xmlResponse.getElementsByTagName("Block");
			for (var i = 0; i < allBlocks.length; i++)	{
				var sContext = "";
				var blockNode = allBlocks[i].firstChild;
				while(blockNode)
				{
					sContext += blockNode.nodeValue;
					blockNode = blockNode.nextSibling;
				}

				var cd = eval('('+ sContext +')');
				this.AddContextData(cd);
			}

			if (typeof g_firefoxPluginHelper != "undefined" && g_firefoxPluginHelper != null)
			{
				g_firefoxPluginHelper.addDebugLogsFromResponseXML(xmlResponse);
			}
		}
	}
	clearLastViewerAction(this.m_oCV);
};

CCDManager.prototype.FetchContextData = function(ctxids) {
	var missingCtxids = [];
	var c = null, ctxLen = ctxids.length;
	for (var i = 0; i < ctxLen; ++i ) {
		c = ctxids[i];
		if (c != "" && !this.ContextIdExists(c)) {
			missingCtxids.push(c);
		}
	}
	if (missingCtxids.length) {
		if (this.m_oCV) {
			this.getContextData(missingCtxids);
		}
	}

	//Note that this is not the number fetched (they come back in blocks controlled by the
	//ContextBlockSize option), but rather the number of ctxids which did not have context info
	return missingCtxids.length;
};

CCDManager.prototype.getContextData = function(ctxids)
{
	var oCV = this.m_oCV;
	var callbacks = {};
	var asynchRequest = new AsynchJSONRequest(oCV.getGateway(), oCV.getWebContentRoot(), callbacks);

	if (oCV.isBux) {
		asynchRequest.setPromptDialog(new BuxPromptDialog(oCV.getId()));
		asynchRequest.setLogonDialog(new BuxLogOnDialog());
		asynchRequest.setFaultDialog(new BuxFaultDialog());
	} else {
		asynchRequest.forceSynchronous();
		asynchRequest.setPromptDialog(new StandaloneViewerPromptDialog());
		asynchRequest.setLogonDialog(new StandaloneViewerLogOnDialog( oCV.getId()));
	}

	var form = document["formWarpRequest" + oCV.getId()];
	var conversation = oCV.getConversation();

	var tracking = oCV.getTracking();
	if (!tracking && form && form["m_tracking"] && form["m_tracking"].value) {
		tracking = form["m_tracking"].value;
	}
	if (conversation && oCV.envParams["ui.action"] != 'view') {
		callbacks["complete"] = {"object":this,"method":this.onComplete_GetCDRequest};
		asynchRequest.addFormField("ui.action", "getContext");
		asynchRequest.addFormField("ui.conversation", conversation);
	} else {
		callbacks["complete"] = {"object":this,"method":this.onComplete_GetCDRequest};
		var uiObject = form["ui.object"];
		if (typeof uiObject.length != 'undefined' && uiObject.length >1) {
			asynchRequest.addFormField("ui.object", form["ui.object"][0].value);
		} else {
			asynchRequest.addFormField("ui.object", form["ui.object"].value);
		}
		asynchRequest.addFormField("ui.action", "getObjectContext");
	}
	asynchRequest.addFormField("context.format", "initializer");
	asynchRequest.addFormField("context.type", "reportService");
	asynchRequest.addFormField("context.selection", ctxids.join(','));
	if (oCV.envParams["ui.routingServerGroup"] != null)
	{
		asynchRequest.addFormField("ui.routingServerGroup=", oCV.envParams["ui.routingServerGroup"]);
	}

	if (tracking) {
		asynchRequest.addFormField("m_tracking", tracking);
	}

	var formFields = asynchRequest.m_formFields;
	var formFieldNames = formFields.keys();
	oCV["LAST_ACTION"] = "getContextData";
	var paramsForRetry = {};
	for (var index = 0; index < formFieldNames.length; index++)
	{
		var name = formFieldNames[index];
		var value = formFields.get(formFieldNames[index]);
		paramsForRetry[name] = value;
	}
	oCV["LAST_ACTION_PARAMS"] = paramsForRetry;

	asynchRequest.sendRequest();
};


// Existential Tests
CCDManager.prototype.ContextIdExists = function(ctxid) {
	return (this.m_cd && this.m_cd[ctxid]?true:false);
};

CCDManager.prototype.HasContextData = function() {
	return (this.m_cd ? true:false);
};

CCDManager.prototype.HasMetadata = function() {
	return (this.m_md ? true:false);
};


// Access Functions
CCDManager.prototype._getMDPropertyFromCD = function(ctxid, sCdProp, sMdProp) {
	var p = null;
	this.FetchContextData([ctxid]);
	var cd = this.m_cd && this.m_cd[ctxid];
	if (cd) {
		var md = this.m_md[ cd[sCdProp] ];
		if (md) {
			p = md[sMdProp];
		}
	}
	return p;
};

// Properties Derived from Reference Data Item
CCDManager.prototype.GetDrillFlag = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'r', 'drill');
};

CCDManager.prototype.GetDrillFlagForMember = function(ctxid) {
	// Return the correct drill flag for members only
	var drillFlag = null;
	var d = this._getMDPropertyFromCD(ctxid, 'r', 'drill');
	if (d !== null && this.m_cd[ctxid].m) {
		drillFlag = d;
	}
	return drillFlag;
};

CCDManager.prototype.GetDataType = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'r', 'dtype');
};

CCDManager.prototype.GetUsage = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'r', 'usage');
};

CCDManager.prototype.GetHUN = function(ctxid) {
	var hun = null;
	var h = this._getMDPropertyFromCD(ctxid, 'r', 'h');
	if (h) {
		hun = this.m_md[h].h;
	}

	if (hun!=null && hun.indexOf("[__ns_")==0 ) {
		/* Query Framework will occasionally  return a HUN from a temporary namespace it uses for internal processing.
		 * To avoid downstream problems with this, any HUN that begins with  the [__ns_ prefix is removed here.
		 * QFW has been notified and should attempt to eliminate this situation.
		 */
		hun = null;
	}
	return hun;
};

CCDManager.prototype.GetQuery = function(ctxid) {
	var qry = null;
	var q = this._getMDPropertyFromCD(ctxid, 'r', 'q');
	if (q) {
		qry = this.m_md[q].q;
	}
	return qry;
};

CCDManager.prototype.GetDepth = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'r', 'level');
};


//Properties Derived from Context Data
CCDManager.prototype.GetDisplayValue = function(ctxid) {
	var useVal = null;
	this.FetchContextData([ctxid]);
	if (this.ContextIdExists(ctxid) && this.m_cd[ctxid]) {
		useVal = this.m_cd[ctxid].u;
	}
	return useVal;
};

CCDManager.prototype.GetPUN = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'p', 'p');
};

CCDManager.prototype.GetLUN = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'l', 'l');
};

CCDManager.prototype.GetMUN = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'm', 'm');
};

CCDManager.prototype.GetDUN = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'd', 'd');
};

CCDManager.prototype.GetQMID = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'i', 'i');
};

CCDManager.prototype.GetRDIValue = function(ctxid) {
	return this._getMDPropertyFromCD(ctxid, 'r', 'r');
};

CCDManager.prototype.getContextIdForMetaData = function(lun, hun)
{
	var metaArray = [ {"expression":lun, "type":"l"},{"expression":hun, "type" :"h"} ];

	for(var index = 0; index < metaArray.length; ++index)
	{
		var sMetaItem = metaArray[index].expression;
		var sType = metaArray[index].type;

		if(sMetaItem == "")
		{
			continue;
		}

		for(var metaDataItem in this.m_md)
		{
			if(this.m_md[metaDataItem][sType] == sMetaItem)
			{
				for(var refDataItem in this.m_md)
				{
					if(this.m_md[refDataItem].r && this.m_md[refDataItem][sType] == metaDataItem)
					{
						if(this.m_md[refDataItem].drill != 0)
						{
							for(var ctx in this.m_cd)
							{
								if(this.m_cd[ctx].r == refDataItem && this.m_cd[ctx].m)
								{
									return ctx;
								}
							}
						}
					}
				}
			}
		}
	}

	return "";
};


// Get Context id given a MUN - these methods assume that the context ids are available.
CCDManager.prototype.GetContextIdForMUN = function(mun) {
	var mdIndex = null;
	var ctxid = null;
	// Find the mun in the metadata
	for (var i in this.m_md) {
		if (this.m_md[i].m == mun) {
			mdIndex = i;
			break;
		}
	}
	if (mdIndex != null) {
		for (var j in this.m_cd) {
			if (this.m_cd[j].m == mdIndex) {
				ctxid = j;
				break;
			}
		}
	}
	return ctxid;
};

// Get Context ids with a given RDI (may be more than 1) - these methods assume that the context ids are available.
CCDManager.prototype.GetContextIdsForRDI = function(rdi) {
	var ctxids = [];
	// Find the mun in the metadata
	for (var i in this.m_md) {
		if (this.m_md[i].r == rdi) {
			ctxids.push(i);
		}
	}
	return ctxids;
};

// Return, if applicable, the min/max values CURRENTLY IN THE CONTEXT TABLE for this rdi
CCDManager.prototype.GetPageMinMaxForRDI = function(rdi) {
	var pageMin=null;
	var pageMax=null;
	var ctxids = this.GetContextIdsForRDI(rdi);

	//TODO: Until we know all context data has been fetched previously, we need
	//		to fetch it here to guarantee that we have the full page of data.
	this.FetchContextData([0]);

	for (var i in this.m_cd) {
		for (var j in ctxids) {
			if (this.m_cd[i].r == ctxids[j]) {
				var currentFloatValue = parseFloat(this.m_cd[i].u);
				if (currentFloatValue == this.m_cd[i].u) {
					if ( pageMin == null || currentFloatValue < pageMin) {
						pageMin = currentFloatValue;
					}
					if ( pageMax == null || currentFloatValue > pageMax) {
						pageMax = currentFloatValue;
					}
				}
			}
		}
	}

	if (pageMin != null && pageMax != null) {
		return eval('({ pageMin: ' + pageMin +', pageMax: ' + pageMax + '})');
	}
};

// Get Context id given a display value
CCDManager.prototype.GetContextIdForDisplayValue = function(value) {
	var ctxid = null;
	for (var i in this.m_cd) {
		if (this.m_cd[i].u == value) {
			ctxid = i;
			break;
		}
	}
	return ctxid;
};

// Get Context id given a use value
CCDManager.prototype.GetContextIdForUseValue = function(value) {
	var mdIndex = null;
	var mdValueType = null;
	var ctxid = null;
	// Find the value in the metadata
	for (var i in this.m_md) {
		var md = this.m_md[i];
		for (var j in md) {
			if (md[j] == value) {
				mdIndex = i;
				mdValueType = j;
				break;
			}
		}
	}

	if (mdIndex != null) {
		for (var k in this.m_cd) {
			if (this.m_cd[k][mdValueType] == mdIndex) {
				ctxid = k;
				break;
			}
		}
	}
	return ctxid;
};

// Dump the contents of the metadata table as a JSON string.
CCDManager.prototype.MetadataToJSON = function() {
	if (this.m_md) {
		return dojo.toJson(this.m_md);
	}
	return "";
};

// Dump the contents of the context data table as a JSON string.
CCDManager.prototype.ContextDataToJSON = function() {
	if (this.m_cd) {
		return dojo.toJson(this.m_cd);
	}
	return "";
};

// Dump the contents of A SUBSET of the context data table as a JSON string.
CCDManager.prototype.ContextDataSubsetToJSON = function(maxValuesPerRDI) {
	if (maxValuesPerRDI<=0) {
		return this.ContextDataToJSON();
	}
	if (this.m_cd) {
		var rdiCount = {};
		var cdSubset = {};

		for (var i in this.m_cd) {
			var rdiKey=this.m_cd[i].r;
			if (typeof rdiKey != "undefined") {
				if (rdiCount[rdiKey]==null) {
					rdiCount[rdiKey]=0;
				} else {
					rdiCount[rdiKey]++;
				}
				if (rdiCount[rdiKey] < maxValuesPerRDI) {
					cdSubset[i]=this.m_cd[i];
				}
			}
		}
		return dojo.toJson(cdSubset);
	}
	return "";
};

