/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

dojo.provide("InfoBarBase");

// InfoBar Base class
dojo.declare("InfoBarBase", null, {
	constructor : function(cognosViewer) {
		this.m_cognosViewer = cognosViewer;
		this.m_parameterValues = new CParameterValues();

		var documentNode = XMLBuilderLoadXMLFromString(this.m_cognosViewer.getExecutionParameters());
		if (documentNode.childNodes.length == 1) {
			this.m_parameterValues.loadWithOptions(documentNode.childNodes[0], /*credentials*/false);
		}
		this.m_parameterStringOperators = new CParameterValueStringOperators(RV_RES.IDS_JS_FILTER_BETWEEN,
																			 RV_RES.IDS_JS_FILTER_NOT_BETWEEN,
																			 RV_RES.IDS_JS_FILTER_LESS_THAN,
																			 RV_RES.IDS_JS_FILTER_GREATER_THAN,
																			 RV_RES.IDS_JS_INFOBAR_AND_TEXT,
																			 RV_RES.IDS_JS_FILTER_EXCLUDE);
		this.c_SliderCheckboxParameterPrefix = "BusinessInsight."; //non-nls string constant
		this.m_containerInfo = null;
		this.m_editableFilters = null;
		this.m_editableSorts = null;
		this.m_id = null;
		this.m_layoutIndex = null;
	},

	setContainerInfo: function(containerInfo) {
		this.m_containerInfo = containerInfo;
	},

	setEditableFilters: function(editableFilters) {
		this.m_editableFilters = editableFilters;
	},

	setId: function(theId) {
		this.m_id = theId;
	},

	getId: function() {
		return this.m_id;
	},

	hasPrompts : function() {
		var numberOfParameters = this.m_parameterValues.length();
		for (var index = 0; index < numberOfParameters; ++index) {
			var parameter = this.m_parameterValues.getAt(index);
			if (parameter !== null && parameter.length() > 0 &&
				parameter.name().indexOf(this.c_SliderCheckboxParameterPrefix) !== 0) {
				return true;
			}
		}
		return false;
	},

	addPromptDetails : function(viewOnlyMode) {
		var promptDetails = "";
		var showControl = (viewOnlyMode !== true); // showControl is true unless the viewOnlyMode is passed with value of "true"

		if (this.hasPrompts()) {
			var staticLabelledBy = this.m_id + "instructions " + this.m_id + "_promptString " + this.m_id + "_lockedString ";

			for (var index = 0, numberOfParameters = this.m_parameterValues.length(); index < numberOfParameters; ++index) {
				var labelledBy = staticLabelledBy + this.m_id + index + "_promptDetail";
				var parameter = this.m_parameterValues.getAt(index);
				var numberOfPromptValueItems = parameter.length();

				if (parameter !== null && numberOfPromptValueItems > 0 &&
					parameter.name().indexOf(this.c_SliderCheckboxParameterPrefix) !== 0) {
					promptDetails += "<tr>";
					promptDetails += "<td class=\"infoBarLeft\"></td>";
					promptDetails += "<td class=\"infoBarDetailRow\" aria-labelledby=\"" + labelledBy + "\" role=\"listitem\"" + (!showControl?">":" tabindex=\"-1\">");
					promptDetails += "<table><tr>";
					promptDetails += this.addInfoBarIcon("infobar_filter.gif", true);
					promptDetails += "<td class=\"infoBarDetailsText\" id=\"" + this.m_id + index + "_promptDetail\">";
					promptDetails += "<span class=\"bibi\">";
					promptDetails += html_encode(parameter.name());
					promptDetails += "</span>";
					promptDetails += " : ";
					promptDetails += parameter.toString(this.m_parameterStringOperators);
					promptDetails += "</td>";
					promptDetails += (showControl ? this.addInfoBarIcon("infobar_locked.gif", false) :"");
					promptDetails += "</tr></table></td>";
					promptDetails += "<td class=\"infoBarRight\"></td>";
					promptDetails += "</tr>";
				}
			}
		}
		return promptDetails;
	},

	addSliderDetails : function(viewOnlyMode) {
		var sliderDetails = "";
		var showControl = (viewOnlyMode !== true); // showControl is true unless the viewOnlyMode is passed with value of "true"

		if (this.m_containerInfo !== null && typeof this.m_containerInfo.sliders !== "undefined") {
			var staticLabelledBy = this.m_id + "instructions " + this.m_id + "_sliderString ";
			var numberOfSliders = this.m_containerInfo.sliders.length;

			for (var index=0; index<numberOfSliders; ++index) {
				var labelledBy = staticLabelledBy + this.m_id + index + "_sliderDetail";
				var slider = this.m_containerInfo.sliders[index];
				sliderDetails += "<tr>";
				sliderDetails += "<td class=\"infoBarLeft\"></td>";
				sliderDetails += "<td class=\"infoBarDetailRow\" type=\"slider\" aria-labelledby=\"" + labelledBy + "\" role=\"listitem\"" + (!showControl?">":" tabindex=\"-1\">");
				sliderDetails += "<table><tr>";

				if (typeof slider.controlType != "undefined" && slider.controlType=="checkbox") {
					sliderDetails += this.addInfoBarIcon("infobar_select.gif", true);
				} else {
					sliderDetails += this.addInfoBarIcon("infobar_slider.gif", true);
				}
				sliderDetails += "<td class=\"infoBarDetailsText\" id=\"" + this.m_id + index + "_sliderDetail\">";
				sliderDetails += "<span class=\"bibi\">";
				sliderDetails += slider.name;
				sliderDetails += "</span>";
				sliderDetails += " : ";

				if (typeof slider.values != "undefined") {
					sliderDetails += this.processFilterOperatorItems(slider.values);
				} else {
					if (slider.min == slider.max) {
						sliderDetails += slider.min;
					} else {
						sliderDetails += this.processFilterOperatorRange(RV_RES.IDS_JS_INFOBAR_BETWEEN_TEXT, slider.min, slider.max);
					}
				}
				sliderDetails += "</td>";
				sliderDetails += (showControl ? this.addRemoveSliderButton(slider.clientId) : "");
				sliderDetails += "</tr></table></td>";
				sliderDetails += "<td class=\"infoBarRight\"></td>";
				sliderDetails += "</tr>";
			}
		}
		return sliderDetails;
	},

	hasLockedFilters : function() {
		var result = false;
		if (this.m_containerInfo !== null && typeof this.m_containerInfo.lockedFilters !== "undefined") {
			var numberOfLockedFilters = this.m_containerInfo.lockedFilters.length;
			for (var index=0; index<numberOfLockedFilters; ++index) {
				var lockedFilter = this.m_containerInfo.lockedFilters[index];
				if (typeof lockedFilter.staticText != "undefined" && lockedFilter.staticText.charAt(0) !== ' ') {
					result = true;
				}
			}
		}
		return result;
	},

	addLockedFilterDetails : function(viewOnlyMode) {
		var lockedFilterDetails = "";
		var showControl = (viewOnlyMode !== true); // showControl is true unless the viewOnlyMode is passed with value of "true"

		if (this.m_containerInfo !== null && typeof this.m_containerInfo.lockedFilters !== "undefined") {
			var staticLabelledBy = this.m_id + "instructions " + this.m_id + "_filterString " + this.m_id + "_lockedString ";
			var numberOfLockedFilters = this.m_containerInfo.lockedFilters.length;

			for (var index=0; index<numberOfLockedFilters; ++index) {
				var labelledBy = staticLabelledBy + this.m_id + index + "_lockedFilterDetail";
				var lockedFilter = this.m_containerInfo.lockedFilters[index];

				if (typeof lockedFilter.staticText != "undefined" && lockedFilter.staticText.charAt(0) !== ' ') {
					lockedFilterDetails += "<tr>";
					lockedFilterDetails += "<td class=\"infoBarLeft\"></td>";
					lockedFilterDetails += "<td class=\"infoBarDetailRow\" aria-labelledby=\"" + labelledBy + "\" role=\"listitem\"" + (!showControl?">":" tabindex=\"-1\">");
					lockedFilterDetails += "<table><tr>";
					lockedFilterDetails += this.addInfoBarIcon("infobar_filter.gif", true);
					lockedFilterDetails += "</td>";
					lockedFilterDetails += "<td class=\"infoBarDetailsText\" id=\"" + this.m_id + index + "_lockedFilterDetail\">";
					lockedFilterDetails += "<span class=\"bibi\">";
					lockedFilterDetails += html_encode(lockedFilter.staticText);
					lockedFilterDetails += "</span>";
					lockedFilterDetails += "</td>";
					lockedFilterDetails += (showControl ? this.addInfoBarIcon("infobar_locked.gif", false) : "");
					lockedFilterDetails += "</tr></table></td>";
					lockedFilterDetails += "<td class=\"infoBarRight\"></td>";
					lockedFilterDetails += "</tr>";
				}
			}
		}
		return lockedFilterDetails;
	},

	addFilteredItemsDetails : function(viewOnlyMode) {
		var filteredItemsDetails = "";
		var showControl = (viewOnlyMode !== true); // showControl is true unless the viewOnlyMode is passed with value of "true"

		if (this.m_editableFilters !== null) {
			for (var index = 0, filterLength = this.m_editableFilters.length; index < filterLength; ++index) {
				var filterItem = this.m_editableFilters[index];
				var labelledBy = this.m_id + "instructions " + this.m_id + "_filterString " + this.m_id + index + "_filterDetail";
				var labelText = (typeof filterItem.itemLabel == "string") ? filterItem.itemLabel : filterItem.item;
				filteredItemsDetails += "<tr>";
				filteredItemsDetails += "<td class=\"infoBarLeft\"></td>";
				filteredItemsDetails += "<td class=\"infoBarDetailRow\" type=\"filter\" aria-labelledby=\"" + labelledBy + "\" role=\"listitem\"" + (!showControl?">":" tabindex=\"-1\">");
				filteredItemsDetails += "<table><tr>";
				filteredItemsDetails += this.addInfoBarIcon("infobar_filter.gif", true);
				filteredItemsDetails += "<td class=\"bibi infoBarDetailsText\" id=\"" + this.m_id + index + "_filterDetail\">";
				filteredItemsDetails += this.createHiddenSpan(this.m_id + index + "_filterPosition", getCurrentPosistionString(this.m_cognosViewer, index + 1, filterLength));
				filteredItemsDetails += html_encode(labelText);
				filteredItemsDetails += this.processFilterOperator(filterItem);
				filteredItemsDetails += "</td>";
				filteredItemsDetails += (showControl ? this.addRemoveFilterButton(filterItem.item, this.processFilterDetails(filterItem)) : "");
				filteredItemsDetails += "</tr></table></td>";
				filteredItemsDetails += "<td class=\"infoBarRight\"></td>";
				filteredItemsDetails += "</tr>";
			}
		}
		return filteredItemsDetails;
	},

	addSortDetails : function(viewOnlyMode) {
		var sortDetails = "";
		var showControl = (viewOnlyMode !== true); // showControl is true unless the viewOnlyMode is passed with value of "true"

		if (this.m_editableSorts !== null) {
			for (var index = 0, sortLength = this.m_editableSorts.length; index < sortLength; ++index) {
				var sortInfo = this.m_editableSorts[index];
				var labelledBy = this.m_id + "instructions " + this.m_id + "_sortString " + this.m_id + index + "_sortDetail";
				sortDetails += "<tr>";
				sortDetails += "<td class=\"infoBarLeft\"></td>";
				sortDetails += "<td class=\"infoBarDetailRow\" type=\"sort\" aria-labelledby=\"" + labelledBy + "\" role=\"listitem\"" + (!showControl?">":" tabindex=\"-1\">");
				sortDetails += "<table><tr>";
				sortDetails += this.addInfoBarIcon("infobar_sort.gif", true);

				var itemLabel;
				if (typeof sortInfo.labels == "string" || typeof sortInfo.valuesOf == "string") {
					var itemName = (typeof sortInfo.labels == "string") ? sortInfo.labels : sortInfo.valuesOf;
					itemLabel = (typeof sortInfo.itemLabel == "string") ? sortInfo.itemLabel : itemName;
					sortDetails += "<td class=\"bibi infoBarDetailsText\" id=\"" + this.m_id + index + "_sortDetail\">";
					sortDetails += this.createHiddenSpan(this.m_id + index + "_sortPosition", getCurrentPosistionString(this.m_cognosViewer, index+1, sortLength));
					sortDetails += html_encode(itemLabel);
					sortDetails += " : ";

					if (sortInfo.byItems instanceof Array) {
						//TODO: Leave out string "Sorted by" until rp1 (see infoBarAdornment)
						//sortDetails += this.m_sortStrings["byItems"];
						for (var byItemsIdx = 0; byItemsIdx < sortInfo.byItems.length; ++byItemsIdx) {
							sortDetails += (byItemsIdx>0) ? " - " : "";
							sortDetails += sortInfo.byItems[byItemsIdx].item;
							sortDetails += ", ";
							sortDetails += this.m_sortStrings[sortInfo.byItems[byItemsIdx].order];
						}
					} else {
						if (typeof sortInfo.valuesOf == "string") {
							sortDetails += this.m_sortStrings["byValue"];
						} else {
							sortDetails += this.m_sortStrings["byLabel"];
						}
						sortDetails += ", ";
						sortDetails += this.m_sortStrings[sortInfo.order];
					}
					sortDetails += "</td>";
					sortDetails += (showControl ? this.addRemoveSortButton(itemName, "true") : "");
				} else if (sortInfo.valuesOf instanceof Array) {
					if (sortInfo.valuesOf.length > 0 && typeof sortInfo.valuesOf[0].item == "string") {
						sortDetails += "<td class=\"bibi infoBarDetailsText\" id=\"" + this.m_id + index + "_sortDetail\">";
						sortDetails += this.createHiddenSpan(this.m_id + index + "_sortPosition", getCurrentPosistionString(this.m_cognosViewer, index+1, sortLength));

						itemLabel = "";
						if (typeof sortInfo.itemLabel == "string") {
							itemLabel = sortInfo.itemLabel;
						} else {
							for (var valueSortIdx = 0; valueSortIdx < sortInfo.valuesOf.length; ++valueSortIdx) {
								itemLabel += " - ";
								itemLabel += sortInfo.valuesOf[valueSortIdx].item;
							}
							itemLabel = itemLabel.substring(3);
						}

						sortDetails += itemLabel;
						sortDetails +=  " : ";
						sortDetails += this.m_sortStrings["byValue"];
						sortDetails += ", ";
						sortDetails += this.m_sortStrings[sortInfo.order];
						sortDetails += "</td>";
						sortDetails += (showControl ? this.addRemoveSortButton(sortInfo.valuesOf[0].item, "false") : "");
					}
				}
				sortDetails += "</tr></table></td>";
				sortDetails += "<td class=\"infoBarRight\"></td>";
				sortDetails += "</tr>";
			}
		}
		return sortDetails;
	},

	addInfoBarIcon : function(imageName, isLeft) {
		return "<td class=\"infoBar" + (isLeft?"Left":"Right") + "Icon\">" +
					"<img alt=\"\" src=\"" + this.m_cognosViewer.getWebContentRoot() + "/rv/images/" + imageName + "\"/>" +
				"</td>";
	},

	createHiddenSpan : function(id, text) {
		return "<span style=\"display:none\" id=\"" + id + "\"> " + text + " </span>";
	},

	addRemoveFilterButton : function(item, txtDetails) {
		var buttonTitle = RV_RES.IDS_JS_DELETE;
		return	"<td class=\"infoBarRightIcon\" filterItem=\"" + item + "\"" + (txtDetails ? " details=\""+ txtDetails +  "\"" :"") + ">" +
					"<button class=\"dijitStretch infoBarDeleteButton\" onclick=\"InfoBarHelper.infoBarRemoveFilter('" + this.m_cognosViewer.getId() + "'," + this.m_layoutIndex + ",'" + item  + "'" + (txtDetails ? ",'" + txtDetails + "'" : "") + ");\" aria-labelledby=\"" + buttonTitle + "\" type=\"button\" role=\"button\" tabindex=\"0\" value=\"\" title=\"" + buttonTitle + "\">" +
						"<span class=\"dijitInline infoBarIconDelete\" title=\"" + buttonTitle + "\"></span>" +
						"<span class=\"dijitInline dijitDisplayNone infoBarDeleteButtonText\">x</span>" +
					"</button>" +
				"</td>";
	},

	addRemoveSliderButton : function(sliderId) {
		var buttonTitle = RV_RES.IDS_JS_DELETE;
		return	"<td class=\"infoBarRightIcon\" slider=\"" + sliderId + "\">" +
					"<button class=\"dijitStretch infoBarDeleteButton\" onclick=\"InfoBarHelper.infoBarRemoveSlider('" + this.m_cognosViewer.getId() + "','" + sliderId + "');\" aria-labelledby=\"" + buttonTitle + "\" type=\"button\" role=\"button\" tabindex=\"0\" value=\"\" title=\"" + buttonTitle + "\">" +
						"<span class=\"dijitInline infoBarIconDelete\" title=\"" + buttonTitle + "\"></span>" +
						"<span class=\"dijitInline dijitDisplayNone infoBarDeleteButtonText\">x</span>" +
					"</button>" +
				"</td>";
	},

	addRemoveSortButton : function(item, byLabel) {
		var buttonTitle = RV_RES.IDS_JS_DELETE;
		return	"<td class=\"infoBarRightIcon\" sortItem=\"" + item + "\" byLabel=\"" + byLabel + "\">" +
					"<button class=\"dijitStretch infoBarDeleteButton\" onclick=\"InfoBarHelper.infoBarRemoveSort('" + this.m_cognosViewer.getId() + "'," + this.m_layoutIndex + ",'" + item + "','" + byLabel + "');\" aria-labelledby=\"" + buttonTitle + "\" type=\"button\" role=\"button\" tabindex=\"0\" value=\"\" title=\"" + buttonTitle + "\">" +
						"<span class=\"dijitInline infoBarIconDelete\" title=\"" + buttonTitle + "\"></span>" +
						"<span class=\"dijitInline dijitDisplayNone infoBarDeleteButtonText\">x</span>" +
					"</button>" +
				"</td>";
	},

	processFilterOperator : function(filterItem) {
		var filterOperatorText = "";

		if (typeof filterItem["in"] != "undefined") {
			filterOperatorText += " " + CViewerCommon.getMessage(RV_RES.IDS_JS_INFOBAR_INCLUDE_TEXT, this.processFilterOperatorItems(filterItem["in"]));
		} else if (typeof filterItem["notIn"] != "undefined") {
			filterOperatorText += " " + CViewerCommon.getMessage(RV_RES.IDS_JS_INFOBAR_EXCLUDE_TEXT, this.processFilterOperatorItems(filterItem["notIn"]));
		} else if (typeof filterItem["range"] != "undefined" || typeof filterItem["notRange"] != "undefined") {
			var operatorText = typeof filterItem["range"] != "undefined" ? RV_RES.IDS_JS_INFOBAR_BETWEEN_TEXT : RV_RES.IDS_JS_INFOBAR_NOT_BETWEEN_TEXT;
			var rangeArray = typeof filterItem["range"] != "undefined" ? filterItem["range"] : filterItem["notRange"];
			for (var index = 0; index < rangeArray.length; ++index) {
				var range = rangeArray[index];
				filterOperatorText += " " + CViewerCommon.getMessage(operatorText, [range.from, range.to]);
			}
		} else if (typeof filterItem["lessThan"] != "undefined") {
			filterOperatorText += " " + RV_RES.IDS_JS_FILTER_LESS_THAN + " " + filterItem["lessThan"];

		} else if (typeof filterItem["lessThanEqual"] != "undefined") {
			filterOperatorText += " " + RV_RES.IDS_JS_FILTER_LESS_THAN_EQUAL + " " + filterItem["lessThanEqual"];

		} else if (typeof filterItem["greaterThan"] != "undefined") {
			filterOperatorText += " " + RV_RES.IDS_JS_FILTER_GREATER_THAN + " " + filterItem["greaterThan"];

		} else if (typeof filterItem["greaterThanEqual"] != "undefined") {
			filterOperatorText += " " + RV_RES.IDS_JS_FILTER_GREATER_THAN_EQUAL + " " + filterItem["greaterThanEqual"];
		}
		return filterOperatorText;
	},

	processFilterDetails : function(filterItem) {
		return html_encode('[' + dojo.toJson(filterItem) + ']');
	},

	processFilterOperatorItems : function(filterOperatorItems) {
		var filterOperatorItemsText = "";
		for (var index = 0; index < filterOperatorItems.length; ++index) {
			if (index !== 0) {
				filterOperatorItemsText += (RV_RES.IDS_JS_LIST_SEPARATOR + " ");
			}
			filterOperatorItemsText += filterOperatorItems[index];
		}
		return filterOperatorItemsText;
	},

	processFilterOperatorRange : function(rangeTypeText, minRange, maxRange) {
		if (typeof minRange == "undefined") {
			minRange =' ';
		}
		if (typeof maxRange == "undefined") {
			maxRange = ' ';
		}
		return  CViewerCommon.getMessage(rangeTypeText, [minRange, maxRange]);
	}
});

// Info Bar
dojo.declare("InfoBar", InfoBarBase, {
	constructor : function(cognosViewer, layoutElement, containerInfo, childContainers, layoutIndex) {
		this.m_layoutElement = layoutElement;
		this.m_containerInfo = containerInfo;
		this.initializeEditableSorts(childContainers);
		this.initializeEditableFilters(childContainers);

		this.m_id = layoutIndex + cognosViewer.getId();
		this.m_layoutIndex = layoutIndex;
		this.m_sortStrings = {byLabel: RV_RES.IDS_JS_SORTED_BY_LABEL,
							  byValue: RV_RES.IDS_JS_SORTED_BY_VALUE,
							  descending: RV_RES.IDS_JS_SORT_DESCENDING_ORDER,
							  ascending: RV_RES.IDS_JS_SORT_ASCENDING_ORDER};
	},

	getLayoutElement : function() {
		var layoutElement = this.m_layoutElement;
		var layoutId = layoutElement.getAttribute("lid");
		if (layoutId !== null) {
			if (layoutElement.tagName.toUpperCase() == "MAP") {
				var allSiblings = layoutElement.parentNode.childNodes;
				for (var index = 0; index < allSiblings.length; ++index) {
					var useMapValue = allSiblings[index].getAttribute("usemap");
					if (useMapValue !== null && useMapValue == ("#" + this.m_cognosViewer.getId() + layoutId)) {
						return allSiblings[index];
					}
				}
			}
			else if (layoutElement.parentNode.getAttribute("chartcontainer") == "true") {
				return layoutElement.parentNode;
			}
			return layoutElement;
		}
		return null;
	},

	initializeEditableSorts : function(childContainers) {
		if (this.m_containerInfo !== null) {
			if (typeof this.m_containerInfo.sort !== "undefined") {
				this.m_editableSorts = this.m_containerInfo.sort;
			}
			if (typeof childContainers !== "undefined") {
				for (var i=0; i<childContainers.length; ++i) {
					this.m_editableSorts = this.mergeChildInfo(this.m_editableSorts, childContainers[i].sort, 'labels');
					this.m_editableSorts = this.mergeChildInfo(this.m_editableSorts, childContainers[i].sort, 'valuesOf');
				}
			}
		}
	},

	initializeEditableFilters : function(childContainers) {
		if (typeof this.m_containerInfo !== "undefined" && this.m_containerInfo !== null) {
			if (typeof this.m_containerInfo.filter !== "undefined") {
				this.m_editableFilters = this.m_containerInfo.filter;
			}
			if (typeof childContainers!== "undefined") {
				for (var i=0; i<childContainers.length; ++i) {
					this.m_editableFilters = this.mergeChildInfo(this.m_editableFilters, childContainers[i].filter, 'item');
				}
			}
		}
	},

	mergeChildInfo : function(parentList, childList, key) {
		if (typeof childList != "undefined") {
			if (typeof parentList === "undefined" || parentList === null) {
				parentList = childList;
			} else {
				//remove duplicates....(eg shared filters)
				var duplicateChecker = [];
				for (var i = 0; i<parentList.length; ++i) {
					duplicateChecker[parentList[i][key]] = parentList[i];
				}

				for (var j = 0; j<childList.length; ++j) {
					var childEntry = childList[j];
					if (typeof duplicateChecker[childEntry[key]] == "undefined") {
						parentList.push(childEntry);
						duplicateChecker[childEntry.item] = childEntry;
					}
				}
			}
		}
		return parentList;
	},

	render : function() {
		var divElement = document.getElementById("infoBar" + this.m_id);
		if (divElement === null && (this.m_editableSorts !== null || this.m_editableFilters !== null || this.hasPrompts() || typeof this.m_containerInfo.sliders !== "undefined" || this.hasLockedFilters() )) {
			var layoutElement = this.getLayoutElement();

			divElement = document.createElement("div");
			divElement.className = "infoBar";
			divElement.setAttribute("id", "infoBar" + this.m_id);
			divElement.setAttribute("cvid", this.m_cognosViewer.getId());
			divElement.setAttribute("layoutid", this.m_id);
			divElement.setAttribute("containerid", this.m_layoutIndex);

			divElement.appendChild(this.renderHeader());
			divElement.appendChild(this.renderDetails());
				
			layoutElement.parentNode.insertBefore(divElement, layoutElement);

			dojo.connect(divElement, 'onkeydown', this, this.onKeyDown);
			dojo.query('.infoBarHeader', divElement).connect('onclick', this, this.toggle);
		}
	},

	renderHeader : function() {
		var isFiltered = this.isFilterApplied();
		var isSorted = this.isSortApplied();

		var headerDiv = document.createElement("div");
		headerDiv.className="infoBarHeaderDiv";
		var header =
			"<table class=\"infoBarHeader\" id=\"infoBarHeader" + this.m_id + "\">" +
				"<tr>" +
					"<td class=\"infoBarHeaderLeft\"></td>" +
					this.addHeaderNavigationButton(isFiltered, isSorted) +
					this.addHeaderText(isFiltered, isSorted) +
					this.addHeaderCloseButton() +
					"<td class=\"infoBarHeaderRight\"></td>" +
				"</tr>" +
			"</table>";
		headerDiv.innerHTML = header;
		return headerDiv;
	},

	addHeaderNavigationButton : function(isFiltered, isSorted) {
		var buttonTitle = RV_RES.IDS_JS_INFOBAR_EXPAND;
		var buttonClass = "infoBarIcon" + (isFiltered?"Filter":"") + (isSorted?"Sort":"") + "Applied";
		var labelledBy = "infoBarHeaderButton" + this.m_id + " infoBarHeaderText" + this.m_id;
		return "<td class=\"infoBarHeaderNav\">" +
					"<button class=\"dijitStretch infoBarHeaderButton\" id=\"infoBarHeaderButton" + this.m_id + "\" aria-labelledby=\"" + labelledBy + "\" type=\"button\" role=\"button\" tabindex=\"0\" value=\"\" title=\"" + buttonTitle + "\">" +
						"<span class=\"dijitInline " + buttonClass + " \" title=\"" + buttonTitle + "\"></span>" +
						"<span class=\"dijitInline dijitDisplayNone infoBarHeaderButtonText\">!</span>" +
					"</button>" +
			   "</td>";
	},

	addHeaderText : function(isFiltered, isSorted) {
		var headerText = [];
		if (isFiltered) {headerText.push(RV_RES.IDS_JS_FILTER_APPLIED);}
		if (isSorted) {headerText.push(RV_RES.IDS_JS_SORT_APPLIED);}
		return "<td class=\"infoBarHeaderText\" id=\"infoBarHeaderText" + this.m_id + "\" style=\"display:none;\">" +
					headerText.join(', ') +
				"</td>";
	},

	addHeaderCloseButton : function() {
		var buttonTitle = RV_RES.IDS_JS_INFOBAR_COLLAPSE;
		var labelledBy = RV_RES.IDS_JS_INFOBAR_COLLAPSE;
		return "<td class=\"infoBarHeaderClose\" id=\"infoBarHeaderClose" + this.m_id + "\" style=\"display:none;\">" +
					"<button class=\"dijitStretch infoBarHeaderCloseButton\" id=\"infoBarHeaderCloseButton" + this.m_id + "\" aria-labelledby=\"" + labelledBy + "\" type=\"button\" role=\"button\" tabindex=\"0\" value=\"\" title=\"" + buttonTitle + "\">" +
						"<span class=\"dijitInline infoBarIconClose\" title=\"" + buttonTitle + "\"></span>" +
						"<span class=\"dijitInline dijitDisplayNone infoBarCloseButtonText\">x</span>" +					"</button>" +
			   "</td>";
	},

	isFilterApplied : function() {
		if (this.m_editableFilters !== null || typeof this.m_containerInfo.sliders !== "undefined" || typeof this.m_containerInfo.lockedFilters !== "undefined" || this.hasPrompts()) {
			return true;
		}
		return false;
	},

	isSortApplied : function() {
		if (this.m_editableSorts !== null) {
			return true;
		}
		return false;
	},

	renderDetails : function() {
		var divElement = document.getElementById("infoBarFlyout" + this.m_id);
		var instructions = document.getElementById(this.m_id + "instructions");

		if (divElement === null) {
			this.getLayoutElement();
			divElement = document.createElement("div");
			divElement.setAttribute("id", "infoBarFlyout" + this.m_id);
			divElement.setAttribute("cvid", this.m_cognosViewer.getId());
			divElement.setAttribute("layoutid", this.m_id);
			divElement.setAttribute("containerid", this.m_layoutIndex);
			divElement.className = "infoBarFlyout";
			divElement.style.display = "none";

			divElement.innerHTML =  "<table class=\"infoBarFlyoutContent\" id=\"infoBarFlyoutContent" + this.m_id + "\" role=\"list\" width=\"100%\">" +
										this.addLockedInfoBarDetails() +
										this.addEditableInfoBarDetails() +
									"</table>" +
									this.createHiddenSpan(this.m_id + "_filterString", RV_RES.IDS_JS_FILTER) +
									this.createHiddenSpan(this.m_id + "_sortString", RV_RES.IDS_JS_SORT) +
									this.createHiddenSpan(this.m_id + "_promptString", RV_RES.IDS_JS_INFOBAR_PROMPT) +
									this.createHiddenSpan(this.m_id + "_sliderString", RV_RES.IDS_JS_INFOBAR_SLIDER) +
									this.createHiddenSpan(this.m_id + "_lockedString", RV_RES.IDS_JS_INFOBAR_LOCKED);

			dojo.connect(divElement, 'onkeydown', this, this.onKeyDown);

			dojo.query('.infoBarDetailRow', divElement)
				.forEach(function (item, index, array) {
					dojo.attr(item, {
						"aria-setsize" : array.length,
						"aria-posinset" : index+1
					});
				})
				.connect('onfocus', function(e) {
					dojo.query(e.currentTarget).addClass('infoBarDetailRowHover').attr("aria-selected", "true");
				})
				.connect('onblur', function(e) {
					var el = typeof e.explicitOriginalTarget !== "undefined" ? e.explicitOriginalTarget : document.activeElement;
					if (dojo.hasClass(el, 'infoBarDeleteButton')) {
						return true;
					}
					dojo.query(e.currentTarget).removeClass('infoBarDetailRowHover').attr("aria-selected", "false");
				})
				.connect('onmouseover', this, function(e) {
					this.focusDetailRow(e.currentTarget);
				});
		}

		// used for JAWS to let the user know how to navigate the info bar
		if (instructions === null) {
			instructions = document.createElement("span");
			instructions.id = this.m_id + "instructions";
			instructions.setAttribute("role", "presentation");
			instructions.style.visibility = "hidden";
			instructions.style.display = "none";
			document.body.appendChild(instructions);
			instructions.innerHTML = RV_RES.IDS_JS_INFOBAR_EXPLANATION;
		}
		return divElement;
	},

	addLockedInfoBarDetails : function() {
		if (this.hasPrompts() || typeof this.m_containerInfo.lockedFilters !== "undefined") {
			return "<tr>" +
						"<td colspan=\"3\">" +
							"<table class=\"infoBarLockedItems\">" +
								this.addPromptDetails() +
								this.addLockedFilterDetails() +
							"</table>" +
						"</td>" +
					"</tr>";
		}
		return "";
	},

	addEditableInfoBarDetails : function() {
		if (this.m_editableFilters !== null || this.m_editableSorts !== null || typeof this.m_containerInfo.sliders !== "undefined") {
			var html = "";
			if (this.hasPrompts()|| typeof this.m_containerInfo.lockedFilters !== "undefined") {
				html += "<tr>" +
							"<td class=\"infoBarEditableTopLeft\"></td>" +
							"<td class=\"infoBarEditableTopMid\"></td>" +
							"<td class=\"infoBarEditableTopRight\"></td>" +
						"</tr>";
			}
			html += "<tr>" +
						"<td colspan=\"3\">" +
							"<table class=\"infoBarEditableItems\">" +
								this.addFilteredItemsDetails() +
								this.addSliderDetails() +
								this.addSortDetails() +
							"</table>" +
						"</td>" +
					"</tr>" +
					"<tr>" +
						"<td class=\"infoBarEditableBottomLeft\"></td>" +
						"<td class=\"infoBarEditableBottomMid\"></td>" +
						"<td class=\"infoBarEditableBottomRight\"></td>" +
					"</tr>";
			return html;
		}
		return "<tr>" +
					"<td class=\"infoBarLockedBottomLeft\"></td>" +
					"<td class=\"infoBarLockedBottomMid\"></td>" +
					"<td class=\"infoBarLockedBottomRight\"></td>" +
				"</tr>";
	},

	onKeyDown : function(evt) {
		var srcNode = evt.target;
		var cvId = this.m_cognosViewer.getId();

		var infoBarHeaderButton = document.getElementById("infoBarHeaderButton" + this.m_id);
		var instructions = document.getElementById(this.m_id + "instructions");
		var node;

		if (evt.keyCode == "13") { // enter
			instructions.innerHTML = RV_RES.IDS_JS_INFOBAR_EXPLANATION;
			if (srcNode === infoBarHeaderButton) {
				this.toggle();
				return stopEventBubble(evt);
			}

		} else if (evt.keyCode == "27")	{ // esc
			if (this.m_cognosViewer && this.m_cognosViewer.m_currentNodeFocus) {
				node = this.m_cognosViewer.m_currentNodeFocus;
				if (node.nodeName.toLowerCase() != "span") {
					var spans = node.getElementsByTagName("span");
					for (var index=0; index < spans.length; index++) {
						if (spans[index].style.display != "hidden") {
							node = spans[index];
							break;
						}
					}
				}
				this.hide(false);
				this.m_cognosViewer.setFocusToNode(node);
				this.m_cognosViewer.m_currentNodeFocus.focus();
			} else {
				this.hide();
			}
			return stopEventBubble(evt);

		} else if (evt.keyCode == "46") { // del
			if (dojo.hasClass(srcNode, "infoBarDetailRow")) {
				node = dojo.query("td.infoBarRightIcon", srcNode)[0];
				if (typeof node !== "undefined") {
					if (srcNode.getAttribute("type") == "filter") {
						InfoBarHelper.infoBarRemoveFilter(cvId, this.m_layoutIndex, node.getAttribute("filterItem"), node.getAttribute("details"));
					} else if (srcNode.getAttribute("type") == "sort") {
						InfoBarHelper.infoBarRemoveSort(cvId, this.m_layoutIndex, node.getAttribute("sortItem"), node.getAttribute("byLabel"));
					} else if (srcNode.getAttribute("type") == "slider") {
						InfoBarHelper.infoBarRemoveSlider(cvId, node.getAttribute("slider"));
					}
				}
			}

		} else if (evt.keyCode == "38") { // up arrow
			if (dojo.hasClass(srcNode, "infoBarDetailRow")) {
				this.focusDetailRow(srcNode, "prev");
			} else {
				this.focusDetailRow();
			}
			instructions.innerHTML = "";
			return stopEventBubble(evt);

		} else if (evt.keyCode == "40") { // down arrow
			if (dojo.hasClass(srcNode, "infoBarDetailRow")) {
				this.focusDetailRow(srcNode, "next");
			} else {
				this.focusDetailRow();
			}
			instructions.innerHTML = "";
			return stopEventBubble(evt);
		}
	},

	focusDetailRow : function(detailRow, dir) {
		var infoBarFlyout = document.getElementById("infoBarFlyout" + this.m_id);
		var infoBar = document.getElementById("infoBar" + this.m_id);

		if (!dojo.hasClass(infoBar, "infoBarExpanded")) {
			return;
		}

		if (typeof detailRow == "undefined" || detailRow === null) {
			var firstDetailRows = dojo.query(".infoBarLockedItems tr:first-child td.infoBarDetailRow, .infoBarEditableItems tr:first-child td.infoBarDetailRow", infoBarFlyout);
			detailRow = firstDetailRows[0];
		}

		if (detailRow !== null && dojo.hasClass(detailRow, "infoBarDetailRow")) {
			var newRow, rows, len, i;
			if (dir === "prev") {
				if (detailRow.parentNode.previousSibling) { // faster
					newRow = detailRow.parentNode.previousSibling.childNodes[1];
				} else {
					rows = dojo.query(".infoBarDetailRow", infoBarFlyout);
					len = rows.length;
					if (detailRow === rows[0]) {
						newRow = rows[len-1];
					} else {
						for (i = 1; i < len; i++) {
							if (detailRow === rows[i]) {
								newRow = rows[i-1];
								break;
							}
						}
					}
				}
			} else if (dir === "next") {
				if (detailRow.parentNode.nextSibling) { // faster
					newRow = detailRow.parentNode.nextSibling.childNodes[1];
				} else {
					rows = dojo.query(".infoBarDetailRow", infoBarFlyout);
					len = rows.length;
					if (detailRow === rows[len-1]) {
						newRow = rows[0];
					} else {
						for (i = 0; i < len-1; i++) {
							if (detailRow === rows[i]) {
								newRow = rows[i+1];
								break;
							}
						}
					}
				}
			} else {
				newRow = detailRow;
			}
			newRow.focus();
		}
	},

	toggle : function() {
		var infoBar = document.getElementById("infoBar" + this.m_id);
		if (dojo.hasClass(infoBar, "infoBarExpanded")) {
			this.hide();
		} else {
			this.show();
		}
	},

	show : function() {
		var infoBar = document.getElementById("infoBar" + this.m_id);

		if (!dojo.hasClass(infoBar, "infoBarExpanded")) {
			var infoBarHeader = document.getElementById("infoBarHeader" + this.m_id);
			var infoBarFlyout = document.getElementById("infoBarFlyout" + this.m_id);
			var infoBarHeaderButton = document.getElementById("infoBarHeaderButton" + this.m_id);
			var infoBarHeaderText = document.getElementById("infoBarHeaderText" + this.m_id);
			var infoBarHeaderClose = document.getElementById("infoBarHeaderClose" + this.m_id);
			var flyoutCoords, canvasCoords, newWidth;
			var wipeIn = dojo.fx.wipeIn({
				node: infoBarFlyout,
				duration: 300,
				onEnd: dojo.hitch(this, function() {
					this.focusDetailRow();
					infoBarHeaderButton.setAttribute('title', RV_RES.IDS_JS_INFOBAR_COLLAPSE);
					infoBarHeaderButton.childNodes[0].setAttribute('title', RV_RES.IDS_JS_INFOBAR_COLLAPSE);
				})
			});

			dojo.addClass(infoBar, "infoBarExpanded");
			dojo.style(infoBarHeaderText, {display: ""});
			dojo.style(infoBarHeaderClose, {display: ""});

			dojo.style(infoBarFlyout.childNodes[0], "width", 'auto');
			wipeIn.play();

			flyoutCoords = dojo.coords(infoBarFlyout.childNodes[0]);
			canvasCoords = dojo.coords(infoBar.parentNode);

			// IE lets the flyout expand the entire width of the viewer, so the flyoutCoords are >= canvasCoords
			newWidth = Math.min(flyoutCoords.w, Math.min(canvasCoords.w - 6, 500));

			// the -2 is to account for the -2px left margin on the infoBarHeader's parent element
			dojo.style(infoBarHeader, "width", newWidth - 2 + "px");
			// we need to properly set the width of the flyout so that the animation looks proper..
			dojo.style(infoBarFlyout.childNodes[0], "width", newWidth + "px");
		}
	},

	hide : function(focus) {
		var infoBar = document.getElementById("infoBar" + this.m_id);

		if (dojo.hasClass(infoBar, "infoBarExpanded")) {
			var infoBarHeader = document.getElementById("infoBarHeader" + this.m_id);
			var infoBarFlyout = document.getElementById("infoBarFlyout" + this.m_id);
			var infoBarHeaderButton = document.getElementById("infoBarHeaderButton" + this.m_id);
			var infoBarHeaderText = document.getElementById("infoBarHeaderText" + this.m_id);
			var infoBarHeaderClose = document.getElementById("infoBarHeaderClose" + this.m_id);
			dojo.fx.wipeOut({
				node: infoBarFlyout,
				duration: 300,
				onEnd: dojo.hitch(this, function() {
					dojo.removeClass(infoBar, "infoBarExpanded");
					dojo.style(infoBarHeader, "width", 'auto');
					dojo.style(infoBarHeaderText, {display: "none"});
					dojo.style(infoBarHeaderClose, {display: "none"});

					infoBarHeaderButton.setAttribute('title', RV_RES.IDS_JS_INFOBAR_EXPAND);
					infoBarHeaderButton.childNodes[0].setAttribute('title', RV_RES.IDS_JS_INFOBAR_EXPAND);

					if (!focus) {
						infoBarHeaderButton.focus();
					}
				})
			}).play();
		}
	}
});

// ///////////////////////////////////////////////////////////
// AnnotationFilterContext class
dojo.declare("AnnotationFilterContext", null, {
	m_parameterValuesString: null,
	m_parameterValuesSpec: null,
	m_slidersJSON: null,
	m_filterJSON: null,

	constructor : function(paramValuesString, paramValuesSpec, slidersJSON, filterJSON){
		this.m_parameterValuesString = paramValuesString;
		this.m_parameterValuesSpec = paramValuesSpec;
		this.m_slidersJSON = slidersJSON;
		this.m_filterJSON = filterJSON;
	},

	setParameterValuesString: function(param) {
		this.m_parameterValuesString = param;
	},
	setParameterValuesSpec: function(param){
		this.m_parameterValuesSpec = param;
	},
	setSlidersJSON: function(param) {
		this.m_slidersJSON = param;
	},
	setFilterJSON: function(param){
		this.m_filterJSON = param;
	},
	getParameterValuesString: function(){
		return this.m_parameterValuesString || null; // if null or undefined, return null;
	},
	getParameterValuesSpec: function(){
		return this.m_parameterValuesSpec || null; // if null or undefined, return null;
	},
	getPromptsJSON: function(){
		var promptObject = null;
		if (this.m_parameterValuesString) {
			promptObject = {};
			promptObject.parameterValuesString = this.m_parameterValuesString;
		}
		if (this.m_parameterValuesString) {
			promptObject = promptObject || {};
			promptObject.parameterValuesSpec = this.m_parameterValuesSpec;
		}

		return promptObject;
	},
	getSlidersJSON: function(){
		return this.m_slidersJSON || null; // if null or undefined, return null;
	},
	getFilterJSON: function(){
		return this.m_filterJSON || null; // if null or undefined, return null;
	},

	isEmpty: function(){
		if (this.m_parameterValuesString || this.m_parameterValuesSpec || this.m_slidersJSON || this.m_filterJSON ) {
			return false;
		}
		return true;
	}
});

// ///////////////////////////////////////////////////////////
// AnnotationInfoBar class
dojo.declare("AnnotationInfoBar", InfoBarBase, {
	constructor : function(cognosViewer, lid) {
		this.m_id = "annoInfoBar_" + lid;
		this.m_layoutElementId = lid;
	},

	/*
	 * returns AnnotationFilterContext object if parameterValuesString, filter, or sliders exists
	 */
	createFilterContext : function() {
		var infoBarString = this.m_cognosViewer.envParams["json.infoBar"];

		var filter = null;
		var slider = null;
		var paramSpec = null;
		var paramString = null;

		if (typeof infoBarString !== "undefined" && infoBarString.length > 0) {
			//the infoBarObject includes global report properties and a container array.
			var infoBarObject = eval("(" + infoBarString + ')');
			if (typeof infoBarObject.containers != "undefined") {
				var infoBarArray = infoBarObject.containers;
				for(var containerIdx = 0; infoBarArray.length > containerIdx ; ++containerIdx) {
					var container = infoBarArray[containerIdx].container;
					if (this.m_layoutElementId === container) {
						var infoBar = infoBarArray[containerIdx];
						if( typeof infoBar.filter !== "undefined") {
							filter = infoBar.filter;
						}
						if( typeof infoBar.sliders !== "undefined") {
							slider = infoBar.sliders;
						}
						break;
					}
				}
			}
		}

		if (this.hasPrompts()) {
			var tmpExportAction = new ExportAction();
			tmpExportAction.setCognosViewer(this.m_cognosViewer);
			paramSpec = tmpExportAction.buildParameterValuesSpec();
			paramString = this.m_cognosViewer.getExecutionParameters();
		}


		if (filter || slider || paramSpec || paramString) {
			return new AnnotationFilterContext(paramString, paramSpec, slider, filter);
		}

		return null;
	}
});

// ///////////////////////////////////////////////////////////
// AnnotationInfoBar class
dojo.declare("AnnotationInfoBarRenderer", null, {
	m_AnnotationInfoBar: null,
	m_id: null,
	m_labelledByHideButton: "",
	miniCognosViewer: null,


	constructor : function(cognosViewer, lid, filterContext, annoId) {

		this.m_AnnotationInfoBar = null;
		this.miniCognosViewer = {};

		this.miniCognosViewer.m_sId = cognosViewer.id;
		this.miniCognosViewer.getId = cognosViewer.getId;
		this.miniCognosViewer.m_sParameters = (filterContext === null || filterContext.getParameterValuesString() === null)? "" :filterContext.getParameterValuesString();
		this.miniCognosViewer.getExecutionParameters = cognosViewer.getExecutionParameters;
		this.miniCognosViewer.getWebContentRoot = cognosViewer.getWebContentRoot;

		this.m_AnnotationInfoBar = new AnnotationInfoBar(this.miniCognosViewer, lid);
		this.m_AnnotationInfoBar.setId( "annoInfoBar_" + annoId + lid );
		this.m_id = "annoInfoBar_" + annoId + lid + "_renderer";

		if (filterContext) {
			if (filterContext.getSlidersJSON()) {
				this.m_AnnotationInfoBar.setContainerInfo( {"sliders": filterContext.getSlidersJSON() } );
			}

			if (filterContext.getFilterJSON()) {
				this.m_AnnotationInfoBar.setEditableFilters( filterContext.getFilterJSON() );
			}
		}
	},

	render: function() {

		var renderHidden =
			"<div class=\"infoBar infoBarCollapsed\" id=\"infoBarHeaderCollapsed" + this.m_id +"\" >" +
			this.renderHeader(false) +
			"</div>";

		var renderShown =
			"<div class=\"infoBar infoBarExpanded\" id=\"infoBarHeaderExpanded" + this.m_id +"\" style=\"display:none;\" >" +
			this.renderHeader(true) +
			this.renderDetails() +
			"</div>";

		renderShown = this.updateHideButtonLabelledBy(renderShown);

		var instructions = document.getElementById("annotationInfoBarInstructions");

		if (instructions === null) {
			instructions = document.createElement("span");
			instructions.id = "annotationInfoBarInstructions";
			instructions.setAttribute("role", "presentation");
			instructions.style.visibility = "hidden";
			instructions.style.display = "none";

			var details =
				this.m_AnnotationInfoBar.createHiddenSpan("annoInfoBarButtonInstruction",  RV_RES.IDS_JS_ANNO_INFOBAR_BUTTON_EXPLANATION) +
				this.m_AnnotationInfoBar.createHiddenSpan("annoInfoBar_filterString", RV_RES.IDS_JS_FILTER) +
				this.m_AnnotationInfoBar.createHiddenSpan("annoInfoBar_promptString", RV_RES.IDS_JS_INFOBAR_PROMPT) +
				this.m_AnnotationInfoBar.createHiddenSpan("annoInfoBar_sliderString", RV_RES.IDS_JS_INFOBAR_SLIDER);

			document.body.appendChild(instructions);
			instructions.innerHTML = details;
		}

		return renderHidden + renderShown;
	},


	renderHeader: function(shown) {
		var mode = (shown)?"Hide":"Show";
		var buttonText = (shown)? RV_RES.IDS_JS_ANNO_INFOBAR_HIDE :RV_RES.IDS_JS_ANNO_INFOBAR_SHOW;

		var labelledBy = (shown)? ("XXREPLACEXX") : " annoInfoBarHeaderControlButtonText" + mode + this.m_id;

		var header =
			"<table class=\"annoInfoBarHeader\" id=\"annoInfoBarHeader" + mode + this.m_id + "\" role=\"presentation\">" +
				"<tr>" +
					"<td class=\"annoInfoBarHeaderText\" id=\"infoBarHeaderText" + this.m_id + "\" >" +
							RV_RES.IDS_JS_FILTER_APPLIED +
					"</td>" +
					"<td class=\"annoInfoBarHeaderControl\" id=\"infoBarHeader"+ mode + this.m_id + "\" >" +
						"<button id=\"annoInfoBarHeaderControl" + mode + this.m_id + "\" aria-labelledby=\"" + labelledBy + "\" type=\"button\" role=\"button\" tabindex=\"0\" value=\"\" title=\"" + buttonText + "\"" +
									" onclick=\"annotationInfoBarToggle('" + this.m_id + "', '"+ mode+"');\">"  +
							"<span id=\"annoInfoBarHeaderControlButtonText" + mode + this.m_id + "\"class=\"dijitInline annoInfoBarHeaderControlText\" title=\"" + buttonText + "\" >"+ buttonText+ "</span>" +
						"</button>" +
					"</td>"+
				"</tr>" +
			"</table>";

		return header;
	},

	renderDetails: function() {

		var promptDetail = this.m_AnnotationInfoBar.addPromptDetails(true);
		var filterDetail = this.m_AnnotationInfoBar.addFilteredItemsDetails(true);
		var sliderDetail = this.m_AnnotationInfoBar.addSliderDetails(true);

		var renderDetails =
			"<table class=\"annoInfoBarDetails\" id=\"annoInfoBarDetails" + this.m_id + "\" role=\"presentation\">"+
				promptDetail + filterDetail + sliderDetail +
			"</table>";

		this.m_labelledByHideButton =
			(promptDetail.length === 0)? "":(" annoInfoBar_promptString " + this.m_AnnotationInfoBar.getId() + "0_promptDetail ");
		this.m_labelledByHideButton +=
			(filterDetail.length === 0)? "":(" annoInfoBar_filterString " + this.m_AnnotationInfoBar.getId() + "0_filterDetail ");
		this.m_labelledByHideButton +=
			(sliderDetail.length === 0)? "":(" annoInfoBar_sliderString " + this.m_AnnotationInfoBar.getId() + "0_sliderDetail " +
										  " annoInfoBar_sliderString " + this.m_AnnotationInfoBar.getId() + "1_sliderDetail " );
		this.m_labelledByHideButton +=
			" annoInfoBarHeaderControlButtonTextHide" + this.m_id;

		return renderDetails;
	},

	updateHideButtonLabelledBy: function(headerString) {
		return headerString.replace("XXREPLACEXX", this.m_labelledByHideButton);
	}
});

//==============================================
// Global function for annotation InfoBar Toggle
function annotationInfoBarToggle(mId, changeTo)
{
	var expanded = document.getElementById("infoBarHeaderExpanded" + mId);
	var collapsed = document.getElementById("infoBarHeaderCollapsed" + mId);

	var control = null;
	if ("Show" === changeTo) {
		dojo.style(expanded, {display: ""});
		dojo.style(collapsed, {display: "none"});
		control = document.getElementById("annoInfoBarHeaderControl" + 'Hide' + mId);

	} else {
		dojo.style(collapsed, {display: ""});
		dojo.style(expanded, {display: "none"});
		control = document.getElementById("annoInfoBarHeaderControl" + 'Show' + mId);
	}

	if (control) {
		control.focus();
	}
}
