/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */
g_firefoxPluginHelper = new CFirefoxPluginHelper();

function CFirefoxPluginHelper()
{
	this.EVENT_NAME = "DataTransferEvent";
	this.EVENT_BACK_NAME = "DataBackchannelEvent";
	this.CALLER_ID = "dataelement";
	this.ELEMENT_NAME = "DataElement";

	this.m_bInitialized = false;
	this.m_bDebugEnabled = false;
}

/**
 * initializes the Element and Listeners
 */
CFirefoxPluginHelper.prototype.init = function()
{
	if (this.m_bInitialized == true)
	{
		return;
	}
	this.m_bInitialized = true;

	// create data / event firing Elements
	var element = this.createElement();
	// register custom event on callback
	element.addEventListener(this.EVENT_BACK_NAME, this.calledBack, true);

	if (typeof gCognosViewer != "undefined" && gCognosViewer.envParams != null && gCognosViewer.envParams["cv.debugDirectory"] != "")
	{
		this.m_bDebugEnabled = true;
	}
};


/**
 * creates the data element
 *
 * @return {HTMLElement} the created Element of the type <code>Communicator.ELEMENT_NAME</code>
 */
CFirefoxPluginHelper.prototype.createElement = function()
{
	if ("createEvent" in document)
	{
		// if element is not yet existing
		if (!document.getElementById(this.CALLER_ID))
		{
			var element = document.createElement(this.ELEMENT_NAME);
			element.setAttribute("id", this.CALLER_ID);
			// attribute containing "data parameter" for extension call
			element.setAttribute("data", "");
			// attribute containing "return value" of extension
			element.setAttribute("returnvalue", "");
			document.documentElement.appendChild(element);
			return element;
		}
		else
		{
			// element exists - return that
			return document.getElementById(this.CALLER_ID);
		}
	}
	else
	{
		return null;
	}
};

CFirefoxPluginHelper.prototype.processResponseState = function(oState, oCV)
{
	if (typeof oState.debugLogs != "undefined")
	{
		this.addDebugLogsToFirefoxPlugin(oState.debugLogs);
	}

	if (typeof document.getElementById('intermediateXML' + oCV.getId()) != "undefined" && document.getElementById('intermediateXML' + oCV.getId()) != null)
	{
		this.addIntermediateXML(oCV.getId(), document.getElementById('intermediateXML' + oCV.getId()).value);
	}
};

CFirefoxPluginHelper.prototype.addSubscriptionSpec = function(sc)
{
	if (!this.m_bDebugEnabled)
	{
		return;
	}

	if (sc && sc.m_oCognosViewer != null && typeof sc.m_oCognosViewer != "undefined")
	{
		var oCV = sc.m_oCognosViewer;
		if (oCV.m_oSubscriptionManager != null)
		{
			try
			{
				var fWR = document.getElementById("formWarpRequest" + oCV.getId());

				var selectionXml = new CSelectionXml(	fWR["ui.burstID"].value,
														fWR["ui.contentLocale"].value,
														fWR["ui.outputLocale"].value
													);

				selectionXml.BuildSelectionFromController(sc);

				this.debug("Subscription Spec", selectionXml.toXml(), "xml");
			}
			catch (e) {}
		}
	}
};

CFirefoxPluginHelper.prototype.addContextInfo = function(sc)
{
	if (!this.m_bDebugEnabled)
	{
		return;
	}

	if (sc)
	{
		var sText = "";
		var selectedObjects = sc.getAllSelectedObjects();

		if (selectedObjects.length <= 0)
		{
			return;
		}

		this.addSubscriptionSpec(sc);

		for(var s = 0; s < selectedObjects.length; ++s)
		{
			var selection = selectedObjects[s];

			var sCTX = "";
			var aContextIds = selection.getSelectedContextIds();
			var i,j;
			for(i = 0; i < aContextIds.length; ++i)
			{
				if (i > 0)
				{
					sCTX += "::";
				}

				for(j = 0; j < aContextIds[i].length; ++j)
				{
					if (j > 0)
					{
						sCTX += ":";
					}

					sCTX += aContextIds[i][j];
				}
			}

			if (sText != "")
			{
				sText += "\n\n\n";
			}
			sText += "selection ctx: " + sCTX;

			var oCell = selection.getCellRef();
			var dttargets = "";
			if (oCell != null && oCell.firstChild != null)
			{
				dttargets = oCell.firstChild.getAttribute("dttargets");
				if (typeof oCell.firstChild.getAttribute("dttargets") != "undefined" && oCell.firstChild.getAttribute("dttargets") != null)
				{
					dttargets = oCell.firstChild.getAttribute("dttargets");
				}
			}


			var selectionCtx = selection.getSelectedContextIds();
			var muns = selection.getMuns();
			var munCount = muns.length;

			for(i = 0; i < munCount; ++i)
			{
				for(j = 0; j < muns[i].length; ++j)
				{
					var ctxId = selectionCtx[i][j];

					sText += "\n\nctx: " + ctxId;
					sText += "\n\tabc: " + sc.getDisplayValue(ctxId);
					sText += "\n\tuse value: " + sc.getUseValue(ctxId);
					sText += "\n\tref data item: " + sc.getRefDataItem(ctxId);
					sText += "\n\tmun: " + sc.getMun(ctxId);
					sText += "\n\tlun: " + sc.getLun(ctxId);
					sText += "\n\thun: " + sc.getHun(ctxId);

					if (i == 0 && j == 0)
					{
						sText += "\n\tdttargets: " + dttargets;
					}
				}
			}
		}

		this.debug("Context Info", sText, "text");
	}
};

CFirefoxPluginHelper.prototype.debug = function(hint, text, displayHint)
{
	if (!this.m_bDebugEnabled)
	{
		return;
	}

	if (displayHint == null || typeof displayHint == "undefined" || displayHint.length == 0)
	{
		displayHint = "text";
	}

	var json = '{"obj":"misc", "data":{"name":"' + escape(hint) + '","displayHint":"' + displayHint + '","text":"' + escape(text) +'"}}';
	this.sendData(escape(json));
};

CFirefoxPluginHelper.prototype.addIntermediateXML = function(oCVID, xml)
{
	if (typeof xml != "undefined" && xml != null)
	{
		this.debug("Intermediate XML", xml, "xml");
	}
};

CFirefoxPluginHelper.prototype.addDebugLogsFromResponseXML = function(responseXML)
{
	if (responseXML != null)
	{
		var node = responseXML.getElementsByTagName("debugLogs");

		if (node != null && node.length > 0)
		{
			if (typeof node[0].text != "undefined")
			{
				this.addDebugLogsToFirefoxPlugin(node[0].text);
			}
			else
			{
				this.addDebugLogsToFirefoxPlugin(node[0].textContent);
			}
		}
	}
};

/**
 * calls the firefox plugin with JSON - data (object)
 *
 * @param {Object} data the data to transfer to extension - must be convertible to JSON
 */
CFirefoxPluginHelper.prototype.addDebugLogsToFirefoxPlugin = function(data)
{
	this.sendData(escape('{"obj":"debugLogs", "data":' + data + '}'));
};

CFirefoxPluginHelper.prototype.sendData = function(data)
{
	this.init();

	// create or get our element
	var element = this.createElement();
	element.setAttribute("data", data);

	// create and fire custom Event to notify extension
	var ev = document.createEvent("Event");
	ev.initEvent(this.EVENT_NAME, true, false);
	element.dispatchEvent(ev);
};

CFirefoxPluginHelper.prototype.calledBack = function()
{
};
