/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */
/*******************************************
		Common Utility Functions
*******************************************/
var GUtil = {};

GUtil.createHiddenForm = function(name, method, viewerId, target) {
	var form = document.getElementById(name);
	if(form) {
		document.body.removeChild(form);
	}

	form = document.createElement("form");
	form.id = name;
	form.name = name;
	form.method = method;
	form.style.display = "none";
	form.action = document.forms["formWarpRequest" + viewerId].action;
	form.target = target + (new Date()).getTime();

	document.body.appendChild(form);
	return form;
};


GUtil.createFormField = function(el, name, value) {
	var input = document.createElement("input");
	input.type = "hidden";
	input.name = name;
	input.value = value;

	el.appendChild(input);
};

/*----------------------------------------------------------
GenerateCallback:
	func:		Function object to use. To call foo.test() pass in foo.test.
	aParams:	Array of parameters to pass to func.
	oContext:	Calling context. If "foo" in foo.test is and instance of "Foo" object, then you need
				to pass the instance as a context. This is this difference between calling Foo.test() and
				foo.test(), where foo = new Foo();

	Returns:	A function object that can be used as a callback, but allows for the passing of stored parameters.

	Example:	var foo = new Foo();
				var func1 = GUtil.generateCallback(alert, ["Hello World"]);
				var func2 = GUtil.generateCallback(window.alert, ["Hi there"], window);

				//alert "Hello World" after 5 seconds.
				setTimeout(func1, 5000);

				//alert "Hi there" after 10 seconds.
				setTimeout(func2, 10000);
------------------------------------------------------------*/
GUtil.generateCallback = function(func, aParams, oContext){
	//normally "this" is probably the window object, but
	//we can pass in a different context object if we need to.
	if (func) {
		var funcContext = oContext || this;
		aParams = (aParams instanceof Array)? aParams : [];
		return (function(response){
			try {
				if(typeof response != "undefined" && aParams.length == 0) {
					aParams.push(response);
				}
				return func.apply(funcContext,aParams);
			} catch ( except ) {
				alert(except.message);
			}
		});
	} else {
		//do nothing
		return (function() {});
	}
};




//TODO: This is a new function call added (perhaps temporarily) to support things like:
//		1. Synchronous XmlHttp calls
//		2. Modified callbacks.
// This should eventually all go through: CCognosViewerRequest


function XmlHttpObject() {
	this.m_formFields = new CDictionary();
	this.m_useAsynchronousRequest = true;
	this.xmlHttp = XmlHttpObject.createRequestObject();
	//call the wait-for-response in the context of this instance
}

XmlHttpObject.prototype.cancel = function() {
	if(this.xmlHttp != null) {
		this.successCallback = null;
		this.errorCallback = null;
		this.xmlHttp.abort();
	}
};

XmlHttpObject.prototype.setCallbacks = function(scb, ecb) {
	this.successCallback = scb || null;
	this.errorCallback = ecb || null;
};

XmlHttpObject.createRequestObject = function() {
	var request = null;

	if (window.XMLHttpRequest)		{
        request = new XMLHttpRequest();						// Firefox, Safari, ...
	} else if (window.ActiveXObject)	{
       request = new ActiveXObject("Msxml2.XMLHTTP");		// Internet Explorer
	} else {
		//throw error. exit.
	}
	return request;
};


XmlHttpObject.prototype.waitForXmlHttpResponse = function() {
	//explicit so there's no confusion
	var request = this.xmlHttp;
	if (request.readyState === 4) {
		if(request.status === 200 && this.successCallback) {
			this.successCallback();
			this.successCallback = null;
		} else if (request.errorCallback) {
			this.errorCallback();
			this.errorCallback = null;
		}
	} else {
		//wait...
	}
};


XmlHttpObject.prototype.sendHtmlRequest = function(action, gateway, url, async) {

	if (typeof xhrMultiPart !== "undefined" && xhrMultiPart.active) {

		if (url) {
			var params = url.split("&");
			for (var i = 0; i < params.length; i++) {
				var param = params[i];
				var tuple = param.split("=");
				if (tuple && tuple.length > 0) {
					if (tuple.length === 1) {
						this.addFormField(tuple[0], null);
					} else {
						this.addFormField(tuple[0], tuple[1]);
					}
				}
			}
		}

		var args = {
			url: gateway,
			content: this.m_formFields.m_aValues,
			sync: !async,
			preventCache: true,
			load: dojo.hitch(this, function(responseObj) {
				this.xmlHttp = responseObj;
				if (this.successCallback) {
					this.successCallback();
				}
				return responseObj;
			}),

			error: dojo.hitch(this, function(responseObj) {
				this.xmlHttp = responseObj;
				if (this.errorCallback) {
					this.errorCallback();
				}
				return responseObj;
			})
		};

		xhrMultiPart.Post(args);
	}
	else {
		var request = this.xmlHttp;
		request.open(action, gateway, async);

		//should be after the "open" for best results...
		if (async) {
			request.onreadystatechange = GUtil.generateCallback(this.waitForXmlHttpResponse, [], this);
		} else {
			request.onreadystatechange = GUtil.generateCallback(this.waitForXmlHttpResponse, [], this); // IE needs this.
			if (!document.all) {
				if (this.successCallback) {
					request.onload = GUtil.generateCallback(this.successCallback, [], this);
				}
				if (this.errorCallback) {
					request.onerror = GUtil.generateCallback(this.errorCallback, [], this);
				}
			}
		}

		request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
		request.send(this.convertFormFieldsToUrl() + url);
	}
};

XmlHttpObject.prototype.getResponseXml = function() {
	return (this.xmlHttp.status === 200) ? this.xmlHttp.responseXML : null;
};

XmlHttpObject.prototype.getResponseText = function() {
	return (this.xmlHttp.status === 200) ? this.xmlHttp.responseText : "";
};

XmlHttpObject.prototype.getResponseHeader = function(item) {
	return (this.xmlHttp.status === 200) ? this.xmlHttp.getResponseHeader(item) : null;
};

XmlHttpObject.prototype.getStatus = function() {
	return this.xmlHttp.status;
};

XmlHttpObject.prototype.addFormField = function(name, value) {
	this.m_formFields.add(name, value);
};

XmlHttpObject.prototype.convertFormFieldsToUrl = function() {

	var url = "";
	var formFieldNames = this.m_formFields.keys();
	for (var index = 0; index < formFieldNames.length; index++)
	{
		url += encodeURIComponent(formFieldNames[index]) + "=" + encodeURIComponent(this.m_formFields.get(formFieldNames[index])) + "&";
	}
	return url;
};


function AsynchRequest() {
	// initialize the base class
	AsynchRequest.baseConstructor.call(this);

	this.m_soapFault = null;
	this.m_faultDialog = null;
	this.m_promptDialog = null;
	this.m_logonDialog = null;
	this.addFormField("b_action", "cognosViewer");
	this.addFormField("useAjax", "true");
	this.addFormField("cv.catchLogOnFault", "true");
	this.addFormField("cv.showFaultPage", "false");
}
// set up the base class
AsynchRequest.prototype = new XmlHttpObject();
AsynchRequest.baseConstructor = XmlHttpObject;

AsynchRequest.prototype.getTracking = function() {return "";}; // treat as an abstract method
AsynchRequest.prototype.getConversation = function() {return "";}; // treat as an abstract method
AsynchRequest.prototype.getPrimaryAction = function() {return "";}; // treat as an abstract method
AsynchRequest.prototype.getActionState = function() {return "";}; // treat as an abstract method
AsynchRequest.prototype.getAsynchStatus = function() {return "";}; // treat as an abstract method
AsynchRequest.prototype.getResult = function() {return null;}; // treat as an abstract method
AsynchRequest.prototype.getSoapFault = function() { return this.m_soapFault; };
AsynchRequest.prototype.constructFaultEnvelope = function() { return null; }; // treat as an abstract method
AsynchRequest.prototype.getPromptHTMLFragment = function() {return "";};// treat as an abstract method
AsynchRequest.prototype.isRAPWaitTrue = function() {return false;}; // treat as an abstract method
AsynchRequest.prototype.getRAPRequestCache = function() { return null;}; // treat as an abstract method
AsynchRequest.prototype.construct = function() {}; // treat as an abstract method



AsynchRequest.prototype.newAsynchRequest = function() {
	var asynchRequest = this.construct();
	asynchRequest.setPromptDialog(this.m_promptDialog);
	asynchRequest.setFaultDialog(this.m_faultDialog);
	asynchRequest.setLogonDialog(this.m_logonDialog);
	asynchRequest.m_useAsynchronousRequest = this.m_useAsynchronousRequest;

	return asynchRequest;
};

AsynchRequest.prototype.success = function() {
	var asynchStatus = this.getAsynchStatus();
	switch(asynchStatus) {
		case "stillWorking":
		case "working":
			this.working();
			break;
		case "prompting":
			this.prompting();
			break;
		case "fault":
		case "complete":
		case "conversationComplete":
			this.complete();
			break;
	}
};

AsynchRequest.prototype.setFaultDialog = function(faultDialog) {
	if(faultDialog instanceof IFaultDialog) {
		this.m_faultDialog = faultDialog;
	} else if(typeof console != "undefined") {
		console.log("The parameter faultDialog must be an instance of IFaultDialog");
	}
};

AsynchRequest.prototype.setPromptDialog = function(promptDialog) {
	if(promptDialog instanceof IPromptDialog) {
		this.m_promptDialog = promptDialog;
	} else if(typeof console != "undefined") {
		console.log("The parameter promptDialog must be an instance of IPromptDialog");
	}
};

AsynchRequest.prototype.setLogonDialog = function(logonDialog) {
	if(logonDialog instanceof ILogOnDialog) {
		this.m_logonDialog = logonDialog;
	} else if(typeof console != "undefined") {
		console.log("The parameter logOnDialog must be an instance of ILogOnDialog");
	}
};

/**
 * @param gateway - Server path for http requests
 * @param webContentRoot - The path to web resources on the server (images/html pages/etc)
 * @param callbacks - The callbacks structure
 */
AsynchRequest.prototype.init = function(gateway, webContentRoot, callbacks) {
	this.m_gateway = gateway;
	this.m_webContentRoot = webContentRoot;
	this.m_callbacks = callbacks || null;
};

AsynchRequest.prototype.resubmitRequest = function() {
	var asynchRequest = this.newAsynchRequest();
	asynchRequest.m_formFields = this.m_formFields;
	asynchRequest.sendRequest();
	return asynchRequest;
};

AsynchRequest.prototype.sendRequest = function() {
	this.setCallbacks(this.successHandler, this.errorHandler);
	this.sendHtmlRequest("POST", this.m_gateway, "", this.m_useAsynchronousRequest);
};

AsynchRequest.prototype.forceSynchronous = function() {
	this.m_useAsynchronousRequest = false;
};

AsynchRequest.prototype.concatResponseArguments = function(customArguments) {
	var responseArguments = [this];
	if(customArguments) {
		responseArguments = responseArguments.concat(customArguments);
	}
	return responseArguments;
};

AsynchRequest.prototype.errorHandler = function() {
	if(this.m_callbacks && this.m_callbacks.error) {
		var callbackArguments = this.concatResponseArguments(this.m_callbacks.customArguments);
		var errorCallback = GUtil.generateCallback(this.m_callbacks.error.method, callbackArguments, this.m_callbacks.error.object);
		errorCallback();
	}
};

AsynchRequest.prototype.successHandler = function() {

	// hide any currently open dialogs
	if(typeof window["AsynchRequestPromptDialog"] != "undefined" && window["AsynchRequestPromptDialog"] != null) {
		window["AsynchRequestPromptDialog"].hide();
		window["AsynchRequestPromptDialog"] = null;
	}

	// check to see if dispatch caught the request and returned a login dialog
	if(this.getResponseHeader("Content-type").indexOf("text/html") != -1) {
		var responseText = this.getResponseText();
		if(responseText.indexOf("<ERROR_CODE>CAM_PASSPORT_ERROR</ERROR_CODE>") != -1) {
			this.passportTimeout();
		} else {
			// unknown/unexpected html response, throw it up in a new window
			var htmlWindow = window.open("","",'height=400,width=500');
			if(htmlWindow != null) {
				htmlWindow.document.write(responseText);
			}
		}
	} else {
		// verify we didn't receive a fault
		this.m_soapFault = this.constructFaultEnvelope();
		if(this.m_soapFault != null) {
			var camElement = XMLHelper_FindChildByTagName(this.m_soapFault, "CAM", true);
			if(camElement != null && XMLHelper_FindChildByTagName(camElement, "promptInfo", true)) {
				this.passportTimeout();
			} else {
				this.fault();
			}
		} else {
			this.success();
		}
	}
};

AsynchRequest.prototype.working = function() {
	if(this.m_callbacks && this.m_callbacks.working) {
		var callbackArguments = this.concatResponseArguments(this.m_callbacks.customArguments);
		var workingCallback = GUtil.generateCallback(this.m_callbacks.working.method, callbackArguments, this.m_callbacks.working.object);
		workingCallback();
	} else {
		// handle internally
		var asynchRequest = this.newAsynchRequest();
		asynchRequest.addFormField("m_tracking", this.getTracking());

		if (this.isRAPWaitTrue()) {
			// if rapWait is true, we can to send all parameters with current tracking again.
			asynchRequest.m_formFields = this.m_formFields;
			asynchRequest.addFormField("m_tracking", this.getTracking());
			asynchRequest.addFormField("rapWait", "true");
			var requestCache = this.getRAPRequestCache();
			if (requestCache !== null && typeof requestCache != "undefined" ) {
				asynchRequest.addFormField("rapRequestCache", requestCache);
			}
		} else {
			asynchRequest.addFormField("ui.action", "wait");
			asynchRequest.addFormField("ui.primaryAction", this.getPrimaryAction());
			asynchRequest.addFormField("cv.actionState", this.getActionState());
		}

		asynchRequest.sendRequest();
	}
};

AsynchRequest.prototype.prompting = function() {
	if(this.m_callbacks && this.m_callbacks.prompting) {
		var callbackArguments = this.concatResponseArguments(this.m_callbacks.customArguments);
		var promptingCallback = GUtil.generateCallback(this.m_callbacks.prompting.method, callbackArguments, this.m_callbacks.prompting.object);
		promptingCallback();
	} else if(this.m_promptDialog != null){
		this.showPromptPage();
	} else if(typeof console != "undefined") {
		console.log("An unhandled prompt response was returned: %o", this.xmlHttp);
	}
};

AsynchRequest.prototype.promptPageOkCallback = function(promptValues) {
	var asynchRequest = this.newAsynchRequest();
	asynchRequest.addFormField("ui.action", "forward");
	asynchRequest.addFormField("m_tracking", this.getTracking());
	asynchRequest.addFormField("ui.conversation", this.getConversation());
	asynchRequest.addFormField("ui.primaryAction", this.getPrimaryAction());
	asynchRequest.addFormField("cv.actionState", this.getActionState());

	for(var promptValue in promptValues) {
		asynchRequest.addFormField(promptValue, promptValues[promptValue]);
	}

	asynchRequest.sendRequest();
	window["AsynchRequestObject"] = null;
};

AsynchRequest.prototype.promptPageCancelCallback = function() {
	window["AsynchRequestPromptDialog"].hide();
	this.complete();
};

AsynchRequest.prototype.showPromptPage = function() {
	window["AsynchRequestObject"] = this;
	window["AsynchRequestPromptDialog"] = this.m_promptDialog;
	var cvIdParam = this.m_promptDialog.getViewerId() == null ? "" : "?cv.id=" + this.m_promptDialog.getViewerId();
	window["AsynchRequestPromptDialog"].initialize(this.m_webContentRoot + "/rv/showStandalonePrompts.html" + cvIdParam, 400, 400);
	window["AsynchRequestPromptDialog"].show();
};

AsynchRequest.prototype.passportTimeout = function() {
	if(this.m_callbacks && this.m_callbacks.passportTimeout) {
		var callbackArguments = this.concatResponseArguments(this.m_callbacks.customArguments);
		var passportCallback = GUtil.generateCallback(this.m_callbacks.passportTimeout.method, callbackArguments, this.m_callbacks.passportTimeout.object);
		passportCallback();
	} else if(this.m_logonDialog != null) {
		var okCallback = GUtil.generateCallback(this.resubmitRequest, [], this);
		var response = this.getResponseText();
		this.m_logonDialog.show(okCallback, response);
	} else if(typeof console != "undefined") {
		console.log("An unhandled passport timeout fault was returned: %o", this.getSoapFault());
	}
};

AsynchRequest.prototype.fault = function() {
	if(this.m_callbacks && this.m_callbacks.fault) {
		var callbackArguments = this.concatResponseArguments(this.m_callbacks.customArguments);
		var faultCallback = GUtil.generateCallback(this.m_callbacks.fault.method, callbackArguments, this.m_callbacks.fault.object);
		faultCallback();
	} else if(this.m_faultDialog != null) {
		this.m_faultDialog.show(this.getSoapFault());
	} else if(typeof console != "undefined") {
		console.log("An unhandled soap fault was returned: %o", this.getSoapFault());
	}
};

AsynchRequest.prototype.complete = function() {
	if(this.m_callbacks && this.m_callbacks.complete) {
		var callbackArguments = this.concatResponseArguments(this.m_callbacks.customArguments);
		var completeCallback = GUtil.generateCallback(this.m_callbacks.complete.method, callbackArguments, this.m_callbacks.complete.object);
		completeCallback();
	}
};

AsynchRequest.prototype.getSoapFaultCode = function() {
	var soapFault = this.constructFaultEnvelope();
	if(soapFault) {
			var faultCode = XMLHelper_FindChildByTagName(soapFault, "faultcode", true);
			if(faultCode != null) {
				return XMLHelper_GetText(faultCode);
			}
	}
	return null;
};

AsynchRequest.prototype.getSoapFaultDetailMessageString = function() {
	var soapFault = this.constructFaultEnvelope();
	if(soapFault) {
			var entry = XMLHelper_FindChildByTagName(soapFault, "messageString", true);
			if(entry != null) {
				return XMLHelper_GetText(entry);
			}
	}
	return null;
};

function AsynchJSONRequest(gateway, webContentRoot, callbacks) {
	// initialize the base class
	AsynchJSONRequest.baseConstructor.call(this);

	this.m_jsonResponse = null;
	this.addFormField("cv.responseFormat", "asynchJSON");
	this.init(gateway, webContentRoot, callbacks);

}

// set up the base class
AsynchJSONRequest.prototype = new AsynchRequest();
AsynchJSONRequest.baseConstructor = AsynchRequest;

AsynchJSONRequest.prototype.getJSONResponseObject = function() {
	if(this.m_jsonResponse == null) {
		var text = this.getResponseText();
		if(text != null) {
			this.m_jsonResponse = eval("(" + text + ")");
		}
	}
	return this.m_jsonResponse;
};

AsynchJSONRequest.prototype.getTracking = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse) {
		return jsonResponse.tracking;
	}

	return "";
};

AsynchJSONRequest.prototype.getConversation = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse) {
		return jsonResponse.conversation;
	}

	return "";
};

AsynchJSONRequest.prototype.getAsynchStatus = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse) {
		return jsonResponse.status;
	}

	return "unknown";
};

AsynchJSONRequest.prototype.getPrimaryAction = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse) {
		return jsonResponse.primaryAction;
	}

	return "";
};

AsynchJSONRequest.prototype.getActionState = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse) {
		return jsonResponse.actionState;
	}

	return "";
};

AsynchJSONRequest.prototype.isRAPWaitTrue = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse) {
		return (jsonResponse.rapWait === "true");
	}

	return false;
};


AsynchJSONRequest.prototype.getRAPRequestCache = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse) {
		var requestCache = jsonResponse.rapRequestCache;
		if (requestCache !== null && typeof requestCache != "undefined" ) {
			return requestCache;
		}
	}

	return null;
};

AsynchJSONRequest.prototype.getResult = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse && jsonResponse.json) {
		return eval("(" + jsonResponse.json + ")");
	}

	return null;
};

AsynchJSONRequest.prototype.getPromptHTMLFragment = function() {
	var jsonResponse = this.getJSONResponseObject();
	if(jsonResponse && jsonResponse.promptHTMLFragment) {
		return jsonResponse.promptHTMLFragment;
	}
	return "";
};

AsynchJSONRequest.prototype.constructFaultEnvelope = function() {

	if(this.m_soapFault == null) {
		var jsonResponse = this.getJSONResponseObject();
		if(jsonResponse.status == "fault") {
			this.m_soapFault = XMLBuilderLoadXMLFromString(jsonResponse.fault);
		}
	}
	return this.m_soapFault;
};

AsynchJSONRequest.prototype.construct = function() {
	var asynchRequest = new AsynchJSONRequest(this.m_gateway, this.m_webContentRoot, this.m_callbacks);
	return asynchRequest;
};

function AsynchDATARequest(gateway, webContentRoot, callbacks) {
	// initialize the base class
	AsynchDATARequest.baseConstructor.call(this);

	this.m_oResponseState = null;
	this.m_endOfStateIdx = -1;
	this.cStatePrefix = "<xml><state>";
	this.cStateSuffix = "</state></xml>";

	this.addFormField("cv.responseFormat", "data");	//??make sense
	this.init(gateway, webContentRoot, callbacks);
}

// A server request that expects a data response (state and HTML)
// set up the base class
AsynchDATARequest.prototype = new AsynchRequest();
AsynchDATARequest.baseConstructor = AsynchRequest;

AsynchDATARequest.prototype.getEndOfStateIdx = function() {
	if (this.m_endOfStateIdx==-1) {
		var prefix = this.getResponseText().substring(0,12);
		if (prefix==this.cStatePrefix) {
			this.m_endOfStateIdx = this.getResponseText().indexOf(this.cStateSuffix);
			if (this.m_endOfStateIdx!=-1) {
				this.m_endOfStateIdx+=this.cStateSuffix.length;
			}
		}
	}
	return this.m_endOfStateIdx;
};

AsynchDATARequest.prototype.getResponseState = function() {
	if(this.m_oResponseState==null && this.getEndOfStateIdx()!=-1) {
		var text = this.getResponseText().substring(this.cStatePrefix.length,
					this.getEndOfStateIdx() - this.cStateSuffix.length);
		if(text != null) {
			this.m_oResponseState=eval("(" + xml_decode(text) + ")");
		}
	}
	return this.m_oResponseState;
};

AsynchDATARequest.prototype.getAsynchStatus = function() {
	if (this.getResponseState()!=null && typeof this.getResponseState().status != "undefined")	{
		return this.getResponseState().status;
	}
	return "unknown";
};

AsynchDATARequest.prototype.getTracking = function() {
	if (this.getResponseState()!=null && typeof this.getResponseState().tracking != "undefined") {
		return this.getResponseState().tracking;
	}
	return "";
};

AsynchDATARequest.prototype.getConversation = function() {
	if (this.getResponseState()!=null && typeof this.getResponseState().conversation != "undefined") {
		return this.getResponseState().conversation;
	}
	return "";
};

AsynchDATARequest.prototype.getPrimaryAction = function() {
	if (this.getResponseState()!=null && typeof this.getResponseState().primary_action != "undefined") {
		return this.getResponseState().primary_action;
	}
	return "";
};

AsynchDATARequest.prototype.getActionState = function() {
	if (this.getResponseState()!=null && typeof this.getResponseState().action_state != "undefined") {
		return this.getResponseState().action_state;
	}
	return "";
};

AsynchDATARequest.prototype.getResult = function() {
	if (this.getEndOfStateIdx()!=-1) {
		return this.getResponseText().substring(this.getEndOfStateIdx(), this.getResponseText().length);
	}
	return "";
};

AsynchDATARequest.prototype.getPromptHTMLFragment = function() {
	return this.getResult();
};

AsynchDATARequest.prototype.constructFaultEnvelope = function() {

	if(this.m_soapFault == null) {
		var faultDocument = this.getResponseXml();
		if(faultDocument != null) {
			var faultState = XMLHelper_FindChildByTagName(faultDocument, "state", true);
			if(faultState != null) {
				var jsonState = eval("(" + XMLHelper_GetText(faultState) + ")");
				if(jsonState.soapFault) {
					var faultString = jsonState.soapFault;
					this.m_soapFault = XMLBuilderLoadXMLFromString(faultString);
				}
			}
		}
	}

	return this.m_soapFault;
};

AsynchDATARequest.prototype.construct = function() {
	var asynchRequest = new AsynchDATARequest(this.m_gateway, this.m_webContentRoot, this.m_callbacks);
	return asynchRequest;
};

function ILogOnDialog(){}
ILogOnDialog.prototype.show = function(okCallback, /* optional*/ response) {  if(typeof console != "undefined") { console.log("Required method ILogOnDialog:show not implemented."); } };
ILogOnDialog.prototype.hide = function() {  if(typeof console != "undefined") { console.log("Required method ILogOnDialog:hide not implemented."); } };

function BuxLogOnDialog() {}
BuxLogOnDialog.prototype = new ILogOnDialog();
BuxLogOnDialog.prototype.show = function(okCallback) {
	var dialogImpl = new bux.dialogs.LogonDialog({ okHandler: okCallback, cancelHandler: function() { }});
	dialogImpl.startup();
	dialogImpl.show();
};

function IFaultDialog(){}
IFaultDialog.prototype.show = function() { if(typeof console != "undefined") { console.log("Required method IFaultDialog:show not implemented."); } };

function BuxFaultDialog(){}
BuxFaultDialog.prototype = new IFaultDialog();
BuxFaultDialog.prototype.show = function(soapFaultDocument) {

	var detailsString = "";
	var messageString = "", messageStringNode = "";
	var errorMessages = XMLHelper_FindChildrenByTagName(soapFaultDocument, "message", true);

	if(errorMessages.length > 0)
	{
		for(var messageIndex = 1; messageIndex < errorMessages.length; ++messageIndex)
		{
			messageStringNode = XMLHelper_FindChildByTagName(errorMessages[messageIndex], "messageString", false);
			detailsString += XMLHelper_GetText(messageStringNode, false) + "\n";
		}

		messageStringNode = XMLHelper_FindChildByTagName(errorMessages[0], "messageString", false);
		messageString = XMLHelper_GetText(messageStringNode, false);
	}

	bux.messageBox(bux.MB_ERROR, messageString, "",  detailsString);
};

function IPromptDialog(){}
IPromptDialog.prototype.initialize = function(url,width,height) { if(typeof console != "undefined") { console.log("Required method IModalDialog:initialize not implemented."); } };
IPromptDialog.prototype.show = function() { if(typeof console != "undefined") { console.log("Required method IModalDialog:show not implemented."); } };
IPromptDialog.prototype.hide = function() { if(typeof console != "undefined") { console.log("Required method IModalDialog:hide not implemented."); } };

function BuxPromptDialog(cvId){
	this.m_url = "";
	this.m_width = 0;
	this.m_height = 0;
	this.m_dialogImpl = null;
	if (typeof cvId != "undefined") {
		this.m_cvId = cvId;
	}
}
BuxPromptDialog.prototype = new IPromptDialog();
BuxPromptDialog.prototype.initialize = function(url,width,height) {
	this.m_url = url;
	this.m_width = width;
	this.m_height = height;
};
BuxPromptDialog.prototype.show = function() {
	this.m_dialogImpl = new bux.dialogs.IFrameDialog();
	this.m_dialogImpl.src = this.m_url;
	this.m_dialogImpl.width = this.m_width;
	this.m_dialogImpl.height = this.m_height;
	this.m_dialogImpl.startup();
	this.m_dialogImpl.show();
};
BuxPromptDialog.prototype.hide = function() {
	this.m_dialogImpl.hide();
	this.m_dialogImpl.destroy();
	delete this.m_dialogImpl;
	this.m_dialogImpl = null;
};
BuxPromptDialog.prototype.getViewerId = function() {
	return typeof this.m_cvId == "undefined" ? null : this.m_cvId;
};

function StandaloneViewerPromptDialog(webContentRoot) {
	this.m_webContentRoot = webContentRoot;
	this.m_dialogImpl = null;
}
StandaloneViewerPromptDialog.prototype = new IPromptDialog();

StandaloneViewerPromptDialog.prototype.initialize = function(url,width,height) {
	this.m_dialogImpl = new CModal("", "", document.body, null, null, width, height, true, true, false, true, this.m_webContentRoot);
	var dialogIframe = document.getElementById(CMODAL_CONTENT_ID);
	dialogIframe.src = url;
};
StandaloneViewerPromptDialog.prototype.show = function() {
	this.m_dialogImpl.show();
};
StandaloneViewerPromptDialog.prototype.hide = function() {
	this.m_dialogImpl.hide();
	destroyCModal();
};

function StandaloneViewerLogOnDialog(cvId) {
	this.cvId = cvId;
}

StandaloneViewerLogOnDialog.prototype = new ILogOnDialog();
StandaloneViewerLogOnDialog.prototype.show = function(okCallback, response) {
	launchLogOnDialog(this.cvId, response);
};

StandaloneViewerLogOnDialog.prototype.hide = function() {
};
