/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */
dojo.provide("bux.reportViewer");

dojo.require("bux.dialogs.InformationDialog");
dojo.require("bux.Annotation");
dojo.require("bux.AnnotationIndicator");
dojo.require("bux.data.AnnotationStore");
dojo.require("bux.iwidget.common.MultilingualWidgetBase");
dojo.require("InfoBarBase");

var globalViewerIdentifier = 1;

dojo.declare("bux.reportViewer", bux.iwidget.common.MultilingualWidgetBase, {


	annotationStore: null,
	savedOutput: null,
	undoRedoQueue: null,
	buxActionDispatcher: null,
	properties: null,
	promptParametersRetrieved: false,

	//This value is included in the cv.id parameter in order to ensure that
	//the value begins with a letter. This is to guard against javascript
	//variable naming errors (a js var starting with a number is illegal).
	namespacePrefix: "x",

	//Timer for global callback necessary to make sure all scripts are loaded
	//and executed before continuing
	timer: null,

	//Interval (in milliseconds) for global script loader callback
	timerInterval: 20,

	//This constant must match the version value in ViewerIWidget.xml
	GODASHBOARD_REPORT_VERSION: "1.0.0.0",
	BUX_REPORT_VERSION: "1.0.0.0.0",

	//This object is to hold last action and parameters at of IWidget level.
	//If response is unexpected and Viewer object does not exist, this Widget level last action may be used for resubmit.
	viewerWidgetLastAction: null,

	//An object that holds the state of all sliders on the canvas affecting this widget.
	sliderState: {},

	sViewerIWidgetId: null,

	//Preference coming from BUX. This is passed to BUA. Value is string "true" or "false"
	m_sPaneOnRight: null,

	constructor: function() {
		this.onUnloadInProgress = false;
		this.annotationStore = new bux.data.AnnotationStore(this, this.commitComment, this);
		this.savedOutput = new CIWidgetSavedOutput(this);
		this.undoRedoQueue = new UndoRedoQueue(this);
		this.viewerIdentifier = (globalViewerIdentifier++).toString();
		if (typeof BuxDispatcher !== "undefined") {
			this.buxActionDispatcher = new BuxDispatcher();
		}
	},

	getActionDispatcher : function() {
		return this.buxActionDispatcher;
	},

	resetActionDispatcher : function() {
		if (this.buxActionDispatcher !== null) {
			delete this.buxActionDispatcher;
			this.buxActionDispatcher = new BuxDispatcher();
		}
	},

	getUndoRedoQueue: function() {
		return this.undoRedoQueue;
	},

	getDisplayName: function() {
		return this.getAttributeValue("widgetTitle");
	},

	getSavedOutput: function() {
		return this.savedOutput;
	},

	getAttributeValue: function(name) {
		return this.iContext.getiWidgetAttributes().getItemValue(name);
	},

	setAttributeValue: function(name, value) {
		this.iContext.getiWidgetAttributes().setItemValue(name, value);
	},

	removeAttributeValue: function(name) {
		this.iContext.getiWidgetAttributes().removeItem(name);
	},

	getViewerObject: function() {
		try {
			return window["oCV" + this.getViewerId()];
		}
		catch (ex) {
			//In Internet Explorer, if the variable has been previously deleted, an exception is
			//thrown when trying to reference it.
			//Since you can't even check for the existence of the variable without referencing it,
			//ignore the exception and return null.
			return null;
		}

	},

	getViewerId: function() {
		return this.namespacePrefix + this.viewerIdentifier;
	},
	isSavedOutput: function() {
		var viewerObject = this.getViewerObject();
		if(typeof viewerObject !== "undefined" && viewerObject !== null) {
			return viewerObject.envParams["ui.action"] == "view";
		}
		return false;
	},
	onLoad: function() {
		this.sViewerIWidgetId = this.iContext.getRootElement().id;
		this.m_sPaneOnRight = this.getAttributeValue( "paneOnRight");

		var properties = this.getAttributeValue( "viewerProperties");
		var promptParametersRetrieved = this.getAttributeValue( "promptParametersRetrieved");
		if ( promptParametersRetrieved !== null && promptParametersRetrieved == "true") {
			this.promptParametersRetrieved = true;
		}
		this.properties = this.createProperties(properties);
		this.savedOutput.setPagedOutput( !this.properties.getRetrieveAll());

		if (this.isOpeningSavedDashboard()) {
			//Opening a dashboard
			this.disableListenToIfFullReportUpgraded();
			this.enableFlashChartsToIfReportPartUpgraded();
			this.disableListenToIfReportPartUpgraded();
			this.postReport(null);
		} else {
			//Dropping a report on the canvas.
			var payload = { action: 'canvas.filters' };
			this.iContext.iEvents.fireEvent("com.ibm.bux.widget.action", null, payload );
		}
		this.iContext.iEvents.fireEvent("com.ibm.bux.widget.updateStoreID", null, {});
	},

	isReportInteractive: function()
	{
		return this.isSavedOutput() === false && !this.isLimitedInteractiveMode();
	},

	createProperties: function(properties) {
		return new bux.reportViewer.Properties( this, properties );
	},
	disableListenToIfFullReportUpgraded: function() {
		if (this.isGoDashboardReportUpgrade()) {
			if (this.getAttributeValue( "flexParam_properties" )==null) {
				var payload = {};
				payload[ '*' ] = { blockedEvents:
					['com.ibm.bux.data.filter', 'com.ibm.bux.filter.values.get', 'com.ibm.bux.filter.items.get']
				};
				this.iContext.iEvents.fireEvent("com.ibm.bux.widget.updateEventFilter", null, payload);
			}
		}
	},
	enableFlashChartsToIfReportPartUpgraded: function() {
		if (this.isGoDashboardReportUpgrade()) {
			var sOriginalReportPart = this.getAttributeValue("originalReportPart");
			if (typeof sOriginalReportPart !== "undefined" && sOriginalReportPart!==null) {
				this.properties.setProperty('flashCharts',"true");
			}
		}
	},

	disableListenToIfReportPartUpgraded: function() {
		if (this.isGoDashboardReportUpgrade()) {
			var sOriginalReportPart = this.getAttributeValue("originalReportPart");
			if (typeof sOriginalReportPart !== "undefined" && sOriginalReportPart!==null) {
				var payload = {};
				payload[ '*' ] = { blockedEvents:
					['com.ibm.bux.viewer.drill', 'com.ibm.bux.viewer.prompt']
				};
				this.iContext.iEvents.fireEvent("com.ibm.bux.widget.updateEventFilter", null, payload);
			}
		}
	},


	onUnload: function() {
		this.onUnloadInProgress = true;
		this.destroy();
	},

	onGetFiltersDone: function(evt) {
		// filterRequiredAction : Undo | Redo | ResetToOriginal
		if (typeof this.filterRequiredAction != "undefined" && this.filterRequiredAction != null) {
			var dataFiltersParam = this.buildDataFiltersParam(evt.payload.filters);
			var actionFactory = new ActionFactory(this.getViewerObject());
			var action = actionFactory.load(this.filterRequiredAction);
			action.sendRequest(dataFiltersParam, this.filterRequiredAction);
			this.filterRequiredAction = null;
		} else {
			this.postReport(evt.payload.filters);
		}
	},

	postReport: function(filters) {
		this.viewerWidgetLastAction = {"functionName": "postReport", "param": [filters]};

		var args = {
			url: this.buildPostUrl(),
			sync: false,
			preventCache: true,
			content: this.buildPostContent(filters),
			error: dojo.hitch(this, function(response){
					this.iContext.iEvents.fireEvent("com.ibm.bux.widget.render.done", null, {noAutoResize:true});
					this.iContext.iEvents.fireEvent("com.ibm.bux.widget.notification", null, {type: "error", message: response.message, description: response.description});
				})
		};

		if (typeof xhrMultiPart !== "undefined" && xhrMultiPart.active) {
			args.load = dojo.hitch(this, function(response){
				this.loadContent(response.responseText);
			});

			xhrMultiPart.Post(args);
		} else {
			args.load = dojo.hitch(this, function(response, ioArgs){
				var responseContentType = ioArgs.xhr.getResponseHeader("Content-Type");
				if (responseContentType!=null && responseContentType.indexOf("text/html") != -1 && response!=null&& response.match(/<ERROR_CODE>CAM_PASSPORT_ERROR<\/ERROR_CODE>/)!=null) {
					this.handlePassportErrorFromDispatcher();
				} else {
					this.loadContent(response);
				}
			});
			dojo.xhrPost(args);
		}
	},

	loadStylesAndScripts: function(sHTML, oReportDiv) {
		window.gScriptLoader.m_bIgnoreAjaxWarnings = true;

		var domElement = this.iContext.getRootElement();
		if (!domElement)
		{
			return;
		}

		sHTML = window.gScriptLoader.loadCSS(sHTML, oReportDiv, true, this.getViewerId());

		if (dojo.isIE)
		{
			// IE specific 'fix' where if sHTML has script tags at the beginning they won't be loaded into the DOM.
			// Adding a valid tag first causes all subsequent scripts to be loaded into the DOM.
			sHTML = "<span style='display:none'>&nbsp;</span>" + sHTML;
		}
		oReportDiv.innerHTML = sHTML;

		var callback = GUtil.generateCallback(this.fireResizeEvent, [], this);
		window.gScriptLoader.loadAll(sHTML, oReportDiv, callback, this.getViewerId());
	},

	/**
	* A function to be used as a call back to fire a resize event
	*/
	fireResizeEvent: function ()
	{
		var viewerObject = this.getViewerObject();
		if (viewerObject == null) {
			this.iContext.iEvents.fireEvent("com.ibm.bux.widget.render.done", null, {});
		}
		else {
			viewerObject.m_resizeReady = true;
			if (viewerObject.doneLoading) {
				viewerObject.doneLoading();
			}
		}
    },
	/*
	 * We need to reconstruct the saved report search path.  If we don't we could be referencing the wrong
	 * report as in the case when the dashboard is copied in CC.
	 */
	constructSavedReportSearchPath: function(sSavedReportName) {
		var sDashboardId = this.getAttributeValue("dashboardID");
		sDashboardId = sDashboardId.substr("$$dbid$$".length );
		var savedReportSearchPath = "storeID(\"" + sDashboardId + "\")/*[@name=" + xpath_attr_encode(sSavedReportName) +"]";
		return savedReportSearchPath;
	},

	isSavedReport: function( sOriginalItem, sSavedItem ) {
		return (	typeof sSavedItem !== "undefined" &&
					sSavedItem !== null &&
					sSavedItem.length > 0 &&
					sOriginalItem !== sSavedItem );
	},

	getGlobalPromptsInfo : function() {
		var result = null;
		var oCV = null;
		var promptsInfo = [];
		var cvPromptInfo = null;

		if (window.gaRV_INSTANCES) {
			for (var iIndex=0; iIndex < window.gaRV_INSTANCES.length; iIndex++)	{
				oCV = window.gaRV_INSTANCES[iIndex];
				cvPromptInfo = oCV.getPromptParametersInfo();
				if (cvPromptInfo != null) {
					promptsInfo.push(cvPromptInfo);
				}
			}
		}
		if (promptsInfo.length > 0) {
			result = "<globalPromptInfo>" + promptsInfo.join("")+ "</globalPromptInfo>";
		}
		return result;
	},

	buildPostUrl: function () {
		return this.getAttributeValue("gateway");
	},

	buildPostContent: function (filters) {
		var sOriginalItem = this.getAttributeValue("originalReport");
		var sSavedItem = this.getAttributeValue("savedReportName");
		var sOriginalReportPart = this.getAttributeValue("originalReportPart");
		var sFlexParamProperties = this.getAttributeValue("flexParam_properties");
		var sFlexPromptData = this.getAttributeValue("flexParam_promptData");

		var bIsSavedReport = this.isSavedReport( sOriginalItem, sSavedItem);
		var bPromptUser = this.getPromptRunOption(sFlexPromptData, bIsSavedReport);
		var content = {
			"b_action" : "cognosViewer",
			"run.outputFormat" : "HTML",
			"bux" : "true",
			"cv.responseFormat" : "iWidget",
			"cv.id" : this.getViewerId(),
			"cv.gateway" : this.getAttributeValue("gateway"),
			"cv.webcontent" : this.getAttributeValue("webcontent")
		};

		this.sInitialSpec = this.getAttributeValue("specification");

		/*
		 * If there is no saved report, the saved report name  is the same as the original report
		 */
		if (!this.sInitialSpec) {
			if ( bIsSavedReport ) {
				content["ui.object"] = this.constructSavedReportSearchPath( sSavedItem );
				content["savedReportName"] = sSavedItem;
				content["ui.reRunObj"] = this.constructSavedReportSearchPath( sSavedItem );
			} else {
				content["ui.object"] = sOriginalItem;

				//check for G!D properties to upgrade
				if( sFlexParamProperties && sFlexParamProperties.length > 0)
				{
					content["flexParamProperties"] = sFlexParamProperties;
				}

				if( sFlexPromptData && sFlexPromptData.length > 0)
				{
					content["gdPromptAnswers"] = sFlexPromptData;
				}//end of G!D properties check
			}

			content["originalReport"] = sOriginalItem;
		}

		// widget saved to specific output
		if (this.getAttributeValue("savedOutputSearchPath") != null) {
			this.setSavedOutputSearchPath(this.getAttributeValue("savedOutputSearchPath"));
			content["ui.savedOutputSearchPath"] = this.getAttributeValue("savedOutputSearchPath");
			content["ui.action"] = "buxView";
		}
		// widget saved to view most recent output
		else if (this.getAttributeValue("mostRecentSavedOutput") == "true") {
			content["ui.action"] = "buxView";
		}
		// we have a copied report under the dashboard
		else if (bIsSavedReport) {
			if( this.getAttributeValue( "limitedInteractiveMode" ) === "true" ) {
				content["limitedInteractiveMode"] = "true";
				content["ui.action"] = "run";
				content["run.xslURL"] = "bux.xsl";
			}
			else
			{
				content["ui.action"] = "bux";
			}
		}
		// user dragged a report part to the canvas
		else if (sOriginalReportPart != null && typeof sOriginalReportPart != "undefined" && sOriginalReportPart.length > 0) {
			content["ui.reportDrop"] = "true";
			content["ui.action"] = "bux";
			content["reportpart_id"] = sOriginalReportPart;
		}
		// specification passed in from container
		else if (this.sInitialSpec) {
			content["ui.spec"] = this.sInitialSpec;
			content["ui.action"] = "buxRunSpecification";
			content["cv.objectPermissions"] = "read write execute setPolicy traverse";
		}
		// user dragged a report to the canvas
		else {
			content["ui.reportDrop"] = "true";
			content["ui.action"] = "buxDropReportOnCanvas";
		}

		if( bPromptUser !== null )
		{
			content["run.prompt"] = (bPromptUser ? "true" : "false");
		}

		if (this.getAttributeValue("lastLocaleSaved") != null) {
			content["lastLocaleSaved"] = this.getAttributeValue("lastLocaleSaved");
		}
		var dataFiltersParam = this.buildDataFiltersParam(filters);
		if (typeof filters=="undefined" || filters==null) {
			dataFiltersParam = this.loadSliderState();
		}
		if(dataFiltersParam != "")
		{
			content["cv.updateDataFilters"] = dataFiltersParam;
		}

		if (this.properties && this.properties.getFlashCharts() !== null) {
			content["savedFlashChartOption"] = (this.properties.getFlashCharts() ? "true" : "false");
			var isAVS = this.getAttributeValue("hasAVSChart");
			if (typeof isAVS != "undefined" && isAVS != null && isAVS == "true") {
				content["hasAVSChart"] = "true";
			}
			else {
				content["hasAVSChart"] = "false";
			}
		}

		if (this.properties.getRowsPerPage() != null)
		{
			content["run.verticalElements"] = this.properties.getRowsPerPage();
		}

		if ( !bIsSavedReport ) {
			var globalPrompts = this.getGlobalPromptsInfo();
			if (globalPrompts !== null ) {
				content["widget.globalPromptInfo"] = globalPrompts;
			}
		}
		if (this.isOpeningSavedDashboard()) {
			content["ui.preserveRapTags"] = "true";
		}
		if( bIsSavedReport === true && this.getAttributeValue( "limitedInteractiveMode" ) !== "true" )
		{
			this.getUserId();
		}
		return content;
	},

	getUserId : function()
	{
		// get user account info
		var dfd = bux.Helper.getUserAccount();
		dfd.addCallback( dojo.hitch( this, function(userAccount){
											this.m_sUserId = userAccount.cm$searchPath;
											return this.m_sUserId; }
						));
	},

	isBuxAvailable: function()
	{
		return ((typeof this.bux != "undefined") && (this.bux != null) &&
				(typeof this.bux.Helper != "undefined") && (this.bux.Helper != null));
	},

	getPromptRunOption: function( sFlexPromptData, bIsSavedReport )
	{
		if( !bIsSavedReport && sFlexPromptData && sFlexPromptData.length > 0)
		{
			//don't prompt if upgrading G!D with saved prompt answers
			return false;
		}
		else if( bIsSavedReport )
		{
			//this is a saved report, use the saved property values
			return this.properties.getPromptUserOnLoad();
		}
		else
		{
			return null;
		}
	},
	buildDataFiltersParam: function(filters) {
		var updateDataFilters = "";
		if(typeof filters != "undefined" && filters != null && filters.length > 0)
		{
			this.sliderState={};
			updateDataFilters = "<UpdateDataFilters>";
			for(var filter = 0; filter < filters.length; ++filter)
			{
				this.sliderState[filters[filter].clientId] = filters[filter];
				updateDataFilters += "<filter>" + xml_encode(filters[filter]) + "</filter>";
			}

			updateDataFilters += "</UpdateDataFilters>";
		}

		return updateDataFilters;
	},
	loadSliderState: function() {
		var dataFiltersParam = "";
		this.sliderState={}; //Clear any previous slider state.
		var sliderIDs   = dojo.fromJson(this.getAttributeValue("sliderIDs"));
		if (typeof sliderIDs!="undefined" && sliderIDs!=null) {
			for (var i=0; i<sliderIDs.length; ++i) {
				var sliderID = sliderIDs[i];
				var savedState = this.getItemValue("sliderState_" + sliderID);
				if (savedState!=null && savedState!="") {
					this.sliderState[sliderID] = savedState;
				}
			}
			dataFiltersParam = this.buildDataFiltersParam_fromState();
		}
		return dataFiltersParam;
	},
	buildDataFiltersParam_fromState: function() {
		var updateDataFilters = "";
		if(typeof this.sliderState != "undefined" && this.sliderState != null) {
			for(var sliderID in this.sliderState)	{
				updateDataFilters += "<filter>" + xml_encode(this.sliderState[sliderID]) + "</filter>";
			}
			if (updateDataFilters!="") {
				return ("<UpdateDataFilters>" + updateDataFilters + "</UpdateDataFilters>");
			}
		}
		return updateDataFilters;
	},
	loadContent: function(response) {
		var oReportDiv = dojo.byId("_" + this.iContext.widgetId + "_cv");
		this.loadStylesAndScripts(response, oReportDiv);
		if (!window.gScriptLoader.hasCompletedExecution()) {
			var obj = this;
			this.timer = setInterval(function(){
				window.gInitializeViewer(obj);
			}, this.timerInterval);
		}
		else {
			window.gInitializeViewer(this);
		}
		this.viewerWidgetLastAction = null;
	},
	executeAction: function(evt, action) {
		var viewerObject = this.getViewerObject();
		if( evt && viewerObject ) {
			var sAction = action ? action : evt.payload.name;
			var payload = evt.payload;
			// payload = '' seems valid ???
			if(sAction && typeof payload != "undefined" && payload != null) {
				if(evt.payload.payload) {
					payload = evt.payload.payload;
				}
				viewerObject.executeAction(sAction, payload);
			}
		}
	},

	onContextMenuAction: function(evt) {
		this.executeAction(evt);
	},
	onToolbarAction: function(evt) {
		this.executeAction(evt);
	},

	updateToolbarSection: function(actionFactory, toolbarPayload) {
		for(var toolbarItem = 0; toolbarItem < toolbarPayload.length; ++toolbarItem)
		{
			var name = toolbarPayload[toolbarItem]["name"];
			if (typeof name != "undefined" && name != null) {
				var action = actionFactory.load(name);
				if (action != null && typeof action != "undefined") {
					var updatedToolbarItem = action.updateMenu(toolbarPayload[toolbarItem]);
					if (updatedToolbarItem.visible == false) {
						toolbarPayload.splice(toolbarItem, 1);
						--toolbarItem;
					} else {
						toolbarPayload[toolbarItem] = updatedToolbarItem;
					}
				}
			} else if (typeof toolbarPayload[toolbarItem]._root != "undefined") {
				this.updateToolbarSection(actionFactory, toolbarPayload[toolbarItem]._root);
			} else if (toolbarPayload[toolbarItem].separator) {
				if (toolbarItem == 0 || (toolbarItem > 0 && toolbarPayload[toolbarItem - 1].separator) || toolbarItem == toolbarPayload.length)	{
					toolbarPayload.splice(toolbarItem, 1);
					--toolbarItem;
				}
			}
		}
	},

	updateToolbar: function() {
		var viewerObject = this.getViewerObject();
		if( viewerObject.getStatus() === 'prompting')
		{
			return;
		}

		var actionFactory = viewerObject.getActionFactory();

		var toolbarPayload = viewerObject.buxToolbarPayload;

		this.updateToolbarSection(actionFactory, toolbarPayload);

		this.iContext.iEvents.fireEvent("com.ibm.bux.widgetchrome.toolbar.init", null, viewerObject.buxToolbarPayload);
	},
	hideToolbarAndContextMenu: function()
	{
		this.iContext.iEvents.fireEvent("com.ibm.bux.widget.contextMenu.update");
		this.iContext.iEvents.fireEvent("com.ibm.bux.widgetchrome.toolbar.init");
	},

	updateSavedAttributes: function( attribType )
	{
		this.setAttributeValue( "version", this.BUX_REPORT_VERSION );
		this.setAttributeValue( "viewerProperties", this.properties.toString() );
		//saving attributes specific to live report
		if( attribType === 'live' )
		{
			var reportPrompts = this.getEnvParam("reportPrompts");
			if ( this.promptParametersRetrieved == true && reportPrompts && reportPrompts.length > 0 ) {
				this.setAttributeValue( "viewerReportPrompts", this.getEnvParam("reportPrompts"));
			}
			this.setAttributeValue( "promptParametersRetrieved", (this.promptParametersRetrieved ? "true" : "false") );
		}

		//saving attributes sepecific to saved output
		if( attribType === 'savedOutput')
		{
			if (this.getSavedOutputSearchPath() == null) {
				this.setAttributeValue("mostRecentSavedOutput", "true");
			}
			else {
				this.setAttributeValue( "savedOutputSearchPath", this.getSavedOutputSearchPath());
			}
		}
	},

	onWidgetSave: function(evt) {
		var viewerObject = this.getViewerObject();
		var saveDonePayload = {'status':false};
		if( !viewerObject || viewerObject.getStatus() === 'fault')
		{
			this.iContext.iEvents.fireEvent( "com.ibm.bux.widget.save.done", null, saveDonePayload);
			return;
		}

		if( this.isSavedOutput() )
		{
			this.cleanSavedAttributes();
			this.updateSavedAttributes( "savedOutput" );
			saveDonePayload = {'status':true};
			this.iContext.iEvents.fireEvent( "com.ibm.bux.widget.save.done", null, saveDonePayload);
		}
		else if(this.sInitialSpec)
		{
			// just update the attribute with the latest spec
			saveDonePayload = {'status':true};
			this.iContext.iEvents.fireEvent( "com.ibm.bux.widget.save.done", null, saveDonePayload);
		}
		else
		{
			var saveReport = new CCognosViewerSaveReport( viewerObject, evt.payload );
			var limitedInteractiveMode = this.isLimitedInteractiveMode();
			if( !saveReport.canSave( this.getEnvParam("cv.objectPermissions") ) )
			{
				this.showErrorMessage( RV_RES.IDS_CANNOT_SAVE_CONTENT_ERROR );
				this.iContext.iEvents.fireEvent( "com.ibm.bux.widget.save.done", null, saveDonePayload);
				return;
			}

			/**
			 * Make sure we have a report spec or that we're in limited mode before trying to create a report/reportView under
			 * the dashboard in CM. We won't have a report spec if a report is still running when the user tries to save the dashboard
			 * bug COGCQ00277254
			 */
			if (limitedInteractiveMode || (viewerObject.envParams["ui.spec"] && viewerObject.envParams["ui.spec"].length > 0)) {
				saveReport.prepareRequest( evt.payload );
				saveReport.sendRequest(evt.payload );
			} else {
				this.iContext.iEvents.fireEvent( "com.ibm.bux.widget.save.done", null, saveDonePayload);
			}
		}

	},

	/*
	 * Return true if we are opening/upgrading a go dashboard version report.
	 */
	isGoDashboardReportUpgrade: function() {
		var contentVersion = this.getAttributeValue( "version" );
		if (contentVersion==this.GODASHBOARD_REPORT_VERSION) {
			//This is a G!D report being upgraded
			return true;
		}
		return false;
	},

	/*
	 * Return true if we are opening/upgrading a saved dashboard.
	 */
	isOpeningSavedDashboard: function() {
		var viewerProperties = this.getAttributeValue( "viewerProperties");
		//Once saved, viewer properties will be set as a JSON object.
		//Prior to that, they will be <empty> or not defined.
		if ((typeof viewerProperties != "undefined" && viewerProperties!=null && viewerProperties.indexOf("{")==0)
			|| this.isGoDashboardReportUpgrade()) {
			return true;
		}
		return false;
	},

	/*
	 * Reset all the attributes we save so there's no issue when switching from
	 * a live report and a saved output
	 */
	cleanSavedAttributes: function() {
		this.removeAttributeValue("savedOutputSearchPath");
		this.removeAttributeValue("mostRecentSavedOutput");
		this.removeAttributeValue("savedReportName");
		this.removeAttributeValue("dashboardID");
		this.removeAttributeValue("savedReportId");
		this.removeAttributeValue("baseReportModificationTime");
		this.removeAttributeValue("viewerReportPrompts");
		this.removeAttributeValue("promptParametersRetrieved");
	},


	handleWidgetSaveDone: function( asynchJSONResponse ) {
		var oCV = this.getViewerObject();
		var jsonResult = asynchJSONResponse.getResult();
		if (jsonResult===null) {
			jsonResult = {};
		}

		this.cleanSavedAttributes();
		this.updateSavedAttributes( "live" );

		var reportName = jsonResult.reportName;
		var reportId = jsonResult.reportId;
		var dashboardID = jsonResult.dashboard_id;
		this.setAttributeValue( "savedReportName", reportName );
		this.setAttributeValue("dashboardID", '$$dbid$$' + dashboardID );
		this.setAttributeValue( "savedReportId", reportId );

		if (jsonResult.permissions) {
			oCV.envParams["cv.objectPermissions"] = jsonResult.permissions;
		}

		if (jsonResult.objectClass) {
			oCV.envParams["ui.objectClass"] = jsonResult.objectClass;
		}

		// flag needed to disable the snapshots menu on a dashboard open
		if (this.getEnvParam("reportpart_id") && this.getEnvParam("reportpart_id").length > 0)
		{
			this.setAttributeValue("fromReportPart", "true");
		}

		if( this.isLimitedInteractiveMode() )
		{
			this.setAttributeValue( "limitedInteractiveMode", "true" );
		}
		 else
		{
			this.setAttributeValue( "baseReportModificationTime", oCV.envParams["baseReportModificationTime"]);
		}

		var contentLocale = oCV.envParams["contentLocale"];
		if (contentLocale != null) {
			this.saveSliderState();
			this.setAttributeValue( "lastLocaleSaved", contentLocale );
		}

		this.setAttributeValue( "originalReportLocation", oCV.envParams["originalReportLocation"]);

		oCV.envParams["ui.object"] = this.constructSavedReportSearchPath( reportName );
		oCV.envParams["savedReportName"] = reportName;
		oCV.envParams["savedReportId"] = reportId;

		this.setAttributeValue("hasAVSChart", oCV.hasAVSChart() ? "true" : "false");
		// clear the cached available outputs
		this.setSavedOutputsCMResponse(null);

		var saveDonePayload = {'status':true};
		this.iContext.iEvents.fireEvent( "com.ibm.bux.widget.save.done", null, saveDonePayload );

	},

	saveSliderState: function () {
		//Save slider events that correspond to reportInfo....remove the rest.
		var viewerObject = this.getViewerObject();
		var reportInfoSliders = viewerObject.collectSliderSetFromReportInfo();
		var i=0;
		var sliderIDsToSave = [];
		for(var sliderIDFromState in this.sliderState) {
			if (typeof reportInfoSliders[sliderIDFromState] != "undefined") {
				this.setItemValue("sliderState_" + sliderIDFromState, this.sliderState[sliderIDFromState]);
				sliderIDsToSave[i++]=sliderIDFromState;
			} else {
				this.removeItem("sliderState_" + sliderIDFromState);
			}
		}
		this.setAttributeValue("sliderIDs", dojo.toJson(sliderIDsToSave));
	},
	onWidgetRefresh: function ( evt ){
		//TODO: Payload.status and Payload.message for widget.refresh.done
		//      It is not necessary for now
		var refreshDonePayload = {};
		var viewerObject = this.getViewerObject();
		if( evt != null && typeof evt != "undefined" && viewerObject != null && typeof viewerObject != "undefined"){
			if (viewerObject.envParams["ui.action"] == "view") {
				viewerObject.executeAction( "RefreshViewEvent" );
			} else {
				if ((typeof evt.payload != "undefined") && evt.payload != null && evt.payload.refreshType == "preferencesChanged") {
					this.clearRAPCache();
					viewerObject.executeAction( "EditContent", {"preferencesChanged" : true} );
					viewerObject.executeAction( "Reprompt", {"preferencesChanged" : true} );
				} else {
					viewerObject.executeAction( "Refresh" );
				}
			}
			this.iContext.iEvents.fireEvent( "com.ibm.bux.widget.refresh.done", null, refreshDonePayload);
		}
	},

	invokeDisplayTypeDialog: function( targetTypes )
	{
		if( !this.m_chart )
		{
			this.m_chart = new bux.reportViewer.chart( );
		}

		var dialogDefinition = this.m_chart.getDisplayTypeDialogDefinition( targetTypes );

		var payload= { 'supportedChartTypes' : dialogDefinition };
		this.iContext.iEvents.fireEvent("com.ibm.bux.widget.invokeDisplayTypeDialog", null, payload );
	},

	onChangeDisplayType: function(evt) {
		var viewerObject = this.getViewerObject();
		if(evt != null && typeof evt != "undefined" && viewerObject != null && typeof viewerObject != "undefined") {
			viewerObject.executeAction( "ChangeDisplayType", evt.payload );
		}
	},

	onGetDisplayTitleDone: function(evt) {
		if(evt != null && typeof evt != "undefined" && evt.payload.callback) {
			evt.payload.callback(evt.payload.title);
		}
	},

	getWidgetStoreID: function()
	{
		var storeID = this.getAttributeValue( "widgetStoreID");
		if( storeID && storeID.indexOf( 'storeID' ) !== -1)
		{
			var begIndex = 'storeID("'.length;
			var endIndex = storeID.length - '")'.length;
			return storeID.substring( begIndex, endIndex );
		}

		return storeID;
	},

	onUpdateWidgetStoreIDDone: function(evt) {
		var widgetStoreID = evt.payload.storeID;

		var widgetOldStoreID = this.getWidgetStoreID();

		if(widgetOldStoreID != null && widgetOldStoreID !== "" && widgetOldStoreID != widgetStoreID) {
			var viewerObject = this.getViewerObject();
			if (viewerObject) {
				//clone the annotations
				var httpRequest = new XmlHttpObject();
				httpRequest.setCallbacks(function(){}, function(){});

				httpRequest.addFormField("b_action", "cognosViewer");
				httpRequest.addFormField("widgetOldStoreID", widgetOldStoreID);
				httpRequest.addFormField("widgetStoreID", widgetStoreID);
				httpRequest.addFormField("ui.action", "cloneAnnotations");
				httpRequest.addFormField("cv.responseFormat", "json");

				//send asynchronous request
				httpRequest.sendHtmlRequest("POST", viewerObject.getGateway(), "", true);
			}
			else if (console) {
				console.error("Failled to update Widget Store IDs: viewerObject is NULL.");
			}
		}

		if( widgetStoreID && (widgetStoreID.indexOf( 'storeID("') === -1) )
		{
			this.setAttributeValue( "widgetStoreID", 'storeID("' + widgetStoreID + '")' );
		}
		else
		{
			this.setAttributeValue( "widgetStoreID", widgetStoreID );
		}
	},

	onRemoveDone: function(evt) {
		this.destroy();
	},
	onGetProperties : function (evt)
	{
		this.initializeProperties();
		this.iContext.iEvents.fireEvent("com.ibm.bux.widget.properties.get.done", null, this.properties.onGet() );
	},

	onSetProperties : function (evt)
	{
		if( evt && evt.payload && evt.payload.properties )
		{
			this.properties.onSet( evt.payload );
		}

		this.fireEvent("com.ibm.bux.widget.properties.set.done");
		var modifiedEventPayload = {'modified':true};
		this.fireEvent("com.ibm.bux.widget.modified", null, modifiedEventPayload);
	},

	getProperties : function ()
	{
		return this.properties;
	},

	clearPropertiesDialog : function() {
		if (this.properties) {
			this.properties.clearDialogSpec();
		}
	},

	onFilter: function(evt) {
		var viewerObject = this.getViewerObject();
		if( evt != null && typeof evt != "undefined" && viewerObject != null && typeof viewerObject != "undefined" && this.canFilter(evt)) {
			this.sliderState[evt.source] = evt.payload;
			viewerObject.executeAction("UpdateDataFilter", evt.payload);
		}
	},
	canFilter: function(evt) {
		var index;
		if( this.isReportInteractive() === true) {
			var filterPayload = dojo.fromJson(evt.payload);
			var filterOnDataItem = filterPayload.name;
			var filterableItems = this.getRAPCachedItem("filterableItems");
			if(typeof filterOnDataItem != "undefined" && filterOnDataItem != null && filterableItems != null) {
				for(var type in filterableItems) { // visible/invisible
					for(index = 0; index < filterableItems[type].length; ++index) {
						if(filterableItems[type][index].name == filterOnDataItem ||
							filterableItems[type][index].label == filterOnDataItem) {
							return true;
						}
					}
				}
			}
			else
			{
				var viewerObject = this.getViewerObject();
				var reportInfo = viewerObject.envParams["rap.reportInfo"];
				if (reportInfo && reportInfo.length) {
					for(index = 0; index < reportInfo.length; ++index) {
						var container = reportInfo[index];
						var sliders = container.sliders;
						if(sliders && sliders.length) {
							for(var sliderIndex = 0; sliderIndex < sliders.length; ++sliderIndex) {
								var oSlider = sliders[sliderIndex];
								if(oSlider.clientId == filterPayload.clientId ||
								   (oSlider.clientId==oSlider.name && oSlider.name==filterPayload.name)) {
									return true;
								}
							}
						}
						var itemInfo = container.itemInfo;
						if (itemInfo && itemInfo.length) {
							for(var itemInfoIndex = 0; itemInfoIndex < itemInfo.length; ++itemInfoIndex) {
								if(itemInfo[itemInfoIndex].item == filterPayload.name) {
									return true;
								}
							}
						}
					}
				}
			}
		}
		return false;
	},
	isLimitedInteractiveMode: function() {
		return this.getViewerObject().isLimitedInteractiveMode();
	},
	onMouseOver: function(evt) {
		if(DragDropAction_isDragging(evt) == false && this.getViewerObject().getStatus() == "complete")
		{
			var action = ActionFactory_loadActionHandler(evt, this.getViewerObject());
			if(action != null && typeof action != "undefined")
			{
				return action.onMouseOver(evt);
			}
		}
		return true;
	},
	onMouseOut: function(evt) {
		var action = ActionFactory_loadActionHandler(evt, this.getViewerObject());
		if(action != null && typeof action != "undefined" && this.getViewerObject().getStatus() == "complete")
		{
			return action.onMouseOut(evt);
		}
		return true;
	},
	onMouseOverRenamedItem: function(evt) {
		var node = getNodeFromEvent(evt);
		if (node.getAttribute)
		{
			var originalLabel = node.getAttribute("rp_name");
			if(originalLabel != null) {
				var id = node.getAttribute("id");
				node.m_tooltip = new bux.reportViewer.ReportInfo(
				{
					connectId: [id],
					position: ["above","below"],
					title: RV_RES.IDS_JS_RENAMED_ITEM,
					text: RV_RES.IDS_JS_RENAMED_ITEM_ORIGINAL_LABEL + " " + node.getAttribute("rp_name")
				});
				node.m_tooltip.open();
			}
		}
	},
	onMouseOutRenamedItem: function(evt) {
		var node = getNodeFromEvent(evt);
		node.m_tooltip = null;
	},
	onDoubleClick: function(evt) {
		// Double click event is used mostly to perform the default drill.
		var action = ActionFactory_loadActionHandler(evt, this.getViewerObject());
		if(action != null && typeof action != "undefined" && this.getViewerObject().getStatus() == "complete")
		{
			return action.onDoubleClick(evt);
		}
		return true;
	},
	onMouseDown: function(evt) {
		var factory;
		var oCV = this.getViewerObject();
		if (oCV.envParams["ui.action"] == 'view') {
			factory = oCV.getActionFactory();
			factory.load("Selection").onMouseDown(evt);
		}
		else if(oCV.getStatus() == "complete") {
			factory = oCV.getActionFactory();
			if((evt.button == 1 || evt.button == 0) && !this.isLimitedInteractiveMode()) {
				// if the onMouseDown returns true, then we executed a drillThrough and shouldn't be doing a drag/drop action
				// if the onMouseDown for rename returns true then the user clicked in the rename edit box so don't start a drag/drop
				if (!factory.load("Selection").onMouseDown(evt) && !factory.load("RenameDataItem").onMouseDown(evt)) {
					this.initDragAndDrop();
					factory.load("DragDrop").onMouseDown(evt);
				}
			} else {
				factory.load("Selection").onMouseDown(evt);
			}
		}
	},
	onMouseUp: function(evt) {
		var srcNode = evt.originalTarget ? evt.originalTarget : evt.srcElement;

		// if the event did come from an INPUT or TEXTAREA element cancel the bubble,
		// to prevent lost of focus in IE
		var nodeName = srcNode.nodeName.toLowerCase();
		if (nodeName == "input" || nodeName == "textarea") {
				evt.cancelBubble = true;
				return false;
		}
		var eventConsumed = false;
		if (this.getViewerObject().envParams["ui.action"] == 'view') {
			setNodeFocus(evt);
		}
		else if (this.getViewerObject().getStatus() == "complete") {
			eventConsumed |= DragDropAction_isDragging(evt);
				DragDropAction_onmouseup(evt);
				this.removeDragAndDrop();
				setNodeFocus(evt);
			}
		this.getViewerObject().getActionFactory().load("Selection").onMouseUp(evt, eventConsumed);
		clearTextSelection();
	},
	onContextMenu: function(evt) {
		var viewerObject = this.getViewerObject();
		if( viewerObject.getStatus() == "complete" || this.isSavedOutput() ) {
			var actionFactory = viewerObject.getActionFactory();
			var contextMenuPayload = viewerObject.buxContextMenuPayload;

			// we don't want to modify the original context menu spec, so create a temp object
			// TODO: Get a way to simply hide a menu item instead of setting the json for it to empty.
			var tempContextMenu = [];
			for(var contextMenuItem = 0; contextMenuItem < contextMenuPayload.length; ++contextMenuItem)
			{
				var menuItem = contextMenuPayload[contextMenuItem];
				var name = contextMenuPayload[contextMenuItem]["name"];
				if (typeof name != "undefined") {
					var action = actionFactory.load(name);

					if(action != null && typeof action != "undefined")
					{
						if(typeof action.buildMenu == "function") {
							//TODO context menus don't support callbacks, once the support is in, we can call updateMenu as we normally do, for now
							menuItem = action.buildMenu(contextMenuPayload[contextMenuItem]);
						} else {
							menuItem = action.updateMenu(contextMenuPayload[contextMenuItem]);
						}
					}
				}

				if (typeof menuItem != "undefined" && menuItem != "" && menuItem.visible != false) {
					// only add the separator if the previous menuItem wasn't a separator
					if (menuItem.separator == true) {
						if (tempContextMenu.length > 0 && typeof tempContextMenu[tempContextMenu.length-1].separator == "undefined") {
							tempContextMenu[tempContextMenu.length] = menuItem;
						}
					}
					else {
						tempContextMenu[tempContextMenu.length] = menuItem;
					}
				}
			}

			this.iContext.iEvents.fireEvent("com.ibm.bux.widget.contextMenu.update", null, tempContextMenu);
		}
	},

	initDragAndDrop: function() {
		if (window.attachEvent)
		{
			document.attachEvent("onmouseup", DragDropAction_onmouseup);
			document.attachEvent("onmousemove", DragDropAction_onmousemove);
		}
		else if (document.addEventListener)
		{
			document.addEventListener("mouseup", DragDropAction_onmouseup, false);
			document.addEventListener("mousemove", DragDropAction_onmousemove, false);
		}
	},

	removeDragAndDrop: function() {
		if (window.detachEvent)
		{
			document.detachEvent("onmouseup", DragDropAction_onmouseup);
			document.detachEvent("onmousemove", DragDropAction_onmousemove);
		}
		else if (document.removeEventListener)
		{
			document.removeEventListener("mouseup", DragDropAction_onmouseup, false);
			document.removeEventListener("mousemove", DragDropAction_onmousemove, false);
		}
	},

	onWidgetResize: function(evt) {
		var resize = evt.payload.resize;
		if (typeof resize != "undefined" && resize != null) {
			this.executeAction(evt, "ResizeChart");

			if (this.getViewerObject() && this.getViewerObject().getVisibleDialog() != null && typeof this.getViewerObject().getVisibleDialog().widgetResizeEvent == "function") {
				this.getViewerObject().getVisibleDialog().widgetResizeEvent();
			}
		}
	},

	fetchAnnotationData: function()
	{
		var widgetStoreID = this.getWidgetStoreID();
		if (widgetStoreID)
		{
			var cognosViewer = this.getViewerObject();
			var selCon = cognosViewer.getSelectionController();
			var metadata = dojo.toJson(selCon.getCCDManager().m_md);
			var contextData =  dojo.toJson(selCon.getCCDManager().m_cd);

			if (metadata!=null && contextData!=null ) {
				var callbacks = {"complete":{}};
				callbacks.complete["object"] = this;
				callbacks.complete["method"] = this.fetchAnnotationDataResponse;

				var asynchRequest = new AsynchJSONRequest(cognosViewer.getGateway(), cognosViewer.getWebContentRoot(), callbacks);

				asynchRequest.setPromptDialog(new BuxPromptDialog(cognosViewer.getId()));
				asynchRequest.setLogonDialog(new BuxLogOnDialog());
				asynchRequest.setFaultDialog(new BuxFaultDialog());

				asynchRequest.addFormField("b_action", "cognosViewer");
				asynchRequest.addFormField("widgetStoreID", widgetStoreID);
				asynchRequest.addFormField("ui.action", "getAnnotations");
				asynchRequest.addFormField("cv.responseFormat", "asynchJSON");
				asynchRequest.addFormField("cv.metatdata", metadata);
				asynchRequest.addFormField("cv.context", contextData);

				asynchRequest.sendRequest();
			}
		}
	},

	fetchAnnotationDataResponse: function(asynchJSONResponse)
	{
		var jsonResult = asynchJSONResponse.getResult();
		if (jsonResult===null) {
			jsonResult = [];
		}

		var viewerObject = this.getViewerObject();
		viewerObject.envParams["cv.annotationData"] = jsonResult;

		this._cleanupPreviousAnnotations();

		if (jsonResult.length > 0) {
			if (this.annotationStore) {
				var annotationArray = this._generateMatchedAnnotationsFromViewerObject();
				this.annotationStore.load(annotationArray);
				this.displayCommentIndicators();
			}
		}
		this.updateToolbar();
	},

	/*
	 * destroy all commentViewer widget for previous annotations
	 */
	_cleanupPreviousAnnotations: function () {
		var store = this.getAnnotationStore();
		if (store) {
			var annotations = store.getAll();
			//clean up all annotations
			for (var contextId in annotations) {
				var commentId = this.sViewerIWidgetId + "_" + contextId + "_comment";

				var commentViewer = dijit.byId(commentId);
				if (typeof commentViewer != "undefined" && commentViewer != null) {
					commentViewer.destroy();
					delete commentViewer;
				}
				store.clear(contextId);
			}
		}
	},

	/*
	 * Returns array of annotations. Ctx of an annotation is updated with the explicit ctx value of the report cell the annotation is on.
	 * In order to update ctx value, it does the following.
	 *
	 * - remove any OR (|) operator in ctxValue
	 * - find a matching report cell and take ctx value of it
	 *
	 */
	_generateMatchedAnnotationsFromViewerObject: function () {
		var viewerObject = this.getViewerObject();
		var annotationData = [];
		if (viewerObject && viewerObject.envParams && viewerObject.envParams["cv.annotationData"]) {
			annotationData = eval(viewerObject.envParams["cv.annotationData"]);
		}

		var annotationListWithUniqueCTX = this._generateAnnotationListWithoutORoperator(annotationData);
		return this._updateAnnotationCtxWithMatchedCellCtx(annotationListWithUniqueCTX, viewerObject);
	},

	/*
	 * ctx of an annotation may have 'OR' operator.
	 * for example, 9:3|4:*
	 * It means that annotated cell's ctx could be either 9:3:* or 9:4:*
	 *
	 * This function removes 'OR' operator and make two annotations with one of each possible ctx value.
	 *
	 */
	_generateAnnotationListWithoutORoperator: function(annotationData) {

		var annotationListWithUniqueCTX = [];
		for (var i=0, len=annotationData.length; i<len; i++ ) {
			var anno = annotationData[i];
			if(anno.ctx) { //cell level annotation
				var oneOrMoreAnno = bux.Helper.generateCTXList(anno.ctx);

				for(var j=0, jLen=oneOrMoreAnno.length; j<jLen; j++) {
					var newAnno = {ann:anno.ann,ct:anno.ct,ctf:anno.ctf,
									ctx:anno.ctx,desc:anno.desc,fd:anno.fd,
									layoutElementId:anno.layoutElementId,owner:anno.owner,pv:anno.pv,
									rdi:anno.rdi,sd:anno.sd	};
					newAnno.ctx = oneOrMoreAnno[j];
					annotationListWithUniqueCTX.splice(0,0, newAnno);
				}
			} else {//widget level annotation
				annotationListWithUniqueCTX.splice(0,0, anno);
			}
		}
		return annotationListWithUniqueCTX;
	},

	/*
	 * Finds the report cell of an annotations, and update ctx of annotation with the cell's ctx
	 * Returns array of annotations which all has explicit ctx value
	 *
	 * In order to that, it does the following
	 *
	 * first, creates AnnotationCTXLookup object, and
	 * calls populateCtxDetailsForAllAnnotations to load ctx details for all Ids in annotation ctx
	 *
	 * then, for each annotation
	 *
	 * - queries dom nodes with ctx attirbute, report cells, if hasn't
	 * - if ctx of an annotation is not present, it is a widget level. put it in the return array
	 * - calls isAnnotatedCell function of AnnotationCTXLookup with annotation and each cell's ctx value.
	 * - only if the function return true,
	 *   updates ctx of annotation with the ctx value of the cell. and put it in the return array
	 *
	 */
	_updateAnnotationCtxWithMatchedCellCtx: function (newAnnotationListWithUniqueCTX, viewerObject) {

		var processedAnnotationList = [];

		var ctxLookup = new AnnotationCTXLookup(this.getViewerObject().getSelectionController().getCCDManager());
		ctxLookup.populateCtxDetailsForAllAnnotations(newAnnotationListWithUniqueCTX);

		var nodes = [];
		for(var i=0, len=newAnnotationListWithUniqueCTX.length; i<len; i++) {
			var anno = newAnnotationListWithUniqueCTX[i];
			if (anno.ctx) {
				if (nodes.length === 0) {
					//query now. we need it for cell level handling
					nodes = dojo.query('[ctx]', this.iContext.getRootElement());
				}

				for (var j = 0, jLen = nodes.length; j < jLen; j++) {
					var cellCtxValue = nodes[j].getAttribute('ctx'); // get ctx attributes from node
					if (ctxLookup.isAnnotatedCell(anno, cellCtxValue)) {
						anno.ctx = cellCtxValue;
						processedAnnotationList.splice(0,0, anno);
						break;
					}
				}
			} else {
				processedAnnotationList.splice(0,0, anno);
			}
		}

		return processedAnnotationList;
	},

	displayCommentIndicators: function () {
		var store = this.annotationStore;
		var commentedCells = store.getAll();

		for (var cell in commentedCells)
		{
			var indicator = new bux.AnnotationIndicator({annotationStore: store, context: cell, iWidgetContainer: this});
			indicator.addExistingCommentToCell();
		}
	},

	promptForSave: function(yesOkFunc) {
		var saveConfirmationDialog = new bux.dialogs.InformationDialog({
			title: RV_RES.IDS_JS_COMMENT_DLG_TITLE,
			sMainMessage: RV_RES.IDS_JS_COMMENT_DLG_MESSAGE,
			sDescription: RV_RES.IDS_JS_COMMENT_DLG_DETAILS,
			sInfoIconClass: 'bux-informationDialog-warning-icon',
			buttons : ['YES','NO'],
			yesOKHandler : yesOkFunc
		});
		saveConfirmationDialog.startup();
		saveConfirmationDialog.show();
	},

	isSaveNecessary: function () {
		var isNecessary = false;

		if (this.isSavedOutput()) {
			var sHasAlreadySaved = this.getAttributeValue("mostRecentSavedOutput");
			var sSavedOutputSearchPath = this.getAttributeValue("savedOutputSearchPath");
			isNecessary = ( sHasAlreadySaved === "true" || sSavedOutputSearchPath != null) ? false : true;
		} else {
			var sOriginalItem = this.getAttributeValue("originalReport");
			var sSavedItem = this.getAttributeValue("savedReportName");
			if (!this.isSavedReport(sOriginalItem, sSavedItem)) {
				isNecessary = true;
			}
		}
		return isNecessary;
	},

	addComment: function (ctxId, cellValue) {
		if (this.isSaveNecessary()) {
			this.promptForSave( dojo.hitch(this, this.onCommentSaveAcknowledge, ctxId, cellValue) );
		} else {
			this.addNewComment(ctxId, cellValue);
		}
	},

	editComment: function (ctxId) {
		var comment = bux.getComment(this.iContext, this.getAnnotationStore().getEffectiveIds(ctxId));
		if (comment && comment.edit)
		{
			comment.edit();
		}
	},

	deleteComment: function (ctxId) {
		this.annotationStore.deleteOneComment(ctxId, this.iContext);
	},

	addNewComment: function(ctxId, cellValue) {
		var indicator = null;

		var contextId = (ctxId===null)? this.getAnnotationStore().WIDGET_CONTEXT: ctxId;
		var indicatorId = this.getAnnotationStore().getIndicatorWidgetId(contextId);

		if (indicatorId!=null) {
			indicator = dijit.byId(indicatorId);
		}

		cellValue = cellValue + ' - ';

		if (indicator==null) {
			indicator = new bux.AnnotationIndicator({
				annotationStore: this.getAnnotationStore(),
				context: contextId,
				iWidgetContainer: this
			});
			indicator.addNewCommentToCell(cellValue);
		} else {
			indicator.comment.editNewComment(contextId, cellValue);
		}
	},

	onWidgetActionDone: function (iEvent) {
		if (iEvent && iEvent.payload && iEvent.payload.status === true) {
			this.addNewComment(this._pendingContextId, this._pendingCellValue);
		}

		this._pendingContextId = null;
		this._pendingCellValue = null;
	},

	onCommentSaveAcknowledge: function(ctxId, cellValue) {
		this._pendingContextId = ctxId;
		this._pendingCellValue = cellValue;
		this.fireEvent('com.ibm.bux.widget.action', null, {action: 'canvas.save'});
	},

	addWidgetComment: function () {
		if (this.isSaveNecessary()) {
			this.promptForSave( dojo.hitch(this, this.onCommentSaveAcknowledge, null, this.getDisplayName()) );
		} else {
			this.addNewComment(null, this.getDisplayName());
		}
	},

	editWidgetComment: function () {
		var comment = bux.getComment(this.iContext, [this.getAnnotationStore().WIDGET_CONTEXT]);
		if (comment && comment.edit)
		{
			comment.edit();
		}
	},

	deleteWidgetComment: function () {
		this.deleteComment(this.getAnnotationStore().WIDGET_CONTEXT);
	},

	commitComment: function(annotation) {

		if (!annotation.isNew) {
			return null;
		}

		if (!annotation.isLoaded) {
			//If it has never been loaded, it's a brand new comment
			if (annotation.getContext() !== this.getAnnotationStore().WIDGET_CONTEXT) {
				return this.commitCellComment(annotation);
			}
			else {
				return this.commitWidgetComment(annotation, this.getAnnotationStore());
			}
		}
		return null;
	},

	preparePostForCommitCellComment: function(asynchRequest, annotation) {

		var widgetStoreID = this.getWidgetStoreID();
		var viewerObject = this.getViewerObject();
		var selCon = viewerObject.getSelectionController();

		asynchRequest.addFormField("b_action", "cognosViewer");
		asynchRequest.addFormField("widgetStoreID", widgetStoreID);
		asynchRequest.addFormField("ui.action", "commentSave");
		asynchRequest.addFormField("cv.responseFormat", "asynchJSON");
		asynchRequest.addFormField("cv.metatdata", dojo.toJson(selCon.getCCDManager().m_md));
		asynchRequest.addFormField("cv.context", dojo.toJson(selCon.getCCDManager().m_cd));
		asynchRequest.addFormField("cv.commentId", annotation.getContext());
		asynchRequest.addFormField("cv.commentDesc", xml_encode(annotation.getDescription()));

		var lid = selCon.getAllSelectedObjects()[0].getLayoutElementId();
		if (lid) {
			//The "lid" attribute is composed of layout ID + namespacePrefix + widget identifier.
			//We only want the layout ID, so remove the rest.
			lid = lid.replace(this.getViewerId(), "");
			annotation.setParentId(lid);
			asynchRequest.addFormField("cv.commentLayoutId", annotation.getParentId());

			var annInfoBar = new AnnotationInfoBar(viewerObject, lid);
			var filterContext = annInfoBar.createFilterContext();
			if (filterContext!=null) {
				annotation.setFilterContext(filterContext);
				if (filterContext.getPromptsJSON()) {
					asynchRequest.addFormField("cv.filterContext_prompts", dojo.toJson(filterContext.getPromptsJSON()));
				}
				if (filterContext.getFilterJSON()) {
					asynchRequest.addFormField("cv.filterContext_filter", dojo.toJson(filterContext.getFilterJSON()));
				}
				if (filterContext.getSlidersJSON()) {
					asynchRequest.addFormField("cv.filterContext_sliders", dojo.toJson(filterContext.getSlidersJSON()));
				}
			}
		}
	},

	commitCellComment: function(annotation) {
		//Fire off a request to Viewer to commit the annotation
		//Note: Edit requests go directly to the annotation service and not through Viewer,
		//so they are not dealt with here

		var deferred = new dojo.Deferred();
		var cognosViewer = this.getViewerObject();
		var callbacks = {customArguments:[annotation, deferred],
						"complete":{},
						"fault": {}};
		callbacks.complete["object"] = this;
		callbacks.complete["method"] = this.handleCellCommentSaveResponse;
		callbacks.fault["object"] = this;
		callbacks.fault["method"] =this.handleCellCommentSaveErrorResponse;

		var asynchRequest = new AsynchJSONRequest(cognosViewer.getGateway(), cognosViewer.getWebContentRoot(), callbacks);

		asynchRequest.setPromptDialog(new BuxPromptDialog(cognosViewer.getId()));
		asynchRequest.setLogonDialog(new BuxLogOnDialog());
		asynchRequest.setFaultDialog(new BuxFaultDialog());

		this.preparePostForCommitCellComment( asynchRequest, annotation );
		asynchRequest.sendRequest();

		return deferred;
	},

	commitWidgetComment: function(annotation) {
			var	serviceURL = bux.Helper.getGateway() + "/annotationService/creation";
			var contentObj = {};

			contentObj["annotationName"] = annotation.getName();
			if (contentObj["annotationName"] == null) {
				//This parameter is necessary to the API -- the call will fail without it
				contentObj["annotationName"] = "";
			}

			contentObj["annotationDescription"] = annotation.getDescription();
			contentObj["annotationParentID"] = this.getWidgetStoreID();

			var dfd = bux.xhrPost({
				url: serviceURL,
				sync: true,
				content: contentObj
			});

			dfd.addCallback( dojo.hitch( this, function (response) {
					return this.handleCommentSaveDone(response, annotation, this.getAnnotationStore(), this);
					}));

			dfd.addCallback( dojo.hitch( annotation, function (response) {
				annotation.loadDetails();
				return response;
			}));

			return dfd;
	},

	handleCellCommentSaveResponse: function(asynchJSONResponse, annotation, deferred)
	{
		var responseObject = asynchJSONResponse.getJSONResponseObject();
		if (responseObject) {
			var jsonResult = responseObject.json;
			if (jsonResult) {
				var result = this.handleCommentSaveDone(jsonResult, annotation, this.getAnnotationStore(), this);
				if (typeof result != 'Error') {
					annotation.loadDetails();
				}
			}
		}
		deferred.callback(asynchJSONResponse);
	},
	handleCellCommentSaveErrorResponse: function(asynchJSONError, annotation, deferred)
	{
		deferred.errback(new Error(asynchJSONError.getSoapFaultDetailMessageString()));
	},

	handleCommentSaveDone: function(response, annotation, store, viewerWidget) {

		var xmlDom = XMLBuilderLoadXMLFromString(response);
		if (xmlDom) {
			var responseNode = xmlDom.firstChild;
			var annotationNode = XMLHelper_FindChildByTagName(responseNode, "annotation", false);
			if (annotationNode) {
				var annotationIdNode = XMLHelper_FindChildByTagName(annotationNode, "id", false);
				if (annotationIdNode) {
					var annotationId = XMLHelper_GetText(annotationIdNode);
					annotation.setServiceId(annotationId);
								annotation.setCreatedInCurrentSession(true);
								annotation.setIWidgetContainer(viewerWidget);
								store.setWidgetContainer(viewerWidget);
								store.addNewAnnotation(annotation.getServiceId());
					return response;
				}
			}
		}
		return new Error(response.toString());
	},


	getAnnotationStore: function() {
		return this.annotationStore;
	},

	fireEvent: function( eventName, arg, payload) {
		this.iContext.iEvents.fireEvent( eventName, arg, payload );
	},

	onGetFilterableItems: function( evt )
	{
		var viewerObject = this.getViewerObject();
		var cachedFilterableItems = this.getRAPCachedItem("filterableItems");
		if( cachedFilterableItems == null && this.isReportInteractive() && (viewerObject != null && viewerObject.getStatus() == "complete")) {
			this.executeAction( evt, "GetFilterableItems" );
		} else {
			var cachedFilterableItemsJsonString = cachedFilterableItems == null ? "{visible:[],hidden:[]}" : dojo.toJson(cachedFilterableItems);
			this.savedOutputResponseHandler("com.ibm.bux.filter.items.get.done", cachedFilterableItemsJsonString);
		}
	},

	onGetFilterValues: function( evt )
	{
		var viewerObject = this.getViewerObject();
		if ( this.isReportInteractive() && (viewerObject != null && viewerObject.getStatus() == "complete") ) {
			//If the source is defined in the evt but not in the payload, transfer it....
			var payload = evt.payload;
			if (typeof evt.source != "undefined" && typeof payload != "undefined") {
				if (typeof payload.payload != "undefined") {
					payload = payload.payload;
				}
				if (typeof payload.source == "undefined") {
					payload.source = evt.source;
				}
			}
			this.executeAction( evt, "GetFilterValues" );
		} else {
			this.savedOutputResponseHandler("com.ibm.bux.filter.values.get.done", "{}");
		}
	},

	savedOutputResponseHandler: function( eventName, emptyPayload)
	{
		var viewerObject = this.getViewerObject();
		var savedOutputResponse = "window[\"oCV" + viewerObject.getId() + "\"].getViewerWidget().fireEvent(\"" + eventName + "\", null, " + emptyPayload + ");";
		if (viewerObject != null) {
			var window_oCV = window["oCV" + viewerObject.getId()];
			if (typeof window_oCV == "object") {
				setTimeout(savedOutputResponse,1);
			}
		}
		return savedOutputResponse;
	},

	handleGetFilterableItemsResponse: function( oResponse )
	{
		var payload = {};
		var json = this.getJsonFromResponse(oResponse);
		if (json) {
			payload = json.filterableItems;
			// cache the filterable items, so we can re-use later for filter actions or in case the slider dialog is re-launched
			this.addToRAPCache("filterableItems", payload);
		}
		this.fireEvent("com.ibm.bux.filter.items.get.done", null, payload);

	},

	handleGetFilterValuesResponse: function( oResponse )
	{
		var payload = {};
		var json = this.getJsonFromResponse(oResponse);
		if (json) {
			payload = json.filterValues;
		}
		this.fireEvent("com.ibm.bux.filter.values.get.done", null, payload);
	},

	getJsonFromResponse: function (oResponse) {
		var json = null;
		if (oResponse && oResponse.m_jsonResponse && oResponse.m_jsonResponse.json) {
			json = dojo.eval("(" + oResponse.m_jsonResponse.json + ")");
		} else if (oResponse && oResponse.responseText) {
			json = dojo.eval("[" + oResponse.responseText + "]");
			if (json && json[0]) {
				json = json[0];
			}
		}

		return json;

	},

	onDrill: function( evt )
	{
		this.m_handleDrill = false; //used for unit test only

		if( !( evt && evt.payload && evt.payload.drillAction && evt.payload.cv_id && evt.payload.drillSpec)  || this.isLimitedInteractiveMode() )
		{
			return;
		}

		//process event only if fired by different widget
		if( evt.payload.cv_id === this.iContext.widgetId )
		{
			return;
		}

		var sDrillAction = evt.payload.drillAction;
		var oDrillAction = null;
		if( sDrillAction === "DrillDown")
		{
			oDrillAction = new DrillDownAction();
		}
		else if( sDrillAction === "DrillUp")
		{
			oDrillAction = new DrillUpAction();
		}
		else
		{
			return;
		}

		oDrillAction.setCognosViewer(this.getViewerObject());

		oDrillAction.setKeepFocusOnWidget(false);
		if( oDrillAction.parseDrillSpec( evt ) )
		{
			oDrillAction.execute();
		}


		this.m_handleDrill = true;// for unit test only
	},

	/*
	 * Handles share prompt event.
	 * Returns true if sucessful, false otherwise
	 */
	onPrompt: function( evt )
	{
		try
		{
			var cognosViewer = this.getViewerObject();
			//process event only if fired by different widget and we're not looking at saved output
			if( evt.payload.cv_id === this.iContext.widgetId || this.isSavedOutput() )
			{
				return false;
			}
			if (this.promptParametersRetrieved == false)
			{
				cognosViewer.showExecutionCursor();
				//run getParameters to get the prompt parameters for the current widget
				var getParametersAction = new GetParametersAction();
				getParametersAction.setCognosViewer(cognosViewer);
				getParametersAction.setRequestParms(evt.payload);
				getParametersAction.execute();

			}
			else
			{
				var sharePromptAction = new SharePromptAction();
				sharePromptAction.setCognosViewer(cognosViewer);
				sharePromptAction.setRequestParms(evt.payload);
				sharePromptAction.executePrompt();
			}

			return true;
		}
		catch( e )
		{
			return false;
		}
	},

	setPromptParametersRetrieved: function(promptParametersRetrieved)
	{
		this.promptParametersRetrieved = promptParametersRetrieved;
	},

	getEnvParam: function( paramName )
	{
		return this.getViewerObject().envParams[ paramName ];
	},

	destroy: function() {
		var viewer = this.getViewerObject();
		if (viewer) {

			if( this.onUnloadInProgress ) {
				//The viewer code specifies that if no callback is provided, the destroy function is synchronous
				//when we are in window unload processing, we want the http requests sent for
				//releasing conversations to be completed before window closes. Using async
				//requests during such a situation results in http requests
				//to be sent only for some before browser window closes.
				viewer.destroy(null);
			} else {
				//The viewer code specifies that if no callback is provided, the destroy function is synchronous
				//We don't want that, so just provide a dummy callback
				viewer.destroy(function() {});
			}

			var selectionController = viewer.getSelectionController();
			delete selectionController;

			var rmItem = function(element) {
				if (element && element.parentNode && element.parentNode.removeChild) {
					element.parentNode.removeChild(element);
				}
			};

			// Cleanup the markup that was added by this widget -- the nodes are identified by an attribute named namespaceId, first in the <head>, then the body children
			var items = dojo.query("[namespaceId='" + this.getViewerId() + "']", document.getElementsByTagName("head").item(0));
			if (items && items.length > 0) {
				items.forEach( rmItem );
			}

			items = dojo.query('> [namespaceId="' + this.getViewerId() + '"]', document.getElementsByTagName("body").item(0));
			if (typeof items != "undefined" && items && items.length > 0) {
				items.forEach( rmItem );
			}

			this._cleanupPreviousAnnotations();
		}

		delete this.annotationStore;
		delete this.savedOutput;
		delete this.undoRedoQueue;

		delete viewer;
	},

	setSavedOutputsCMResponse: function(cmResponse)
	{
		this.savedOutputsCMResponse = cmResponse;

		if (cmResponse == null) {
			var actionFactory = this.getViewerObject().getActionFactory();
			var action = actionFactory.load("Snapshots");
			action.resetMenu();
		}
	},

	getSavedOutputsCMResponse: function()
	{
		if (typeof this.savedOutputsCMResponse == "undefined" || this.savedOutputsCMResponse == null)
		{
			return null;
		}

		return this.savedOutputsCMResponse;
	},

	setSavedOutputSearchPath: function(searchPath)
	{
		this.m_savedOutputSearchPath = searchPath;
	},

	getSavedOutputSearchPath: function()
	{
		if (typeof this.m_savedOutputSearchPath == "undefined")
		{
			return null;
		}

		return this.m_savedOutputSearchPath;
	},
	initializeProperties: function()
	{
		if( this.properties  )
		{
			var viewerObject = this.getViewerObject();
			this.properties.initialize( viewerObject.envParams );
		}
	},
	addToRAPCache: function(name, value) {
		if(typeof this.m_RAPCache == "undefined" || this.m_RAPCache == null) {
			this.m_RAPCache = {};
		}
		this.m_RAPCache[name] = value;
	},
	getRAPCachedItem: function(name) {
		if(typeof this.m_RAPCache != "undefined" && this.m_RAPCache != null && typeof this.m_RAPCache[name] != "undefined") {
			return this.m_RAPCache[name];
		}

		return null;
	},
	clearRAPCache: function() {
		this.m_RAPCache = null;
	},
	// called by ResetToOriginalAction to reset parameters, cache, queues, etc
	reset: function() {
		this.promptParametersRetrieved = false;
		this.getViewerObject().envParams["reportPrompts"] = "";

		// clear the cached CM response for available outputs since we're reseting to the original
		this.setSavedOutputsCMResponse(null);

		// clear the undo queue
		this.getUndoRedoQueue().clearQueue();
	},

	handleFault: function() {
		var viewerObject = this.getViewerObject();
		viewerObject.hideExecutionCursor();
		if(viewerObject != null) {
			viewerObject.getWaitPageImpl().hide();

			if( this.isGlobalPromptException( viewerObject ) )
			{
				this.m_widgetErrorDlg = new ViewerIWidgetGlobalPromptErrorPage(viewerObject);
				this.handleGlobalPromptException( viewerObject );
			}
			else
			{
				this.m_widgetErrorDlg = new IWidgetErrorPage(viewerObject);
			}

			if( viewerObject.envParams && viewerObject.envParams["ui.action"] === "buxView" )
			{
				this.setSavedOutputsCMResponse( null );
			}

			this.m_widgetErrorDlg.handleFault();
		}
	},

	isGlobalPromptException: function( viewerObject )
	{
		if (viewerObject.LAST_ACTION_PARAMS  && viewerObject.LAST_ACTION_PARAMS.param1 )
		{
			var lastActionFormFields = viewerObject["LAST_ACTION_PARAMS"].param1.getFormFields();
			if( lastActionFormFields.exists("sharedPromptRequest") ||
			   ( lastActionFormFields.m_aValues["ui.reportDrop"] === "true" && lastActionFormFields.exists("widget.globalPromptInfo") ) ){
				return true;
			}
		}

		return false;
	},

	handleGlobalPromptException: function( viewerObject )
	{
		viewerObject.disableRaiseSharePromptEvent();
		var lastActionFormFields = viewerObject["LAST_ACTION_PARAMS"].param1.getFormFields();
		if( !lastActionFormFields.exists("sharedPromptRequest") )
		{
			//for report that failed immediately on drop, the request needs to be resent on OK, otherwise,
			//user will get a blank widget.
			this.m_widgetErrorDlg.setSendRequestOnOK( true );
		}

		this.m_widgetErrorDlg.setErrorMessage(RV_RES.IDS_JS_GLOBAL_PROMPTS_NOT_RESPONDING_WARNING);

	},

	disableListenToForGlobalPrompt: function()
	{
		this.iContext.iEvents.fireEvent("com.ibm.bux.widget.updateEventFilter", null, { "*": { blockedEvents: ['com.ibm.bux.viewer.prompt']} });
		var viewerObject = this.getViewerObject();
		if (window.gaRV_INSTANCES) {
			var payload = {};
			payload[ this.iContext.widgetId ] = { blockedEvents: ['com.ibm.bux.viewer.prompt']};
			for (var iIndex=0; iIndex < window.gaRV_INSTANCES.length; iIndex++)	{
				if(window.gaRV_INSTANCES[iIndex].getId() != viewerObject.getId()) {
					var viewerIWidget = window.gaRV_INSTANCES[iIndex].getViewerWidget();
					if(viewerIWidget != null) {
						viewerIWidget.iContext.iEvents.fireEvent("com.ibm.bux.widget.updateEventFilter", null, payload);
					}
				}
			}
		}
	},

	handlePassportErrorFromDispatcher: function() {//Bug: COGCQ00261642

		if (this.viewerWidgetLastAction) {
			var okCallbackFunction = GUtil.generateCallback(this[this.viewerWidgetLastAction.functionName], this.viewerWidgetLastAction.param, this);
			var dialog = new bux.dialogs.LogonDialog({
				okHandler: okCallbackFunction,
				cancelHandler: dojo.hitch(this, function() {this.iContext.iEvents.fireEvent("com.ibm.bux.widget.action", null,  { action: 'deleteWidget' } );})
				 });
			dialog.startup();
			dialog.show();
		}
	},

	showErrorMessage: function(errorMessage) {
		var viewerObject = this.getViewerObject();
		if(viewerObject != null) {
			this.m_widgetErrorDlg = new IWidgetErrorMessage(viewerObject, errorMessage);
			this.m_widgetErrorDlg.renderInlineDialog();
		}
	},
	getErrorDlg: function() {
		if (this.m_widgetErrorDlg) {
			return this.m_widgetErrorDlg;
		}

		return null;
	},
	setErrorDlg: function(dlg) {
		this.m_widgetErrorDlg = dlg;
	},
	clearErrorDlg: function(dlg) {
		if( this.getErrorDlg() !== null)
		{
			this.m_widgetErrorDlg.ok();
		}
	},

	onOpen: function( evt )
	{
		if( evt && evt.payload && evt.payload.author )
		{
			this.m_sDashboardOwnerId = evt.payload.author.cm$searchPath;
		}
	},

	onNotifyPreviousEventsAreDone: function(evt)
	{
		this.m_resizeReady = true;
	},

	isDashboardOwner: function()
	{
		if( typeof this.m_sDashboardOwnerId !== "undefined" && typeof this.m_sUserId !== "undefined")
		{
			return this.m_sDashboardOwnerId === this.m_sUserId;
		}
	},


	findContainerDiv : function()
	{
		var rvContentDiV = dojo.byId("RVContent" + this.getViewerObject().getId());
		// we need to find the correct container div. It'll be the one with a height set in pixels
		var containerDiv = rvContentDiV;
		while (containerDiv) {
			if (containerDiv.style && containerDiv.style.height && containerDiv.style.height.indexOf("px") != -1) {
				return containerDiv;
			}
			containerDiv = containerDiv.parentNode;
		}

		if (typeof console != "undefined" && console && console.log)
		{
			console.log("ViewrIWidget: Could not find the container div.");
		}

		return rvContentDiV;
	},

	getPaneOnRight: function () {
		return this.m_sPaneOnRight;
	}
});

/**
 * Helper class for the undo/redo queue
 */
function UndoRedoQueue(oWidget)
{
	this.m_queue = [];
	this.m_queueLimit = 20;
	this.m_iCurrentPos = 0;
	this.m_widget = oWidget;
	this.m_bUndoRedoCalled = false;
	this.m_lastAction = null;
	this.currentUndoObj = null;
	this.beenUsed = false;
	this.m_originalSpec = null;
}

UndoRedoQueue.prototype.setOriginalSpec = function(spec)
{
	// only time we need an original spec if the queue is still empty.
	if (this.m_queue.length == 0) {
		this.m_originalSpec = spec;
	}
};

UndoRedoQueue.prototype.clearQueue = function()
{
	while (this.m_queue.length > 0)
	{
		this.m_queue.pop();
	}

	this.m_iCurrentPos = 0;
};

UndoRedoQueue.prototype.getPosition = function()
{
	return this.m_iCurrentPos;
};

UndoRedoQueue.prototype.hasBeenUsed = function()
{
	return this.beenUsed;
};

UndoRedoQueue.prototype.getLength = function()
{
	return this.m_queue.length;
};

UndoRedoQueue.prototype.initUndoObj = function(obj)
{
	// first time we add to the stack for an action, add the current specification/parameters to have the initial report
	if (this.m_queue.length == 0)
	{
		var oCV = this.m_widget.getViewerObject();
		var spec = this.m_originalSpec == null ? oCV.envParams["ui.spec"] : this.m_originalSpec;
		this.m_originalSpec = null;
		this.m_queue.push({"spec": spec,
			"parameters": oCV.getExecutionParameters(),
			"tooltip": obj.tooltip,
			"infoBar": oCV.envParams["json.infoBar"],
			"widgetProperties" : this.m_widget.getProperties().getOldProperties(),
			"hasAVSChart" : oCV.hasAVSChart()
		});
		this.m_iCurrentPos = this.m_queue.length-1;
		this.beenUsed = true;
	}


	this.currentUndoObj = obj;
};

UndoRedoQueue.prototype.add = function(obj)
{
	// Make sure we don't add to the undo stack if the Viewer is getting refreshed
	// because of an undo or redo
	if (this.m_bUndoRedoCalled == true)
	{
		this.m_bUndoRedoCalled = false;
		return;
	}
	else if (this.currentUndoObj == null)
	{
		return;
	}
	else if (obj.reportUpdated == true)
	{
		while (this.m_queue.length > this.m_queueLimit)
		{
			this.m_queue.shift();
		}

		this.clearEnd();

		var viewer = this.m_widget.getViewerObject();
		if (this.currentUndoObj.saveSpec == true || viewer.envParams["ui.doNotUseConversationForUndo"] === "true")
		{
			this.currentUndoObj.spec = viewer.envParams["ui.spec"];
			this.currentUndoObj.parameters = viewer.getExecutionParameters();
		}
		else
		{
			this.currentUndoObj.conversation = viewer.getConversation();
		}

		if (viewer.envParams["json.infoBar"])
		{
			this.currentUndoObj.infoBar = viewer.envParams["json.infoBar"];
		}

		this.currentUndoObj.widgetProperties = this.m_widget.getProperties().getUndoInfo();
		this.currentUndoObj.palette = viewer.m_sPalette;

		if (this.m_widget.getProperties().getFlashCharts())
		{
			this.currentUndoObj.hasAVSChart = viewer.hasAVSChart();
		}



		this.m_queue.push(this.currentUndoObj);
		this.m_iCurrentPos = this.m_queue.length-1;
		this.m_bUndoRedoCalled = false;
		this.m_lastAction = "";
		this.currentUndoObj = null;
		this.beenUsed = true;
	}
};

UndoRedoQueue.prototype.moveBack = function()
{
	if (this.m_iCurrentPos <= 0)
	{
		return null;
	}

	this.m_bUndoRedoCalled = true;
	this.m_lastAction = "undo";
	this.m_iCurrentPos--;
	this.m_viewerState = {"parameters" : this.m_widget.getViewerObject().getExecutionParameters(), "spec" : this.m_widget.getViewerObject().envParams["ui.spec"]};
	if (typeof this.m_queue[this.m_iCurrentPos] != "undefined")
	{
		this.m_widget.getViewerObject().m_sPalette = this.m_queue[this.m_iCurrentPos].palette;
		return this.m_queue[this.m_iCurrentPos];
	}
	else
	{
		return null;
	}
};

UndoRedoQueue.prototype.moveForward = function()
{
	if (this.m_iCurrentPos >= this.m_queue.length)
	{
		return null;
	}

	this.m_bUndoRedoCalled = true;
	this.m_iCurrentPos++;
	this.m_lastAction = "redo";
	this.m_viewerState = {"parameters" : this.m_widget.getViewerObject().getExecutionParameters(), "spec" : this.m_widget.getViewerObject().envParams["ui.spec"]};
	if (typeof this.m_queue[this.m_iCurrentPos] != "undefined")
	{
		this.m_widget.getViewerObject().m_sPalette = this.m_queue[this.m_iCurrentPos].palette;
		return this.m_queue[this.m_iCurrentPos];
	}
	else
	{
		return null;
	}
};

UndoRedoQueue.prototype.clearEnd = function()
{
	while (this.m_queue.length-1 > this.m_iCurrentPos)
	{
		this.m_queue.pop();
	}
};

UndoRedoQueue.prototype.handleCancel = function()
{
	if (this.m_lastAction == "undo")
	{
		this.m_iCurrentPos++;
	}
	else if (this.m_lastAction == "redo")
	{
		this.m_iCurrentPos--;
	}

	if (this.m_lastAction == "redo" || this.m_lastAction == "undo")
	{
		var oCV = this.m_widget.getViewerObject();
		oCV.setExecutionParameters(this.m_viewerState.parameters);
		oCV.envParams["ui.spec"] = this.m_viewerState.spec;
	}

	this.m_lastAction = "";
	this.m_bUndoRedoCalled = false;
	this.currentUndoObj = null;
};

UndoRedoQueue.prototype.handleFault = function()
{
	this.currentUndoObj = null;
	this.m_bUndoRedoCalled = false;
};

UndoRedoQueue.prototype.getUndoTooltip = function()
{
	if (this.m_iCurrentPos > 0)
	{
		return RV_RES.IDS_JS_UNDO + " " + this.m_queue[this.m_iCurrentPos].tooltip;
	}
	else
	{
		return RV_RES.IDS_JS_UNDO;
	}
};

UndoRedoQueue.prototype.getRedoTooltip = function()
{
	if (this.m_iCurrentPos < (this.m_queue.length-1))
	{
		return RV_RES.IDS_JS_REDO + " " + this.m_queue[this.m_iCurrentPos+1].tooltip;
	}
	else
	{
		return RV_RES.IDS_JS_REDO;
	}
};

/*
 * Class to handle showing saved output in an iWidget
 */
function CIWidgetSavedOutput(iWidget) {
	this.iWidget = iWidget;
	this.bContainsChart = false;
	this.bContainsDrillThrough = false;
	this.bPagedOutput = true;
}

CIWidgetSavedOutput.prototype.getIWidget = function() {
	return this.iWidget;
};

CIWidgetSavedOutput.prototype.isPagedOutput = function() {
	return this.bPagedOutput;
};

CIWidgetSavedOutput.prototype.setPagedOutput = function(pagedOutput) {
	this.bPagedOutput = pagedOutput;
};

CIWidgetSavedOutput.prototype.getCognosViewer = function() {
	return this.getIWidget().getViewerObject();
};

CIWidgetSavedOutput.prototype.getIframeWindow = function() {
	return this.savedOutputWindow;
};

CIWidgetSavedOutput.prototype.getSavedOutputDiv = function() {
	return this.savedOutputDiv;
};

CIWidgetSavedOutput.prototype.render = function() {
	if (this.isPagedOutput() && this.getFirstPage()) {
		this.currentPage = this.getFirstPage();

		var sHTML = "";
		if (this.currentPage.outerHTML) {
			sHTML = this.currentPage.outerHTML;
		}
		else {
			sHTML = new XMLSerializer().serializeToString(this.currentPage);
		}
		this.showHTMLFragment( sHTML );
	} else {
		this.showHTMLFragment(this.getIframeWindow().document.body.innerHTML);
	}

	this.writeNavLinks();
};

CIWidgetSavedOutput.prototype.outputDoneLoading = function() {

	var cognosViewerObject = this.getCognosViewer();
	var sCVID = cognosViewerObject.getId();

	this.savedOutputWindow = dojo.byId("CVHiddenIFrame" + sCVID).contentWindow;
	this.savedOutputDiv = dojo.byId("CVSavedOutputDiv" + sCVID);

	var sc = cognosViewerObject.getSelectionController();
	if ( !sc.hasContextData() && this.savedOutputWindow.onLoadEvent ) {
		this.savedOutputWindow.onLoadEvent();
	}


	// load all the style tags from the iframe
	var oReportDiv = dojo.byId("_" + this.iWidget.iContext.widgetId + "_cv");

	this.loadStyleInfoFromIframe(oReportDiv);

	this.bContainsChart = typeof this.savedOutputWindow.displayChart == "function" ? true : false;
	this.bContainsDrillThrough = typeof this.savedOutputWindow.drillThroughTarget != "undefined" ? true : false;

	this.render();
	this.getMetaDataFromIFrame();

	if (this.bContainsDrillThrough) {
		// get the drillTargets array from the iframe
		window[sCVID + "drillTargets"] = this.getIframeWindow().drillTargets;
	}

	cognosViewerObject.fillInContextData();
	cognosViewerObject.updateBorderCollapse();
	this.getIWidget().updateToolbar();
	cognosViewerObject.hideExecutionCursor();
	cognosViewerObject.setReportRenderingDone(true);

	this.getPageContextData();

	this.getIWidget().fetchAnnotationData();

	this.fixDuplicateFormFields();

	setTimeout(getCognosViewerObjectRefAsString(sCVID)+".getViewerWidget().iContext.iEvents.fireEvent(\"com.ibm.bux.widget.render.done\", null, {})",100);
};

/*
	This is not pretty, but necessary. When rendering a saved output in BUX, we need
	the response to contain a bunch of hidden form fields since there's saved output code that
	relies on those form fields. Problem is, that we'll end up with duplicate hidden form fields.
	This code loops through all the form fields and will remove any duplicates using the value from
	the last one as the value of the form field.
*/
CIWidgetSavedOutput.prototype.fixDuplicateFormFields = function() {
	var formWarpRequest = document.getElementById("formWarpRequest" + this.getCognosViewer().getId());
	if (formWarpRequest) {
		for (var formFieldIndex=0; formFieldIndex < formWarpRequest.elements.length; formFieldIndex++) {
			var name = formWarpRequest.elements[formFieldIndex].name;
			// if there's multiple form fields with the same name
			if (formWarpRequest[name].length > 1) {
				// get the value of the last form field with the name. It's the one we want
				var value = formWarpRequest[name][formWarpRequest[name].length-1].value;

				// delete all the duplicate form fields
				while (formWarpRequest[name].length > 1) {
					formWarpRequest[name][1].parentNode.removeChild(formWarpRequest[name][1]);
				}

				// some weird behavior with IE, if there's still an array of form fields set the value correctly.
				if (formWarpRequest[name][0]) {
					formWarpRequest[name][0].value = value;
				}

				formWarpRequest[name].value = value;
			}
		}
	}
};

CIWidgetSavedOutput.prototype.getPageContextData = function(){
	var items = {};

	dojo.query('[ctx]', this.getSavedOutputDiv()).forEach(function(item){
		var ctx = item.getAttribute("ctx");
		var ctxIds = ctx.split(':');
		for (var i = 0; i < ctxIds.length; i++) {
			var value = ctxIds[i];
			if (value && value.length > 0) {
				items[value] = true;
			}
		}
	});

	var arrItems = [];
	for (var item in items) {
		arrItems.push(item);
	}

	return this.getCognosViewer().getSelectionController().fetchContextData(arrItems);
};

CIWidgetSavedOutput.prototype.loadStyleInfoFromIframe = function(oReportDiv){

	var htmlNode = null;
	// For XHTML the first node is the doctype. Loop through the children of the document
	// to find the first child that has children
	var savedOutputDoc = this.savedOutputWindow.document;
	for (var index=0; index < savedOutputDoc.childNodes.length; index++) {
		if (savedOutputDoc.childNodes[index].childNodes && savedOutputDoc.childNodes[index].childNodes.length > 0) {
			htmlNode = savedOutputDoc.childNodes[index];
			break;
		}
	}

	var styles = dojo.query("style", htmlNode);
	var styleHTML = "";
	for (var i = 0; i < styles.length; i++) {
		var styleText = styles[i].innerHTML;
		if (styleText && styleText.length > 0) {
			styleHTML += '<style type="text/css">' + styleText + '</style>';
		}
	}

	window.gScriptLoader.loadStyles(styleHTML, this.iWidget.getViewerId());

	var links = dojo.query("link", htmlNode);
	var linkHTML = "";
	for (i = 0; i < links.length; i++) {
		var href = links[i].getAttribute("href");
		if (href) {
			linkHTML += '<link href="' + href + '" />';
		}
	}
	window.gScriptLoader.loadCSS(linkHTML, oReportDiv, true, this.iWidget.getViewerId());

};

CIWidgetSavedOutput.prototype.getFirstPage = function() {
	var firstPage = this.getIframeWindow().document.body.firstChild;
	while (firstPage && firstPage.nodeName.toLowerCase() != "table") {
		firstPage = firstPage.nextSibling;
	}

	return firstPage;
};

CIWidgetSavedOutput.prototype.getLastPage = function() {
	var lastPage = this.getIframeWindow().document.body.lastChild;
	while (lastPage && lastPage.nodeName.toLowerCase() != "table") {
		lastPage = lastPage.previousSibling;
	}

	return lastPage;
};

CIWidgetSavedOutput.prototype.writeNavLinks = function() {
	var availablePageLinks = "";
	if (this.isPagedOutput()) {
		if (this.currentPage.previousSibling && this.currentPage.previousSibling.nodeName.toLowerCase() == "table") {
			availablePageLinks += " firstPage previousPage ";
		}
		if (this.currentPage.nextSibling && this.currentPage.nextSibling.nodeName.toLowerCase() == "table") {
			availablePageLinks += " nextPage lastPage ";
		}
	}

	this.getCognosViewer().writeNavLinks(availablePageLinks, true);
};

CIWidgetSavedOutput.prototype.pageAction = function(action) {
	switch(action) {
		case "firstPage":
			this.currentPage = this.getFirstPage();
			break;
		case "previousPage":
			this.currentPage = this.currentPage.previousSibling;
			break;
		case "nextPage":
			this.currentPage = this.currentPage.nextSibling;
			break;
		case "lastPage":
			this.currentPage = this.getLastPage();
			break;
	}

	if (this.currentPage) {
		var oCV = this.getCognosViewer();
		oCV.getSelectionController().clearSelectedObjects();
		this.showHTMLFragment(this.currentPage.innerHTML);
		this.writeNavLinks();


		oCV.setKeepFocus("CVNavLinks" + oCV.getId());
		oCV.doneLoadingUpdateA11Y("complete");

		var numMissing = this.getPageContextData();
		if (numMissing > 0) {
			//There were some items on the page which did not have context information.
			//Refetch the annotations to see if there were any annotations on those cells.
			this.getIWidget().fetchAnnotationData();
		}
	}
};

CIWidgetSavedOutput.prototype.getMetaDataFromIFrame = function() {
	if (typeof this.getIframeWindow().getSelectionController == "function") {
		var iframeSelectionController = this.getIframeWindow().getSelectionController();
		if (iframeSelectionController) {
			var selectionController = this.getCognosViewer().getSelectionController();
			selectionController.addMetaData(iframeSelectionController.getCCDManager().m_md);
			selectionController.addContextData(iframeSelectionController.getCCDManager().m_cd);
			selectionController.setSelectionBasedFeaturesEnabled(true);
		}
	}
};

CIWidgetSavedOutput.prototype.processDrillThrough = function(sHTML) {
	if (this.bContainsDrillThrough && (typeof this.bOldOutput == "undefined" || this.bOldOutput == true)) {
		var drillThroughRegEx = new RegExp('onclick=\"doSingleDrillThrough[^\"]*\"|onclick=\'doMultipleDrillThrough[^\']*\'', "gim");
		var aM = sHTML.match(drillThroughRegEx);
		if (aM)
		{
			if (typeof this.bOldOutput == "undefined" && aM.length > 0 && aM[0].indexOf("'_THIS_'") > 0) {
				this.bOldOutput = false;
			} else {
				this.bOldOutput = true;

				// singledrill and multipledrill use different quots (', ") for the onclick. So figure
				// which one we're suppose to use
				var endingQuot = "";
				var quot = "'";
				if (aM.length > 0) {
					endingQuot = aM[0].charAt(aM[0].length - 1);
				}
				if (endingQuot == "'"){
					quot = "\"";
				}

				for (var i = 0; i < aM.length; i++)
				{
					sHTML = sHTML.replace(aM[i], aM[i].substring(0, aM[i].length-3) + "," + quot + this.getCognosViewer().getId() + quot + ");" + endingQuot);
				}
			}
		}
	}

	return sHTML;
};

CIWidgetSavedOutput.prototype.processChart = function(sHTML){
	// get rid of some chart js that's no longer needed.
	return sHTML.replace(/displayChart[^;].*;|var graphicSrc[^;].*;|var graphicName[^;].*;/g, "");
};

CIWidgetSavedOutput.prototype.showHTMLFragment = function(sHTML) {

	if (this.bContainsChart) {
		sHTML = this.processChart(sHTML);
	}

	sHTML = this.processDrillThrough(sHTML);

	sHTML = sHTML.replace(/_THIS_/g,this.getCognosViewer().getId());

	this.getIWidget().loadStylesAndScripts(sHTML, this.getSavedOutputDiv());
};

/*
 * Because of the way CScriptLoader loads and processes scripts (asynchronously with setInterval),
 * this function is necessary. We don't want to have the toolbar initialize until all scripts
 * have been loaded and executed. Otherwise, we may end up referencing variables that are not initialized.
 */
function gInitializeViewer(obj)
{
	if (window.gScriptLoader.hasCompletedExecution()) {
		var viewerObject = obj.getViewerObject();
		if(viewerObject != null && viewerObject.inlineScriptsDoneExecuting == true) {
			clearInterval(obj.timer);
			if(typeof viewerObject.errorOccured != "undefined") {
				obj.fireEvent("com.ibm.bux.widget.render.done", null, {});
			}
			else if(typeof viewerObject.buxToolbarPayload != "undefined") {
				// associate the widget ref with the viewer object
				viewerObject.setViewerWidget(obj);
				if( obj.isDashboardOwner() )
				{
					viewerObject.addReportInfo();
				}
				if (obj.promptParametersRetrieved == true) {
					var reportPrompts = obj.getAttributeValue( "viewerReportPrompts");
					viewerObject.envParams["reportPrompts"] = reportPrompts;
				} else if (viewerObject.envParams && viewerObject.envParams["widget.promptParametersRetrievedOnDrop"] == "true") {
					obj.promptParametersRetrieved =  true;
				}

				//Fire this now that we have the IWidget set.
				obj.fireEvent("com.ibm.bux.widget.setShowBordersWhenInnactive", null, (viewerObject.m_flashChartsObjectIds.length !== 0));
			}
		}
	}
}

function gVWEvent(sId, evt, sEvent) {
	var oCV = window['oCV' + sId];
	if (oCV) {
		var iWidget = oCV.getViewerWidget();
		if (iWidget) {
			iWidget[sEvent](evt);
		}
	}
}
