/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */


dojo.provide("IWidgetErrorPage");

dojo.declare("IWidgetErrorPage", null, {

	m_soapFault: null,
	m_errorMessage : null,
	m_oCV: null,

	constructor: function(oCV) {
		this.m_oCV = oCV;
		if (oCV && oCV.getViewerWidget && oCV.getViewerWidget()) {
			oCV.getViewerWidget().clearErrorDlg();
		}
	},
	getSoapFault: function() {
		if(this.m_soapFault == null && this.m_oCV.getSoapFault() != "") {
			this.m_soapFault = XMLBuilderLoadXMLFromString(this.m_oCV.getSoapFault());
		}
		return this.m_soapFault;
	},
	canRetryRequest: function() {
		var soapFault = this.getSoapFault();
		var canRetry = true;
		var canRetryNode = XMLHelper_FindChildrenByTagName(soapFault, "allowRetry", true);
		if(canRetryNode.length > 0)
		{
			canRetry = ( XMLHelper_GetText(canRetryNode[0], false) === "true" );
		}

		return canRetry;
	},
	isWarningMessage: function() {
		return false;
	},
	renderDialogIcon: function() {
		var imageClass = this.isWarningMessage() ? "bux-informationDialog-warning-icon" : "bux-informationDialog-error-icon";
		var sError = RV_RES.IDS_JS_ERROR;
		return	"<td class=\"bux-InformationDialog-Icon\">" +
					"<div title=\"" + sError + "\" alt=\"" + sError + "\" class=\"" + imageClass + "\">" +
						"<span class=\"buxErrorNodeInner\">x</span>" +
						"<span class=\"buxInfoNodeInner\">i</span>" +
						"<span class=\"buxWarningNodeInner\">!</span>" +
					"</div>" +
				"</td>";
	},
	renderErrorMessage: function() {
		var cvId = this.m_oCV.getId();
		return	"<tr class=\"bux-InformationDialog-dialogTypeSection\">" +
					"<td>" +
						"<div></div>" +
					"</td>" +
				"</tr>" +
				"<tr>" +
					"<td class=\"bux-InformationDialog-mainMessage\">" +
						"<div tabindex=\"0\" aria-labelledby=\"errorString" + cvId + " errorMessage" + cvId + "\" id=\"errorMessage" + cvId + "\">" + this.getErrorMessage() + "</div>" +
						"<div style=\"display:none\" id=\"errorString" + cvId + "\">" + RV_RES.IDS_JS_ERROR + ": </div>" +
					"</td>" +
				"</tr>" +
				"<tr>" +
					"<td class=\"bux-dialog-blank\"></td>" +
				"</tr>";

	},
	renderErrorDetailsMessage: function() {
		var details = this.getDetails();
		if(details != "") {
			var buttonAttributes = " aria-labelledby=\"detailText" + this.m_oCV.getId() + "\" tabindex=\"0\" role=\"button\" onkeydown=\"buxErrorPage_toggleErrorDetails(event, '" + this.m_oCV.getId() + "', this);\"";
			return "<tr>" +
						"<td class=\"bux-InformationDialog-detail-section\">" +
							"<div class=\"dijitTitlePane\" style=\"overflow:visible\" role=\"group\" onclick=\"buxErrorPage_toggleErrorDetails(event, '" + this.m_oCV.getId() + "', this);\">" +
								"<div id=\"titlePaneTitle" + this.m_oCV.getId() + "\" class=\"dijitTitlePaneTitle dijitClosed\">" +
									"<img " + buttonAttributes + " class=\"digitOpen dijitArrowNode\" alt=\"" + RV_RES.IDS_JS_DETAILS + "\" src=\"" + this.m_oCV.getWebContentRoot() + "/rv/images/blank.gif\" role=\"presentation\"></img>" +
									"<span " + buttonAttributes + " class=\"dijitArrowNodeInner\">+</span>" +
									"<span id=\"detailText" + this.m_oCV.getId() + "\" class=\"dijitTitlePaneTextNode\" style=\"-moz-user-select: none;\">" + RV_RES.IDS_JS_DETAILS + "</span>" +
								"</div>" +
								"<div class=\"dijitTitlePaneContentOuterClosed\" id=\"buxErrorPageDetailContentOuter" + this.m_oCV.getId() + "\">" +
									"<div class=\"dijitReset\">" +
										"<div id=\"buxErrorPageDetails" + this.m_oCV.getId() + "\" tabindex=\"-1\" class=\"dijitTitlePaneContentInner\" style=\"display:none;\"role=\"region\" >" +
											"<div aria-labelledby=\"errorDetails" + this.m_oCV.getId() + "\" id=\"errorDetails" + this.m_oCV.getId() + "\" style=\"overflow: auto; width:400px; height: 200px;\">" +
												details +
											"</div>" +
										"</div>" +
									"</div>" +
								"</div>" +
							"</div>" +
						"</td>" +
					"</tr>";
		} else {
			return "";
		}
	},
	renderErrorDetailsButtonBar: function() {
		return "<tr>" +
					"<td>" +
						"<div class=\"bux-basedialog-button-area\">" +
							this.renderRetryButton() +
							this.renderOKButton() +
						"</div>" +
					"</td>" +
				"</tr>";
	},
	renderRetryButton: function()
	{
		if( !this.canRetryRequest())
		{
			return "";
		}

		var errorDlg = getCognosViewerObjectRefAsString(this.m_oCV.getId()) + ".getViewerWidget().getErrorDlg()";
		var retryButton = new ViewerIWidgetDialogButton("if(" + errorDlg+ ") { " + errorDlg + ".retry();}", RV_RES.IDS_JS_RETRY);
		return retryButton.render();
	},
	renderOKButton: function()
	{
		var errorDlg = getCognosViewerObjectRefAsString(this.m_oCV.getId()) + ".getViewerWidget().getErrorDlg()";
		var okButton = new ViewerIWidgetDialogButton("if(" + errorDlg+ ") { " + errorDlg + ".ok();}", RV_RES.IDS_JS_OK);
		return okButton.render();
	},
	renderErrorDetails: function() {
		return this.renderErrorDetailsMessage() +
				this.renderErrorDetailsButtonBar();
	},
	renderError: function() {
		return	"<td class=\"bux-InformationDialog-message-section\">" +
					"<table cellspacing=\"0\" cellpadding=\"0\" class=\"bux-InformationDialog-message-table\" width=\"360\" role=\"presentation\">" +
						"<tbody>" +
							this.renderErrorMessage() +
							this.renderErrorDetails() +
						"</tbody>" +
					"</table>" +
				"</td>";
	},
	renderInlineDialog: function() {
		var sHtml =
			"<table width=\"400px\" cellspacing=\"0\" cellpadding=\"0\" role=\"presentation\">" +
				"<tbody>" +
					"<tr>" +
						this.renderDialogIcon() +
						this.renderError() +
					"</tr>" +
				"</tbody>" +
			"</table>";

		this.m_dlg = new ViewerIWidgetInlineDialog(this.m_oCV, sHtml);
		this.m_dlg.setVerticalPositionOption("top");
		if(this.m_oCV.isReportRenderingDone()) {
			this.m_dlg.setBackgroundClassOverride("inlineDialog-background-error");
		}

		this.m_dlg.show();

		// only set the focus to the error message if it's being displayed after
		// the user did an action. This will make JAWS read the error message. We don't
		// want JAWS to read error messages on a dashboard open
		if (this.m_oCV.getKeepFocus() != false) {
			setFocusToFirstTabItem(this.m_dlg.getDialogDiv());
		}
	},
	showLogOnDialog: function(promptInfoNamespace) {
		var cvId = this.m_oCV.getId();

		if (window["CVEditContentActionInstance"])
		{
			window["CVEditContentActionInstance"].transitionFromBUA();
		}

		IWidgetLogonhandler.handleLogon(cvId, promptInfoNamespace);
	},
	handleFault: function() {
		if(this.isAuthenticationFault()) {
			var promptInfoNamespace = this.getPromptInfoNamespacesFromAuthenticationFault();
			this.showLogOnDialog(promptInfoNamespace);
		} else {
			this.m_oCV.m_resizeReady = true;
			this.renderInlineDialog();
		}
	},
	setErrorMessage: function(errorMessage) {
		this.m_errorMessage = errorMessage;
	},
	getErrorMessage: function() {
		if(this.m_errorMessage == null) {
			var soapFault = this.getSoapFault();
			var errorMessageNodes = XMLHelper_FindChildrenByTagName(soapFault, "message", true);
			if(errorMessageNodes.length > 0)
			{
				var messageStringNode = XMLHelper_FindChildByTagName(errorMessageNodes[0], "messageString", false);

				this.checkAndSetErrorMessage( XMLHelper_GetText(messageStringNode, false) );
			}
		}

		return this.m_errorMessage;
	},

	checkAndSetErrorMessage: function( errMsg ) {

		this.m_detailMessageStartIndex = 1;

		if( errMsg.indexOf('RSV-BBP-0038') == 0 ) {
			errMsg = RV_RES.IDS_CONV_CANCELED_ERROR;
			this.m_detailMessageStartIndex = 0;
		}
		this.m_errorMessage = errMsg;
	},
	getDetails: function() {
		var soapFault = this.getSoapFault();
		var detailsString = "";
		var errorMessageNodes = XMLHelper_FindChildrenByTagName(soapFault, "message", true);
		if(errorMessageNodes.length > 0)
		{
			if (typeof this.m_detailMessageStartIndex === "undefined" || this.m_detailMessageStartIndex===null) {
				this.m_detailMessageStartIndex = 1;
			}
			for(var messageIndex = this.m_detailMessageStartIndex; messageIndex < errorMessageNodes.length; ++messageIndex)
			{
				var messageStringNode = XMLHelper_FindChildByTagName(errorMessageNodes[messageIndex], "messageString", false);
				detailsString += XMLHelper_GetText(messageStringNode, false) + "\n";
			}
		}
		return detailsString;
	},
	isAuthenticationFault: function() {
		var soapFaultDocument = this.getSoapFault();
		if(soapFaultDocument != null) {
			var camElement = XMLHelper_FindChildByTagName(soapFaultDocument, "CAM", true);
			return (camElement != null && XMLHelper_FindChildByTagName(camElement, "promptInfo", true) != null);
		}
	},
	getPromptInfoNamespacesFromAuthenticationFault: function() {
		var soapFaultDocument = this.getSoapFault();
		if (soapFaultDocument == null) { return null; }

		var camElement = XMLHelper_FindChildByTagName(soapFaultDocument, "CAM", true);
		if (camElement == null) { return null; }

		var promptInfo = XMLHelper_FindChildByTagName(camElement, "promptInfo", true);
		if (promptInfo == null) { return null; }

		var displayObjects = XMLHelper_FindChildByTagName(camElement, "displayObjects", true);
		if (displayObjects == null) { return null; }

		var items = XMLHelper_FindChildrenByTagName(displayObjects, "item", true);
		if (items == null || items.length != 1) { return null; }

		var itemNode = items[0];
		if (itemNode == null) { return null; }
		var nameNode = XMLHelper_FindChildByTagName(itemNode, "name", true);
		var name = XMLHelper_GetText(nameNode, false);
		if (name === 'CAMNamespace') {
			var valueNode = XMLHelper_FindChildByTagName(itemNode, "value", true);
			var value = XMLHelper_GetText(valueNode, false);
			if (value!=null) {
				return value;
			}
		}
		return null;
	},
	retry: function() {
		this.ok();
		this.m_oCV.executeAction("RetryRequest", this.m_oCV["LAST_ACTION_PARAMS"]);
	},
	ok: function() {
		if (this.m_dlg) {
			this.m_dlg.destroy();
		}
		this.m_oCV.getViewerWidget().setErrorDlg(null);
	}
});

dojo.declare("IWidgetErrorMessage", IWidgetErrorPage, {
	m_oCV: null,
	m_errorMessage: "",
	constructor: function(oCV, errorMessage) {
		this.m_errorMessage = errorMessage;
	},
	getErrorMessage: function() {
		return this.m_errorMessage;
	},
	getDetails: function() {
		return "";
	},
	canRetryRequest: function() {
		return false;
	}
});

dojo.declare("IWidgetWarningMessage", IWidgetErrorMessage, {
	constructor: function(oCV, errorMessage) {
	},
	isWarningMessage: function() {
		return true;
	}
});

dojo.declare("ViewerIWidgetLogonHandler", null, {
	constructor: function() {
		this.m_cvIdList = [];
	},
	handleLogon: function(cvId, promptInfoNamespace) {
		this.m_cvIdList.push(cvId);
		if(this.m_cvIdList.length == 1) {

			var dialog = (promptInfoNamespace==null|| promptInfoNamespace.length==0) ?
				new bux.dialogs.LogonDialog({
					okHandler: GUtil.generateCallback(this.okHandler, [], this),
					cancelHandler: function() { }
				}):
				new bux.dialogs.LogonDialog({
					okHandler: GUtil.generateCallback(this.okHandler, [], this),
					cancelHandler: function() { },
					params: {"h_CAM_action": "logonAs", "CAMNamespace": promptInfoNamespace}
				});

			dialog.startup();
			dialog.show();
		}
	},
	okHandler: function() {
		for(var index = 0; index < this.m_cvIdList.length; ++index) {
			var cvId = this.m_cvIdList[index];
			var oCV = window["oCV" + cvId];

			// need to let chrome know the user logged on so they can refresh the user name and content tree
			if (index === 0 && oCV.getViewerWidget) {
				oCV.getViewerWidget().fireEvent("com.ibm.bux.widget.action", null, { action: "refreshAfterLogon" });
			}

			var lastActionParams = oCV.LAST_ACTION_PARAMS;
			if (lastActionParams && typeof lastActionParams.retryAction != "undefined") {
				oCV.executeAction(lastActionParams.retryAction, lastActionParams.retryActionRequestParams);
			}
			else {
				oCV.executeAction("RetryRequest", lastActionParams);
			}
		}
		this.m_cvIdList = [];
	}
});

function buxErrorPage_toggleErrorDetails(event, cvId, htmlNode) {
	// left mouse, enter or space bar
	if ((event.button == 0 && (event.keyCode == "0" || typeof event.keyCode == "undefined")) || event.keyCode == "32" || event.keyCode == "13") {
		var buxErrorPageDetailContentOuter = document.getElementById("buxErrorPageDetailContentOuter" + cvId);
		var buxErrorPageDetailsDiv = document.getElementById("buxErrorPageDetails" + cvId);
		var buxTitlePaneTitleDiv = document.getElementById("titlePaneTitle" + cvId);
		if(buxErrorPageDetailsDiv.style.display == "none") {
			dojo.removeClass(buxErrorPageDetailContentOuter, 'dijitTitlePaneContentOuterClosed');
			dojo.addClass(buxErrorPageDetailContentOuter, 'dijitTitlePaneContentOuterOpen');
			dojo.removeClass(buxTitlePaneTitleDiv, "dijitClose");
			dojo.addClass(buxTitlePaneTitleDiv, "dijitOpen");
			buxErrorPageDetailsDiv.style.display = "";
		} else {
			dojo.removeClass(buxErrorPageDetailContentOuter, 'dijitTitlePaneContentOuterOpen');
			dojo.addClass(buxErrorPageDetailContentOuter, 'dijitTitlePaneContentOuterClosed');
			dojo.removeClass(buxTitlePaneTitleDiv, "dijitOpen");
			dojo.addClass(buxTitlePaneTitleDiv, "dijitClose");
			buxErrorPageDetailsDiv.style.display = "none";
		}
	}
}

IWidgetLogonhandler = new ViewerIWidgetLogonHandler();



