/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

function DrillAction() { }
DrillAction.prototype = new ModifyReportAction();

DrillAction.prototype.getHoverClassName = function() { return ""; };

DrillAction.prototype.setKeepFocusOnWidget = function(keepFocus)
{
	this.m_bKeepFocusOnWidget = keepFocus;
};

DrillAction.prototype.keepFocusOnWidget = function()
{
	if (typeof this.m_bKeepFocusOnWidget != "undefined")
	{
		return this.m_bKeepFocusOnWidget;
	}

	return true;
};

DrillAction.prototype.getDrillabilityForItemFromReportInfo = function(itemName)
{
	var params = (this.m_oCV ? this.m_oCV.envParams : null);
	if (params) {
		var reportInfo = params["rap.reportInfo"];
		var d = null, dLen = 0;

		if(reportInfo)
		{
			var iLenReportInfo = reportInfo.length;
			for(var container = 0; container < iLenReportInfo; ++container)
			{
				d = reportInfo[container].drillability;
				if(d)
				{
					dLen = d.length;
					for(var index = 0; index < dLen; ++index)
					{
						var drillabilityInfo = d[index];
						if (drillabilityInfo.item == itemName) {
							return drillabilityInfo;
						}
					}
				}
			}
		}
	}

	return null;
};


DrillAction.prototype.onDoubleClick = function(evt)
{
	this.execute();
};

DrillAction.prototype.preProcess = function()
{
	//should only have drill spec if handling a synchronize drill event - in which
	//case we don't want to fire drill event again
	if( typeof this.m_drillSpec === "undefined" || this.m_drillSpec === null )
	{
		var drillSpec = this.generateDrillSpec();
		if( drillSpec === null )
		{
			return;
		}
		var cvWidget = this.getCognosViewer().getViewerWidget();
		var payload = {
			"cv_id": cvWidget.iContext.widgetId,
			"drillSpec": drillSpec,
			"drillAction": this.m_sAction };
		cvWidget.fireEvent( "com.ibm.bux.viewer.drill", null, payload);
	}
};

DrillAction.prototype.generateDrillSpec = function()
{
	try
	{
		var oCV = this.getCognosViewer();
		var drillMgr = oCV.getDrillMgr();
		var selectionController = oCV.getSelectionController();

		var aDrillParams = drillMgr.getDrillParameters(this.m_drillOption, true);
		if (aDrillParams.length === 0) {
			return null;
		}
		var oSelectedObject = drillMgr.getSelectedObject();
		
		if (aDrillParams.length > 3*4 && (oSelectedObject.getDataContainerType() == "crosstab" || oSelectedObject.getLayoutType() == "chartElement" )) {
			//In drillParams, which is a flat array, each context id corresponds to 4 entries (value, mun, lun, hun). 
			//For crosstab and chart, the fourth ctx id and beyond are from master-detail links,
			//need to remove them to avoid synced-drilling on them.
			aDrillParams.length = 3 * 4;
		}
		var drillSpec = drillMgr.buildDrillParametersSpecification(aDrillParams);

		var aContextIds = drillMgr.getSelectedObject().getSelectedContextIds();
		for (var drillGroupIndex = 0; drillGroupIndex < aContextIds.length; ++drillGroupIndex) {
			var ctxValue = aContextIds[drillGroupIndex][0];
			var sDataItem = selectionController.getRefDataItem(ctxValue);
			var sMUN = selectionController.getMun(ctxValue);
			var sDisplayValue = selectionController.getDisplayValue(ctxValue);
			var searchString, strCursor;

			//insert the display values.
			if (sDataItem != "" && sDisplayValue != "") {
				searchString = sDataItem + "</DataItem>";
				strCursor = drillSpec.indexOf(searchString);
				if (strCursor > 0) {
					strCursor += searchString.length;
					drillSpec = drillSpec.substring(0, strCursor) + "<DisplayValue>" + sXmlEncode(sDisplayValue) + "</DisplayValue>" + drillSpec.substring(strCursor, drillSpec.length);
				}
			}

			//insert if drilling on summary
			var drillSummary = false;
			if (sMUN != "" && selectionController.getUsageInfo(ctxValue) != '2') {
				var drillabilityObj = this.getDrillabilityForItemFromReportInfo(sDataItem);
				if ((drillabilityObj != null && drillabilityObj.disableDown == true) || this.m_oCV.getSelectionController().getDrillFlagForMember(ctxValue) == 1) {
					drillSummary = true;
				}
			}
			if (drillSummary) {
				searchString = sDataItem + "</DataItem>";
				strCursor = drillSpec.indexOf(searchString);
				if (strCursor > 0) {
					strCursor += searchString.length;
					drillSpec = drillSpec.substring(0, strCursor) + "<Summary>true</Summary>" + drillSpec.substring(strCursor, drillSpec.length);
				}
			}
		}

		return drillSpec;
	} catch( e )
	{
		return null;
	}
};

/*
 * This function parses the drill spec and creates a selected object
 * based on the spec.
 * Returns false if an exception occurs, true otherwise
 */
DrillAction.prototype.parseDrillSpec = function( evt )
{
	try
	{
		var oCV = this.getCognosViewer();
		if( oCV.getStatus() !== 'complete' || oCV.getConversation() === "")
		{
			return false;
		}

		this.m_drillSpec = evt.payload.drillSpec;
		var xmlDom = XMLBuilderLoadXMLFromString(this.m_drillSpec);
		var drillParametersNode = xmlDom.firstChild;
		var selectionController = getCognosViewerSCObjectRef(oCV.getId());
		selectionController.m_aSelectedObjects = []; //do we need to do this?

		// For some testcases (please see COGCQ00245956), especially for charts, the selected chart area is not cleaned-up properly
		// and the object of the old selection is hanging around creating strange behaviour.
		// So, clear the selection chart area if an old one is hanging around. Here is the only place to do this.
		if (selectionController.m_selectedChartArea != null)
		{
			var className = selectionController.m_selectedChartArea.className;
			if (className == "chart_area")
			{
				selectionController.setSelectedChartArea(null);
			}
		}

		var aDrillGroups = XMLHelper_FindChildrenByTagName(drillParametersNode, "DrillGroup", false);
		for(var iDrillGroupIndex = 0; iDrillGroupIndex < aDrillGroups.length; ++iDrillGroupIndex)
		{
			var munNode = XMLHelper_FindChildByTagName(aDrillGroups[iDrillGroupIndex], "MUN", false);

			var sMun = XMLHelper_GetText(munNode);
			var sLun = "";
			var sHun = "";
			var sDisplayValue = "";
			var sSummary = "";

			var displayValueNode = XMLHelper_FindChildByTagName(aDrillGroups[iDrillGroupIndex], "DisplayValue", false);
			if(displayValueNode != null)
			{
				sDisplayValue = XMLHelper_GetText(displayValueNode);
			}
			var lunNode = XMLHelper_FindChildByTagName(aDrillGroups[iDrillGroupIndex], "LUN", false);
			if(lunNode != null)
			{
				sLun = XMLHelper_GetText(lunNode);
			}

			var hunNode = XMLHelper_FindChildByTagName(aDrillGroups[iDrillGroupIndex], "HUN", false);
			if(hunNode != null)
			{
				sHun = XMLHelper_GetText(hunNode);
			}

			var summaryNode = XMLHelper_FindChildByTagName(aDrillGroups[iDrillGroupIndex], "Summary", false);
			if(summaryNode != null)
			{
				sSummary = XMLHelper_GetText(summaryNode);
			}

			this.selectObject(sMun, sLun, sHun, sDisplayValue, sSummary, selectionController );
		}
	}
	catch( e )
	{
		return false;
	}

	return( selectionController.m_aSelectedObjects.length > 0 );
};

DrillAction.prototype.selectObject = function(sMun, sLun, sHun, sDisplayValue, sOnSummary, selectionController)
{
	var sCtxId = selectionController.getCtxIdFromMun(sMun);
	var sCtxIdByMun = sCtxId;
	if(sCtxId === "")
	{
		sCtxId = selectionController.getCtxIdFromMetaData(sLun, sHun);

		if( sCtxId === "" )
		{
			return false;
		}
	}

	if((this.m_sAction == "DrillDown" && selectionController.canDrillDown(sCtxId)) || (this.m_sAction == "DrillUp" && selectionController.canDrillUp(sCtxId)))
	{
		var beforeNumber = selectionController.getSelections().length;
		selectionController.selectObject( sMun, sLun, sHun );
		var selectionObjects = selectionController.getSelections();
		if (sCtxIdByMun === "" && selectionObjects.length > beforeNumber) {
			var aMuns = selectionObjects[selectionObjects.length -1].m_aMuns;
			aMuns[aMuns.length] = [];
			aMuns[aMuns.length-1].push(sMun);
			var aDisplayValues = selectionObjects[selectionObjects.length -1].m_aDisplayValues;
			aDisplayValues.push(sDisplayValue);
			selectionObjects[selectionObjects.length -1].useDisplayValueFromObject = true;
		}
		if (sOnSummary == "true") {
			selectionObjects = selectionController.getSelections();
			selectionObjects[selectionObjects.length-1].onSummary = true;
		}
	}
};

DrillAction.prototype.addActionContextAdditionalParms = function()
{
	var params = "";

	var selectionObjects = this.getCognosViewer().getSelectionController().getSelections();
	for (var i = 0; i < selectionObjects.length; ++i)
	{
		if (selectionObjects[i].onSummary)
		{
			params += "<dataItem>" + xml_encode(selectionObjects[i].getDataItems()[0][0]) + "</dataItem>"; //expect only one because this is passed down from drill event.
		}
	}
	if (params != "") {
		params = "<onSummary>" + params + "</onSummary>";
	}
	return params;
};