/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

 function ExportAction()
{
	this.m_format = "";
	this.m_responseFormat = "downloadObject";
}

ExportAction.prototype = new CognosViewerAction();

ExportAction.prototype.getWindowTitle = function()
{
	return "";
};

ExportAction.prototype.execute = function()
{
	var formElement = this.prepareRequest();
	this.submitRequest(formElement);
};

ExportAction.prototype.prepareRequest = function()
{
	var formElement = document.createElement("form");
	this.insertGenericFormElements(formElement);
	this.insertSpecializedFormElements(formElement);
	document.body.appendChild(formElement);
	return formElement;
};

ExportAction.prototype.insertGenericFormElements = function(formElement)
{
	var sRunPrompt = "false";
	var bAction = 'cognosViewer';
	var sDispatcherURI = location.protocol +'//'+ location.host + this.m_oCV.m_sGateway;

	formElement.setAttribute("method","post");
	formElement.setAttribute("action", sDispatcherURI);
	formElement.appendChild(createHiddenFormField("b_action", bAction));
	formElement.appendChild(createHiddenFormField("cv.toolbar", "false"));
	formElement.appendChild(createHiddenFormField("cv.header", "false"));
	formElement.appendChild(createHiddenFormField("ui.windowtitleformat", 'chromeless_window_action_format'));
	formElement.appendChild(createHiddenFormField("ui.name", this.getObjectDisplayName()));
	formElement.appendChild(createHiddenFormField("cv.responseFormat", this.m_responseFormat));
	formElement.appendChild(createHiddenFormField("ui.reuseWindow", "true"));

	var sUiSpec = this.m_oCV.envParams["ui.spec"]; // TODO: we may not need this when we move to one Tomcat environment
	var sUiConversation = this.m_oCV.getConversation();

	formElement.appendChild(createHiddenFormField("ui.action", 'export'));
	formElement.appendChild(createHiddenFormField("ui.conversation", sUiConversation));
	formElement.appendChild(createHiddenFormField("run.prompt", sRunPrompt));
	formElement.appendChild(createHiddenFormField('asynch.attachmentEncoding', 'base64'));
	formElement.appendChild(createHiddenFormField("run.outputEncapsulation", 'URLQueryString'));
	formElement.appendChild(createHiddenFormField("ui.spec", sUiSpec));
	formElement.appendChild(createHiddenFormField("rap.reportInfo", this.m_oCV.envParams["json.infoBar"]));

	if (this.m_oCV.envParams["ui.routingServerGroup"]) {
		formElement.appendChild(createHiddenFormField("ui.routingServerGroup", this.m_oCV.envParams["ui.routingServerGroup"]));
	}

	var viewerWidget = this.m_oCV.getViewerWidget();
	if(viewerWidget != null)
	{
		var widgetStoreID = viewerWidget.getWidgetStoreID();
		if(typeof widgetStoreID != "undefined" && widgetStoreID != null) {
			formElement.appendChild(createHiddenFormField('widgetStoreID', widgetStoreID));
		}
		var cvGateway = viewerWidget.getAttributeValue("gateway");
		if(cvGateway) {
			formElement.appendChild(createHiddenFormField('cv.gateway', cvGateway));
		}
		var cvWebcontent = viewerWidget.getAttributeValue("webcontent");
		if(cvWebcontent) {
			formElement.appendChild(createHiddenFormField('cv.webcontent', cvWebcontent));
		}
	}
	formElement.appendChild(createHiddenFormField("rap.parametersInfo", this.buildParameterValuesSpec()));

};

ExportAction.prototype.buildParameterValuesSpec = function()
{
	var documentNode = XMLBuilderLoadXMLFromString(this.m_oCV.getExecutionParameters());
	var parameterValues = new CParameterValues();
	if(documentNode.childNodes.length == 1)
	{
		parameterValues.loadWithOptions(documentNode.childNodes[0], /*credentials*/false);
	}

	var numberOfParameters = parameterValues.length();
	var parameterValuesSpec = "<pvs>";

	if(numberOfParameters > 0)
	{
		var parameterStringOperators = new CParameterValueStringOperators(RV_RES.IDS_JS_FILTER_BETWEEN,
																		  RV_RES.IDS_JS_FILTER_NOT_BETWEEN,
																		  RV_RES.IDS_JS_FILTER_LESS_THAN,
																		  RV_RES.IDS_JS_FILTER_GREATER_THAN,
																	      RV_RES.IDS_JS_INFOBAR_AND_TEXT,
																	      this.m_oCV.buxStrings["sNotIn"]);
		for(var index = 0; index < numberOfParameters; ++index)
		{
			var parameter = parameterValues.getAt(index);
			if(parameter != null)
			{
				parameterValuesSpec += "<pv>";
				parameterValuesSpec += "<name>";
				parameterValuesSpec += xml_encode(parameter.name());
				parameterValuesSpec += "</name>";
				parameterValuesSpec += "<value>";
				parameterValuesSpec += xml_encode(parameter.toString(parameterStringOperators));
				parameterValuesSpec += "</value>";
				parameterValuesSpec += "</pv>";
			}
		}
	}

	parameterValuesSpec += "</pvs>";

	return parameterValuesSpec;
};

ExportAction.prototype.insertSpecializedFormElements = function(formElement)
{
	var viewerID = this.m_oCV.getId();
	var sName = 'get' + this.m_format + viewerID;

	formElement.setAttribute("id",sName);
	formElement.setAttribute("name", sName);
	formElement.setAttribute("target", this.m_format + 'Window' + viewerID);
	formElement.appendChild(createHiddenFormField("run.outputFormat", this.m_format));
	formElement.appendChild(createHiddenFormField("ui.windowtitleaction", this.getWindowTitle()));
};

ExportAction.prototype.submitRequest = function(formElement)
{
	var sWindowId = formElement.getAttribute("target");
	window.open("",sWindowId,'resizable=yes,menubar=no,directories=no,location=no,status=no,toolbar=no,titlebar=no');
	formElement.submit();
	document.body.removeChild(formElement);
	formElement = null;
};

ExportAction.prototype.updateMenu = function(json)
{
	json.visible = !this.isPromptWidget();
	return json;
};