/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

function GetParametersAction()
{
	this.m_payload = "";
	this.isPrimaryPromptWidget = false;
	this.m_requestParamsCopy = null;
}

GetParametersAction.prototype = new RunReportAction();

GetParametersAction.prototype.setRequestParms = function(payload)
{
	this.m_payload = payload;
};


/*
 * Adds the options to the request that is sent to the server
 */
GetParametersAction.prototype.addRequestOptions = function(asynchRequest)
{
	asynchRequest.addFormField("b_action", "cognosViewer"  );
	asynchRequest.addFormField("asynch.alwaysIncludePrimaryRequest", "false" );
	asynchRequest.addFormField("ui.action", "getParameters" );
	asynchRequest.addFormField("ui.spec", this.m_oCV.envParams["ui.spec"] );
	asynchRequest.addFormField("ui.object", this.m_oCV.envParams["ui.object"] );
	asynchRequest.addFormField("cv.showFaultPage", "false" );
	asynchRequest.addFormField("isPrimaryPromptWidget", this.isPrimaryPromptWidget? "true" : "false" );
	asynchRequest.addFormField("parameterValues", this.m_oCV.getExecutionParameters());
	if (this.m_oCV.envParams["bux"] == "true") {
		asynchRequest.addFormField("bux", "true");
	}

	if (this.m_oCV.envParams["ui.routingServerGroup"] != null) {
		asynchRequest.addFormField("ui.routingServerGroup", this.m_oCV.envParams["ui.routingServerGroup"]);
	}
};

GetParametersAction.prototype.execute = function()
{
	var cognosViewer = this.getCognosViewer();
	var callbacks = {	"complete":{ "object": this, "method": this.handleGetParametersResponse },
						"passportTimeout":{"method":function(){}}
					};
	var asynchRequest = new AsynchJSONRequest(cognosViewer.getGateway(), cognosViewer.getWebContentRoot(), callbacks);

	asynchRequest.setPromptDialog(new BuxPromptDialog(cognosViewer.getId()));
	asynchRequest.setLogonDialog(new BuxLogOnDialog());
	asynchRequest.setFaultDialog(new BuxFaultDialog());

	this.addRequestOptions( asynchRequest );

	// save prompt params
	this.m_requestParamsCopy = this.m_oCV.m_oServerRequest.m_oParams;

	asynchRequest.sendRequest();
};


GetParametersAction.prototype.handleGetParametersResponse = function (asynchResponse)
{
	try
	{
		if (this.m_requestParamsCopy && this.m_requestParamsCopy != null) {
			this.m_oCV.m_oServerRequest.m_oParams = this.m_requestParamsCopy;
			var reParam = new RegExp("^p_");
			for (var v in this.m_oCV.m_oServerRequest.m_oParams) {
				if (!v.match(reParam)) {
					delete this.m_oCV.m_oServerRequest.m_oParams[v];
				}
			}
		}

		if (this.m_oCV.m_oServerRequest.m_sAction.length == 0)
		{
			this.m_oCV.m_oServerRequest.setAction(this.m_oCV.envParams["ui.action"]);
		}

		var jsonResponse = asynchResponse.getResult();
		var response = jsonResponse.xml;

		var cognosViewer = this.getCognosViewer();
		if (typeof response != "undefined" && response != null) {
			var sReportPrompts = xml_decode(response);
			this.m_oCV.envParams["reportPrompts"] = sReportPrompts;
			if (this.isPrimaryPromptWidget) {
				this.m_oCV.raisePromptEvent(sReportPrompts);
			}
			else {
				var sharePromptAction = new SharePromptAction();
				sharePromptAction.setCognosViewer(this.getCognosViewer());
				sharePromptAction.setRequestParms(this.m_payload);
				sharePromptAction.m_bAbortAction = false;
				sharePromptAction.executePrompt();
			}
		} else {
			cognosViewer.hideExecutionCursor();
		}

		var viewerWidget = cognosViewer.getViewerWidget();
		if (typeof viewerWidget != "undefined") {
			viewerWidget.promptParametersRetrieved = true;
		}
	}
	catch(e) {	}
};
