/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

function InvokeChangeDisplayTypeDialogAction(){}

InvokeChangeDisplayTypeDialogAction.prototype = new CognosViewerAction();

InvokeChangeDisplayTypeDialogAction.prototype.execute = function() {
	var viewer = this.getCognosViewer();
	var selectedObject = this.getSelectedReportInfo();
	if(selectedObject)
	{
		var viewerWidget = viewer.getViewerWidget();
		if (typeof selectedObject.possibleDisplayTypes == "undefined")
		{
			var callbacks = {
				"complete": {}
			};
			callbacks.complete["object"] = this;
			callbacks.complete["method"] = this.handleResponse;

			var asynchRequest = new AsynchJSONRequest(this.m_oCV.getGateway(), this.m_oCV.getWebContentRoot(), callbacks);
			asynchRequest.setFaultDialog(new BuxFaultDialog());
			asynchRequest.setPromptDialog(new BuxPromptDialog(this.m_oCV.getId()));
			asynchRequest.setLogonDialog(new BuxLogOnDialog());

			asynchRequest.addFormField("ui.action", "getInfoFromReportSpec");
			asynchRequest.addFormField("bux", "true");

			if (typeof this.m_oCV.envParams["modelPath"] != "undefined" && this.m_oCV.envParams["modelPath"] != "") {
				asynchRequest.addFormField("modelPath", this.m_oCV.envParams["modelPath"]);
			}

			asynchRequest.addFormField("ui.object", this.m_oCV.envParams["ui.object"]);

			if(typeof this.m_oCV.envParams["ui.spec"] != "undefined")
			{
				asynchRequest.addFormField("ui.spec", this.m_oCV.envParams["ui.spec"]);
			}

			asynchRequest.addFormField("cv.actionContext", this.addActionContext());

			if (this.m_oCV.envParams["ui.routingServerGroup"]) {
				asynchRequest.addFormField("ui.routingServerGroup", this.m_oCV.envParams["ui.routingServerGroup"]);
			}

			viewer.showExecutionCursor();
			asynchRequest.sendRequest();
		}
		else
		{
			viewerWidget.invokeDisplayTypeDialog(selectedObject.possibleDisplayTypes );
		}
	}

};

InvokeChangeDisplayTypeDialogAction.prototype.handleResponse = function(asynchJSONResponse)
{
	var viewer = this.getCognosViewer();
	var viewerWidget = viewer.getViewerWidget();

	viewer.hideExecutionCursor();
	var reportInfos = asynchJSONResponse.getResult();
	for ( var i in reportInfos.containers)
	{
		var selectedReportInfo = this.getReportInfo(reportInfos.containers[i].container);
		selectedReportInfo.possibleDisplayTypes = reportInfos.containers[i].possibleDisplayTypes;
	}
	var selectedObject = this.getSelectedReportInfo();
	viewerWidget.invokeDisplayTypeDialog(selectedObject.possibleDisplayTypes);
};

InvokeChangeDisplayTypeDialogAction.prototype.addActionContext = function()
{
	var actionContext = "<getInfoActions>";
	actionContext += "<getInfoAction name=\"GetInfo\">";
	actionContext += "<include><possibleDisplayTypes/></include>";
	actionContext += this.addClientContextData(/*maxValuesPerRDI*/3);
	actionContext += "</getInfoAction>";
	actionContext += "</getInfoActions>";
	return actionContext;
};
