/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */


/**
 * Base class for interactive report actions
 */
function ModifyReportAction() {
	this.m_reuseConversation = true;
}
ModifyReportAction.prototype = new CognosViewerAction();

ModifyReportAction.prototype.addActionContextAdditionalParms = function() {};
ModifyReportAction.prototype.runReport = function() { return true; };
ModifyReportAction.prototype.updateRunReport = function() {};
ModifyReportAction.prototype.reuseQuery = function() { return false; };
ModifyReportAction.prototype.reuseConversation = function(reuseConversation) {
	if (typeof reuseConversation != "undefined") {
		this.m_reuseConversation = reuseConversation;
	}
	return this.m_reuseConversation;
};
ModifyReportAction.prototype.updateInfoBar = function() { return true; };
ModifyReportAction.prototype.getUndoHint = function() { return ""; };
ModifyReportAction.prototype.isUndoable = function() { return true; };
ModifyReportAction.prototype.saveSpecForUndo = function() { return false; };
ModifyReportAction.prototype.keepFocusOnWidget = function() { return true; };
ModifyReportAction.prototype.keepRAPCache = function() { return true; };
ModifyReportAction.prototype.getActionKey = function() { return null; };
ModifyReportAction.prototype.canBeQueued = function() { return false; };
ModifyReportAction.prototype.getPromptOption = function() { return "false"; };
ModifyReportAction.prototype.useActionDispatcher = function() { return true; };

ModifyReportAction.prototype.createActionDispatcherEntry = function(isAsynch)
{
	var actionDispatcherEntry = new ActionDispatcherEntry();
	actionDispatcherEntry.initializeAction(this);

	if(isAsynch == true) {
		var dummyArg = 'dummyArg';
		var callbacks = {customArguments:[dummyArg],
							"complete":{"object":actionDispatcherEntry,"method":actionDispatcherEntry.onComplete},
							"working":{"object":actionDispatcherEntry,"method":actionDispatcherEntry.onWorking},
							"fault":{"object":actionDispatcherEntry,"method":actionDispatcherEntry.onFault},
							"passportTimeout":{"object":actionDispatcherEntry,"method":actionDispatcherEntry.onFault}
						};
		actionDispatcherEntry.m_asynchRequest = this.m_oCV.createDispatcherEntryServerRequest(callbacks, isAsynch);
	} else {
		actionDispatcherEntry.m_asynchRequest = new XmlHttpObject();
	}
	return actionDispatcherEntry;
};

ModifyReportAction.prototype.executeUsingActionDispatcher = function() {
	var actionDispatcherEntry = this.createActionDispatcherEntry(true);
	actionDispatcherEntry.initialize(true);
	if(this.runReport() == false) {
		actionDispatcherEntry.addFormField("cv.responseFormat", "xml");
		this.getActionDispatcher().dispatchRequest(this, serverRequest);  // TODO: where is serverRequest?
	} else {
		actionDispatcherEntry.addFormField("cv.responseFormat", "data");

		if(this.reuseQuery() == true) {
			actionDispatcherEntry.addFormField("reuseResults", "true");
		}

		if(this.keepRAPCache() == false) {
			// delete the rap cache
			this.m_oCV.getViewerWidget().clearRAPCache();
		}

		actionDispatcherEntry.addFormField("keepIterators", "true");
		if (this.reuseConversation() == true) {
			actionDispatcherEntry.addFormField("cv.reuseConversation", "true");
		}
		actionDispatcherEntry.addFormField("cv.showFaultPage", "false");
		actionDispatcherEntry.addFormField("run.prompt", this.getPromptOption());

		var formFields = actionDispatcherEntry.m_asynchRequest.m_formFields;
		var formFieldNames = formFields.keys();
		this.m_oCV["LAST_ACTION_PARAMS"] = {
			param1: new CCognosViewerRequest(this.m_sAction),
			newFramework: true
		};
		for (var index = 0; index < formFieldNames.length; index++)
		{
			this.m_oCV["LAST_ACTION_PARAMS"].param1.addFormField(formFieldNames[index], formFields.get(formFieldNames[index]));
		}

		this.getActionDispatcher().dispatchRequest(actionDispatcherEntry);
	}

	this.fireModifiedReportEvent();
};

ModifyReportAction.prototype.isSelectSingleMember = function(selectedObject, reportInfo)
{
	var containerId = this.getContainerId(this.m_oCV.getSelectionController());

    if (reportInfo!=null && typeof reportInfo != "undefined")
    {
        var dataItems = selectedObject.getDataItems();
        if (dataItems.length > 0)
        {

            for(var container = 0; container < reportInfo.length; ++container)
            {
                if(reportInfo[container].container == containerId && typeof reportInfo[container].itemInfo != "undefined")
                {
                    for(var index = 0; index < reportInfo[container].itemInfo.length; ++index)
                    {
                        var itemInfo = reportInfo[container].itemInfo[index];
                        if (itemInfo.item == dataItems[0][0] && itemInfo.single =="true") {
                            return true;
                        }
                    }
                }
            }
        }
    }
    return false;
};


ModifyReportAction.prototype.execute = function() {
	this.getCognosViewer().setKeepFocus(this.keepFocusOnWidget());
	this.updateRunReport();
	if (this.useActionDispatcher()==true && this.getActionDispatcher() != null && this.runReport()==true) {
		this.executeUsingActionDispatcher();
		return;
	}
	var cognosViewerRequest = this.createCognosViewerRequest( "modifyReport" );

	if(this.runReport() == false)
	{
		cognosViewerRequest.addOption("cv.responseFormat", "xml");

		cognosViewerRequest.setCallback(GUtil.generateCallback(this.updateReportSpecCallback, [], this));

		this.m_oCV.sendRequest(cognosViewerRequest);
	}
	else
	{
		if(this.reuseQuery() == true)
		{
			cognosViewerRequest.addOption("reuseResults", "true");
		}

		if(this.keepRAPCache() == false)
		{
			// delete the rap cache
			this.m_oCV.getViewerWidget().clearRAPCache();
		}

		cognosViewerRequest.addOption("keepIterators", "true");
		if (this.reuseConversation() == true) {
			cognosViewerRequest.addOption("cv.reuseConversation", "true");
		}
		cognosViewerRequest.addOption("cv.showFaultPage", "false");
		cognosViewerRequest.addOption("run.prompt", this.getPromptOption());

		if (this.isUndoable())
		{
			this.getUndoRedoQueue().initUndoObj({"tooltip" : this.getUndoHint(), "saveSpec" : this.saveSpecForUndo()});
		}

		this.getCognosViewer().sendRequest(cognosViewerRequest);
	}

	this.fireModifiedReportEvent();

};

ModifyReportAction.prototype.updateReportSpecCallback = function(httpResponse) {

	this.m_oCV.hideExecutionCursor();

	var responseXML = httpResponse.responseXML;

	var response = XMLHelper_FindChildByTagName(responseXML, "response", true);
	if(response != null)
	{
		var updatedV5 = XMLHelper_GetText(response);
		this.m_oCV.envParams["ui.spec"] = updatedV5;

		// we'd sometimes add 2 items into the undo stack. One from the onclick and one from
		// the onblur. Make sure we only add one item to the undo stack
		if (!this.m_bUndoAdded)
		{
			this.m_bUndoAdded = true;
			this.getUndoRedoQueue().initUndoObj({"tooltip" : this.getUndoHint(), "saveSpec" : true});
			this.getUndoRedoQueue().add({"reportUpdated": true});
			this.getCognosViewer().getViewerWidget().updateToolbar();
		}
	}
};

/**
 * Builds the action context needed for the modifyReport action
 */
ModifyReportAction.prototype.addActionContext = function() {
	var cognosViewer = this.getCognosViewer();
	var selectionController = cognosViewer.getSelectionController();

	var actionContext = "<reportActions";

	if(this.runReport() == false)
	{
		actionContext += " run=\"false\"";
	}

	actionContext += ">";

	actionContext += "<reportAction name=\"" + this.m_sAction + "\">";

	var containerId = this.getContainerId(selectionController);
	if(containerId != "")
	{
		actionContext += "<id>" + xml_encode(containerId) + "</id>";
	}

	actionContext += this.getSelectionContext();

	var sAdditionalParms = this.addActionContextAdditionalParms();
	if( sAdditionalParms != null && sAdditionalParms != "undefined")
	{
		actionContext += sAdditionalParms;
	}


	actionContext += "</reportAction>";

	if(this.updateInfoBar())
	{
		actionContext += "<reportAction name=\"GetInfo\"/>";
	}

	actionContext += "</reportActions>";

	return actionContext;
};

ModifyReportAction.prototype.createEmptyMenuItem = function()
{
	// Temporary UI String
	return { name: "None", label: "(empty)", iconClass: "", action: null, items: null };
};

ModifyReportAction.prototype.getStateFromResponse = function(oResponse)
{
	var oResponseState = null;
	if( oResponse && typeof oResponse != "undefined" && oResponse.responseText && typeof oResponse.responseText != "undefined" && oResponse.responseText.length > 0 )
	{
		var responseXML = XMLBuilderLoadXMLFromString(oResponse.responseText);
		var stateData = responseXML.getElementsByTagName("state");
		if (stateData != null && stateData.length > 0)
		{
			try {
				if (typeof stateData[0].text != "undefined")
				{
					oResponseState = eval("(" + stateData[0].text + ")");
				}
				else
				{
					oResponseState = eval("(" + stateData[0].textContent + ")");
				}
			}
			catch(e) {
				if (typeof console != "undefined" && console && console.log) {
					console.log(e);
				}
			}
		}
	}
	return oResponseState;
};


ModifyReportAction.prototype.getSelectedCellTags = function()
{
	var params = "";
	var selectionObjects = this.getCognosViewer().getSelectionController().getSelections();
	for (var i = 0; i < selectionObjects.length; ++i)
	{
		var cellRef = selectionObjects[i].getCellRef();
		var sDataItem = selectionObjects[i].getDataItems()[0];
		if (typeof sDataItem == "undefined" || sDataItem == null)
		{
			sDataItem = "";
		}
		var tag = this.getRAPLayoutTag(cellRef);
		if (tag != null)
		{
				params += "<tag><tagValue>" + xml_encode(tag)  + "</tagValue><dataItem>" + xml_encode(sDataItem) + "</dataItem></tag>";
		}
		else
		{
			params += "<tag><tagValue/><dataItem>" + xml_encode(sDataItem) + "</dataItem></tag>";
		}
	}
	if (params != "") {
		params = "<selectedCellTags>" + params + "</selectedCellTags>";
	}
	return params;

};

ModifyReportAction.prototype.getIsNumericFromReportInfo = function(refDataItem)
{
	var containerInfo = this.getSelectedReportInfo();
	if (containerInfo != null && typeof containerInfo.itemInfo!="undefined")
	{
		//This container has filters....does it filter this item?
		for (var item = 0; item < containerInfo.itemInfo.length; ++item)
		{
			if (refDataItem == containerInfo.itemInfo[item].item &&
				typeof containerInfo.itemInfo[item].numeric != "undefined") {
				return (containerInfo.itemInfo[item].numeric == "true");
			}
		}
	}
	return false;
};
