/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

/**
 * This is the base class for refreshing the report. It'll get the latest
 * saved output or rerun the report if there's no saved output
 */
function RefreshViewAction()
{
	this.m_CognosViewerRequest = null;
	this.m_bCanvasRefreshEvent = false;
}
RefreshViewAction.prototype = new CognosViewerAction();

RefreshViewAction.prototype.addCommonOptions = function()
{
	// removed the cache CM response about the saved outputs. This will
	// force the Snapshots menu to get updated
	var widget = this.getCognosViewer().getViewerWidget();
	this.m_CognosViewerRequest = this.createCognosViewerRequest( "buxDropReportOnCanvas" );

	if (this.m_bCanvasRefreshEvent && widget.getSavedOutputSearchPath() != null)
	{
		this.m_CognosViewerRequest.addOption("ui.savedOutputSearchPath", encodeURIComponent(widget.getSavedOutputSearchPath()));
	}
	else
	{
		widget.setSavedOutputsCMResponse(null);
		widget.setSavedOutputSearchPath(null);
	}

	this.m_CognosViewerRequest.addOption("run.outputFormat", "HTML");
	this.m_CognosViewerRequest.addOption("bux", "true");

	// need since we might be going from saved output to live if the saved output is no longer available
	this.m_CognosViewerRequest.addOption("widget.reloadToolbar", "true");

	// Clear the properties dialog to it'll get rebuilt. This is needed for the 'View report specification' link in
	// case we go from saved output to live during the refresh operation
	widget.clearPropertiesDialog();

	var formWarpRequest = document.getElementById("formWarpRequest" + this.getCognosViewer().getId());
	this.m_CognosViewerRequest.addOption("ui.object", formWarpRequest["reRunObj"].value);
};

RefreshViewAction.prototype.execute = function()
{
	this.addCommonOptions();


	this.getCognosViewer().sendRequest(this.m_CognosViewerRequest);
};

RefreshViewAction.prototype.doAddActionContext = function()
{
	return false;
};

RefreshViewAction.prototype.updateMenu = function(jsonSpec)
{
	jsonSpec.disabled = false;
	var oCV = this.getCognosViewer();
	if( oCV )
	{
		var widget = oCV.getViewerWidget();
		if( widget && widget.getSavedOutputSearchPath() != null)
		{
			jsonSpec.disabled = true;
		}

	}
	return jsonSpec;
};
