/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

function ResetToOriginalAction() {}

ResetToOriginalAction.prototype = new CognosViewerAction();

ResetToOriginalAction.prototype.updateMenu = function(jsonSpec) {
	var bBaseReportIsAvailable = this.getCognosViewer().envParams.baseReportAvailable;
	jsonSpec.disabled = ( bBaseReportIsAvailable === "false" ) ? true : jsonSpec.disabled;
	return jsonSpec;
};

ResetToOriginalAction.prototype.execute = function()
{
	var confirmationDialog  = viewer.dialogs.ConfirmationDialog(
		RV_RES.IDS_JS_RESET_TO_ORIGINAL,	/* title */
		RV_RES.IDS_JS_RESET_TO_ORIGINAL_WARNING, /* main message */
		RV_RES.IDS_JS_RESET_TO_ORIGINAL_WARNING_DESC, /* description */
		null, /* icon class */
		this, /* caller object */
		this.executeAction /* yes Handler function of caller object.  */
		);

	confirmationDialog.startup();
	confirmationDialog.show();
};

ResetToOriginalAction.prototype.executeAction = function(actionObject)
{
	this.gatherFilterInfoBeforeAction("ResetToOriginal");
	ChangePaletteAction.reset(this.getCognosViewer());
};

ResetToOriginalAction.prototype.sendRequest = function(filters)
{
	var viewer = this.getCognosViewer();
	var widget = viewer.getViewerWidget();

	widget.reset();

	var sOriginalItem = widget.getAttributeValue("originalReport");
	var sOriginalReportPart = widget.getAttributeValue("originalReportPart");
	delete viewer.envParams;
	viewer.envParams = {};
	viewer.envParams["ui.object"] = sOriginalItem;
	viewer.envParams["originalReport"] = sOriginalItem;
	viewer.envParams["bux"] = "true";

	var cognosViewerRequest = this.createCognosViewerRequest( "resetToOriginal" );
	cognosViewerRequest.addOption("run.outputFormat", "HTML");
	cognosViewerRequest.addOption( "bux", "true" );
	cognosViewerRequest.addOption( "cv.id", viewer.getId() );
	cognosViewerRequest.addOption( "originalReport", sOriginalItem );
	cognosViewerRequest.addOption( "widget.reloadToolbar", "true");
	cognosViewerRequest.addOption( "ui.reportDrop", "true");

	if (filters != "") {
		cognosViewerRequest.addParameter("cv.updateDataFilters", filters);
	}
	var globalPrompts = viewer.getViewerWidget().getGlobalPromptsInfo();
	if (globalPrompts !== null ) {
		cognosViewerRequest.addOption("widget.globalPromptInfo", globalPrompts);
	}

	var bIsReportPart = (sOriginalReportPart && sOriginalReportPart.length > 0);
	if ( bIsReportPart )
	{
		cognosViewerRequest.addOption( "reportpart_id", sOriginalReportPart );
	}

	viewer.hideReportInfo();

	viewer.sendRequest( cognosViewerRequest );

	//fire the modified event
	this.fireModifiedReportEvent();
};

ResetToOriginalAction.prototype.doAddActionContext = function()
{
	return false;
};