/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

function RetryRequestAction()
{
	this.m_lastActionParams = null;
}

RetryRequestAction.prototype = new CognosViewerAction();

RetryRequestAction.prototype.setRequestParms = function(parms)
{
	this.m_lastActionParams = parms;
};

RetryRequestAction.prototype.execute = function()
{
	var requestAction;
	if(typeof this.m_lastActionParams.newFramework != "undefined") {
		var formFields = this.m_lastActionParams.param1.getFormFields();
		requestAction = formFields.get("ui.action");
		if(requestAction && requestAction != "cancel" && requestAction != "release") {
			// the request was send with the new framework, we can re-use this new framework for the retry request
			var callbacks = {"complete":{"object":this,"method":this.onComplete},
							 "prompting":{"object":this,"method":this.onComplete},
							 "fault":{"object":this,"method":this.onFault},
							 "passportTimeout":{"object":this,"method":this.onFault},
							 "working":{"object":this,"method":this.onWorking}
							 };

			var asynchRequest = this.m_oCV.createDispatcherEntryServerRequest(callbacks, true);
			this.m_oCV.showExecutionCursor();

			formFields = this.m_lastActionParams.param1.getFormFields();
			var formFieldNames = formFields.keys();
			for (var index = 0; index < formFieldNames.length; index++)
			{
				if(formFieldNames[index] != "m_tracking") {
					asynchRequest.addFormField(formFieldNames[index], formFields.get(formFieldNames[index]));
				}
			}

			asynchRequest.addFormField("cv.responseFormat", "data");
			asynchRequest.addFormField("widget.reloadToolbar", "true");

			var permissions = this.m_oCV.envParams["cv.objectPermissions"];
			if (permissions) {
				asynchRequest.addFormField("cv.objectPermissions", permissions);
			}

			asynchRequest.sendRequest();
		}
	} else {
		// the original request was sent with the old framework, we have to re-use the old framework for the retry..eventually this should be deprecated
		var oRequest = this.m_oCV["LAST_ACTION_PARAMS"].param1;
		requestAction = oRequest.getAction();
		// don't try to re-execute release or cancel requests
		if(requestAction != "cancel" && requestAction != "release") {
			this.m_oCV.reExecuteLastRequest();
		}
	}

};

RetryRequestAction.prototype.onWorking = function(asynchRequest)
{
	this.m_oCV.processResponseText(asynchRequest.getResponseText());
};

RetryRequestAction.prototype.onComplete = function(asynchRequest)
{
	this.m_oCV.hideExecutionCursor();
	this.m_oCV.processResponseText(asynchRequest.getResponseText());
};

RetryRequestAction.prototype.onFault = function(asynchRequest)
{
	this.m_oCV.setSoapFault(XMLBuilderSerializeNode(asynchRequest.getSoapFault()));
	this.m_oCV.hideExecutionCursor();
	if(this.m_oCV.getViewerWidget() != null) {
		this.m_oCV.getViewerWidget().handleFault();
	}
};
