/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

 /**
 * This is the base class for generating request to re-run the report.
 * Classes derived from this will add to it their specific options.
 */
function RunReportAction()
{
	this.m_CognosViewerRequest = null;
	this.m_reuseQuery = false;
}

RunReportAction.prototype = new CognosViewerAction();
RunReportAction.prototype.reuseQuery =function() { return this.m_reuseQuery; };
RunReportAction.prototype.setReuseQuery = function( value ) { this.m_reuseQuery = value; };
RunReportAction.prototype.getPromptOption = function() { return "false";};

RunReportAction.prototype.addCommonOptions = function()
{
	var limitedInteractiveMode = this.getCognosViewer().isLimitedInteractiveMode();
	if( typeof this.m_action === "undefined" )
	{
		this.m_action = limitedInteractiveMode  ? 'run' : 'runSpecification';
	}

	this.m_CognosViewerRequest = this.createCognosViewerRequest( this.m_action );

	if( limitedInteractiveMode )
	{
		this.m_CognosViewerRequest.addOption("run.xslURL", "bux.xsl");
	}

	this.m_CognosViewerRequest.addOption("run.outputFormat", "HTML");
	this.m_CognosViewerRequest.addOption("bux", "true");
	if (this.m_action === "runSpecification" || this.m_action === 'run') {
		this.m_CognosViewerRequest.addOption("run.prompt", this.getPromptOption());
	}
	if( this.reuseQuery() === true )
	{
		this.m_CognosViewerRequest.addOption( "reuseResults", "true" );
	}

	this.addRunOptionsFromProperties();
};

RunReportAction.prototype.addRunOptionsFromProperties = function()
{
	var properties = this.getCognosViewer().getViewerWidget().getProperties();
	if (properties.getRowsPerPage() != null)
	{
		this.m_CognosViewerRequest.addOption( "run.verticalElements", properties.getRowsPerPage() );
	}
};

RunReportAction.prototype.execute = function()
{
	this.addCommonOptions();

	if( (this.m_action === "forward" || this.m_action === "back") && ( typeof this.m_bAbortAction === "undefined" || this.m_bAbortAction === true ) )
	{
		return false;
	}

	this.getCognosViewer().sendRequest(this.m_CognosViewerRequest);
	return true;
};

RunReportAction.prototype.doAddActionContext = function()
{
	return false;
};

RunReportAction.prototype.updateMenu = function(json)
{
	json.visible = !this.isPromptWidget();
	return json;
};
