/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

 /**
 * RunSavedOutputReportAction - implements re-run in cognos viewer
 */
function RunSavedOutputReportAction(){}
RunSavedOutputReportAction.prototype = new CognosViewerAction();

RunSavedOutputReportAction.prototype.sendRequest = function(filters) {
	var cognosViewer = this.getCognosViewer();

	// The savedReportName will only be set if the user opened a saved dashboard, switched to saved output
	// and then reran the report. In this situation we need to clear the savedReportName so that when the user
	// saves the dashboard, a new report is created under the dashboard (bug COGCQ00278882)
	if (cognosViewer.envParams["savedReportName"]) {
		delete cognosViewer.envParams["savedReportName"];
	}

	// clear off the error page if this is invoked after a fault
	if( cognosViewer.getStatus() === 'fault')
	{
		var widget = this.getCognosViewer().getViewerWidget();
		widget.clearErrorDlg();
	}

	var sAction = cognosViewer.envParams["ui.action"];
	var formWarpRequest = document.getElementById("formWarpRequest" + cognosViewer.getId());

	if (cognosViewer.envParams["ui.reRunObj"])
	{
		cognosViewer.envParams["ui.object"] = cognosViewer.envParams["ui.reRunObj"];
	}
	else if (sAction == "view" && formWarpRequest && typeof formWarpRequest["reRunObj"] != "undefined" && formWarpRequest["reRunObj"] != null && formWarpRequest["reRunObj"].value.length > 0)
	{
		cognosViewer.envParams["ui.object"] = formWarpRequest["reRunObj"].value;
	}

	var oReq = new CCognosViewerRequest( 'bux' );
	oReq.addOption("widget.runFromSavedOutput", "true");
	oReq.addOption("ui.object", cognosViewer.envParams["ui.object"]);
	oReq.addOption("run.outputFormat", "HTML");
	oReq.addOption("ui.primaryAction","");
	oReq.addOption("widget.reloadToolbar", "true");

	// send down the permissions or we'll do an extra CM query to get them
	var permissions = cognosViewer.envParams["cv.objectPermissions"];
	if (permissions) {
		oReq.addOption("cv.objectPermissions", permissions);
	}

	var promptOnRerun = cognosViewer.envParams["promptOnRerun"];
	if (promptOnRerun != null)
	{
		oReq.addOption("run.prompt", promptOnRerun);
	}

	var limitedInteractiveMode = cognosViewer.envParams["limitedInteractiveMode"];
	if( limitedInteractiveMode )
	{
		oReq.addOption("limitedInteractiveMode", limitedInteractiveMode);
	}

	var globalPrompts = cognosViewer.getViewerWidget().getGlobalPromptsInfo();
	if (globalPrompts !== null ) {
		oReq.addOption("widget.globalPromptInfo", globalPrompts);
	}

	if(filters != "") {
		oReq.addOption("cv.updateDataFilters", filters);
	}

	if (cognosViewer.envParams["baseReportSearchPath"]) {
		oReq.addOption("baseReportSearchPath", cognosViewer.envParams["baseReportSearchPath"]);
	}

	// Clear the properties dialog to it'll get rebuilt. This is needed for the 'View report specification' link
	cognosViewer.getViewerWidget().clearPropertiesDialog();

	cognosViewer.preparePromptValues(oReq);
	cognosViewer.sendRequest(oReq);

	this.fireModifiedReportEvent();

	cognosViewer.envParams["ui.action"] = "run";
};

RunSavedOutputReportAction.prototype.execute = function() {
	this.gatherFilterInfoBeforeAction("RunSavedOutputReport");
};
