/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */


function SharePromptAction()
{
	this.m_bAbortAction = true;
	this.m_requestParams = null;
}

SharePromptAction.prototype = new RunReportAction();

SharePromptAction.prototype.setRequestParms = function(payload)
{
	this.m_sharePromptParameters = payload.parameters;
	this.m_action = "forward";
};

/**
 * Parse the prompt parameters, and try to match them by parameter name first or model item,
 * if a potential match is found save in an instance member to add later to the request.
 *
 * @param {Object} promptParameters
 * @return (Boolean) True if a potential match is found
 * @author Osmani Gomez
 */
SharePromptAction.prototype.parsePromptParameters = function()
{
	var result = false;
	var promptParameters = this.m_sharePromptParameters;
	var reportParameterNodes = this.getReportParameterNodes();
	if (reportParameterNodes) {
		var requestParams = {};
		for ( var i in reportParameterNodes ) {
			var modelItemMatched = false;
			for ( var j in promptParameters )
			{
				var targetModelItem = reportParameterNodes[i].getAttribute( "modelItem" );
				var targetParameterName = reportParameterNodes[i].getAttribute( "parameterName" );
				var promptParameterName = promptParameters[j].parmName;
				var promptModelItem = promptParameters[j].modelItem;
				if( (typeof targetParameterName !== "undefined" && targetParameterName === promptParameterName) ||
					 (typeof targetModelItem !== "undefined" && promptModelItem !== "undefined" && promptModelItem !== "" && targetModelItem === promptParameters[j].modelItem) )
				{
					var sValue = null;
					var promptValue = promptParameters[j].parmValue;
					var multivaluedPrompt = this._isPromptParamMultiValued(promptParameters[j].multivalued, promptValue);
					// if shared prompt param -> many, report param -> one, only one option can be used
					var hasSelectOption = new RegExp(/^<selectChoices><selectOption/);
					if ( multivaluedPrompt && reportParameterNodes[i].getAttribute( "multivalued" ) == null && promptValue.match(hasSelectOption)) {
						// make sure only one option is used, extract the the first one
						var reOneOption = new RegExp(/^(<selectChoices><selectOption.*?><)/);
						var extractedMatch = reOneOption.exec(promptValue);
						sValue = extractedMatch[1] + '/selectChoices>';
					} else {
						sValue = promptValue;
					}

					this.m_bAbortAction = false;
					result =  true;
					var sName = 'p_' + targetParameterName;
					if (targetParameterName === promptParameterName) {
						requestParams[sName] = sValue;
						break;
					} else { // match model item
						if ( !modelItemMatched ) {
							requestParams[sName] = sValue;
							modelItemMatched = true;
						}
					}
				}
			}
		}
		if (result) {
			this.m_requestParams = requestParams;
		}
	}
	return result;
};


/*
 * Safer check for multivalue, in certain cases GetParameters won't return multivalued when it should,
 *
*/
SharePromptAction.prototype._isPromptParamMultiValued = function(multiValuedParamInfo, promptValue)
{
	var result = false;
	if ( multiValuedParamInfo != "undefined" && multiValuedParamInfo) {
		result = true;
	} else {
		var reIsMulti = new RegExp(/^<selectChoices><selectOption.*?>\s*<selectOption/);
		if (promptValue.match(reIsMulti)) {
			result = true;
		}
	}
	return result;
};

SharePromptAction.prototype.getReportParameterNodes = function()
{
	var cv = this.getCognosViewer();
	var reportParameterNodes = null;
	try {
		if (cv.envParams && cv.envParams.reportPrompts) {
			var reportParameters = cv.envParams.reportPrompts;
			var xmlDom = XMLBuilderLoadXMLFromString(reportParameters);
			if (!(xmlDom && xmlDom.childNodes && xmlDom.childNodes.length > 0 && xmlDom.childNodes[0].nodeName === "parsererror")) {
				var cvTransientSpec = xmlDom.firstChild;
				var reportParametersNode = XMLHelper_FindChildByTagName(cvTransientSpec, "reportParameters", true);
				reportParameterNodes = XMLHelper_FindChildrenByTagName(reportParametersNode, "reportParameter", false);
			}
		}
	}
	catch ( e ) {}
	return reportParameterNodes;
};

SharePromptAction.prototype.addAdditionalOptions = function(cognosViewerRequest)
{

	var cv = this.getCognosViewer();
	if( cv.envParams && cv.envParams.reportPrompts )
	{

		cognosViewerRequest.addOption( "run.prompt", "false");
		cv.m_raiseSharePromptEvent = false;

		this.addRequestParameters( cognosViewerRequest );
	}

};

SharePromptAction.prototype.addRequestParameters = function( cognosViewerRequest )
{
	if (this.m_requestParams)
	{
		cognosViewerRequest.addFormField("sharedPromptRequest", "true");
		for (var p in this.m_requestParams)
		{
			cognosViewerRequest.addParameter( p, this.m_requestParams[p] );
		}
	}
	else
	{
		this.getCognosViewer().hideExecutionCursor();
	}
};


SharePromptAction.prototype.executePrompt = function ()
{
	var cognosViewer = this.getCognosViewer();
	if (this.parsePromptParameters())
	{
		cognosViewer.showExecutionCursor();
		this.execute();
	} else
	{
		cognosViewer.hideExecutionCursor();
	}
};
