/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

function SnapshotsAction() {}
SnapshotsAction.prototype = new CognosViewerAction();

SnapshotsAction.prototype.updateMenu = function(jsonSpec) {
	var widget = this.m_oCV.getViewerWidget();
	jsonSpec.disabled = (widget.getAttributeValue("fromReportPart") == "true") || (this.m_oCV.envParams["reportpart_id"] && this.m_oCV.envParams["reportpart_id"].length) > 0 ? true : false;
	jsonSpec.visible = !this.isPromptWidget();
	return jsonSpec;
};

SnapshotsAction.prototype.execute = function()
{
	var oCV = this.getCognosViewer();
	var widget = oCV.getViewerWidget();
	if (widget.getSavedOutputsCMResponse() == null)
	{
		var searchPath = "";
		var formWarpRequest = document.getElementById("formWarpRequest" + oCV.getId());
		if (oCV.envParams["originalReport"])
		{
			searchPath = oCV.envParams["originalReport"];
		}
		else if (formWarpRequest && formWarpRequest["reRunObj"] != null && formWarpRequest["reRunObj"].value.length > 0)
		{
			searchPath = formWarpRequest["reRunObj"].value;
		}
		else
		{
			searchPath = oCV.envParams["ui.object"];
		}
		searchPath += "/reportVersion/*[@format='HTML' or @format='XHTML']/..";

		var cmRequest =
			"<CMRequest>" +
				"<searchPath>" + xml_encode(searchPath) + "</searchPath>" +
				"<properties>" +
					"<property>searchPath</property>" +
					"<property>creationTime</property>" +
					"<property>storeID</property>" +
				"</properties>" +
				"<sortBy>" +
					"<sortItem>" +
						"<property>creationTime</property>" +
						"<order>descending</order>" +
					"</sortItem>" +
				"</sortBy>" +
			"</CMRequest>";

		var cognosViewerRequest = new CCognosViewerRequest("CMRequest");
		cognosViewerRequest.addOption("b_action", "cognosViewer");
		cognosViewerRequest.addOption("ui.action", "CMRequest");
		cognosViewerRequest.addOption("cv.responseFormat", "CMRequest");
		cognosViewerRequest.addOption("ui.object", searchPath);
		cognosViewerRequest.addOption("CMRequest", cmRequest);
		cognosViewerRequest.setCallback( this.getCallback() );

		this.getCognosViewer().sendRequest(cognosViewerRequest);
	} else if (typeof widget.savedOutputMenuUpdated != "undefined" && widget.savedOutputMenuUpdated == false) {
		this.populateMenu(true);
		widget.savedOutputMenuUpdated = true;
	}
};

SnapshotsAction.prototype.getCallback = function()
{
	var viewer = this.getCognosViewer();
	var action = this;

	var callback = function(response) {
		action.handleQueryForSnapshotsResponse(response);
		viewer.hideExecutionCursor();
	};

	return callback;
};

SnapshotsAction.prototype.handleQueryForSnapshotsResponse = function(response) {
	var oCV = this.getCognosViewer();
	var widget = oCV.getViewerWidget();

	var sContentType = response.getResponseHeader("Content-type");
	// if we got html back it must be a fault.
	if(sContentType.indexOf("text/html") != -1)
	{
		var responseText = response.responseText;
		if(responseText.indexOf("<ERROR_CODE>CAM_PASSPORT_ERROR</ERROR_CODE>") != -1) {
			oCV.hideExecutionCursor();
			var logOnDialog = new BuxLogOnDialog();
			logOnDialog.show();
		} else {
			// unknown/unexpected html response, throw it up in a new window
			var htmlWindow = window.open("","",'height=400,width=500');
			if(htmlWindow != null) {
				htmlWindow.document.write(responseText);
			}
		}
	}
	else
	{
		var responseXML = response.responseXML;
		if (oCV.isValidAjaxResponse(responseXML)) {
			var fault = responseXML.getElementsByTagName("fault");
			if (fault != null && fault.length > 0) {
				oCV.setSoapFault(XMLBuilderSerializeNode(fault[0]));
				oCV.hideExecutionCursor();
				widget.handleFault();
			}
			else {
				var cmResponse = responseXML.getElementsByTagName("cmResponse");
				var xmlParsedCMresponse = null;
				if (typeof cmResponse[0].text != "undefined") {
					xmlParsedCMresponse = XMLBuilderLoadXMLFromString(cmResponse[0].text);
				} else {
					xmlParsedCMresponse = XMLBuilderLoadXMLFromString(cmResponse[0].textContent);
				}

				widget.setSavedOutputsCMResponse(xmlParsedCMresponse);
				this.populateMenu(true);
			}
		}
	}
};

SnapshotsAction.prototype.populateMenu = function(bOpen) {
	var oCV = this.getCognosViewer();
	var widget = oCV.getViewerWidget();
	var sAction = oCV.envParams["ui.action"];

	var cmResponse = widget.savedOutputsCMResponse;

	var root = oCV.findBlueDotMenu();
	root.open = bOpen;
	var buttonSpec = oCV.findToolbarItem("Snapshots", root);

	var queryResult = XMLHelper_FindChildByTagName(cmResponse, "queryResult", true);
	var queryItems = XMLHelper_FindChildrenByTagName(queryResult, "item", false);

	var menuItems = [];
	var mostRecentCreationTime = null;
	var mostRecentStoreID = null;
	var oMenuItem = null;
	var bSameAsCurrent;

	if ( oCV.isLimitedInteractiveMode() || (oCV.envParams["cv.objectPermissions"] && oCV.envParams["cv.objectPermissions"].indexOf("execute") != -1) ) {
		bSameAsCurrent = (sAction != "view" && sAction != "buxView" && oCV.getStatus() !== "fault");

		oMenuItem = { name: "live", label: RV_RES.IDS_JS_SNAPSHOTS_LIVE, action: bSameAsCurrent ? {} : { name: "RunSavedOutputReport", payload: {} }, items: null };
		this.addMenuItemChecked(bSameAsCurrent, oMenuItem);
		menuItems.push(oMenuItem);
		if (queryItems.length > 0) {
			menuItems.push({separator: true});
		}
	}

	if (queryItems.length > 0) {
		var versions = [];
		for (var iIndex=0; iIndex < queryItems.length; iIndex++) {
			if (iIndex < 5) {
				var queryItem = queryItems[iIndex];
				var sItemLabel = XMLHelper_GetText(XMLHelper_FindChildByTagName(queryItem, "creationTime_localized", true));

				var storeIDNode = XMLHelper_FindChildByTagName(queryItem, "storeID", true);
				var sStoreID = XMLHelper_GetText(XMLHelper_FindChildByTagName(storeIDNode, "value", true));

				var creationTimeNode = XMLHelper_FindChildByTagName(queryItem, "creationTime", true);
				var sCreationTime = XMLHelper_GetText(XMLHelper_FindChildByTagName(creationTimeNode, "value", true));

				if (mostRecentCreationTime == null) {
					mostRecentCreationTime = sCreationTime;
					mostRecentStoreID = sStoreID;
				}

				bSameAsCurrent = sAction == "view" && oCV.envParams["creationTime"] == sCreationTime && widget.getSavedOutputSearchPath() != null;
				oMenuItem = { name: "savedOutput", label: sItemLabel, action: bSameAsCurrent ? {} : { name: "ViewSavedOutput", payload: {obj:sStoreID, creationTime:sCreationTime, mostRecent:false} }, items: null };
				this.addMenuItemChecked(bSameAsCurrent, oMenuItem);
				versions.push(oMenuItem);
			}
			else {
				versions.push({ name: "viewAllSnapshots", label: RV_RES.IDS_JS_VIEW_ALL_SNAPSHOTS, action: { name: "ViewAllSnapshots", payload: {} }, items: null });
				break;
			}
		}

		bSameAsCurrent = false;
		if (widget.getSavedOutputSearchPath() == null && sAction == "view") {
			bSameAsCurrent = true;
		}

		oMenuItem = { name: "savedOutput", label: RV_RES.IDS_JS_MOST_RECENT_SNAPSHOT, action: bSameAsCurrent ? {} : { name: "ViewSavedOutput", payload: {obj:mostRecentStoreID, creationTime:mostRecentCreationTime, mostRecent:true} }, items: null };
		this.addMenuItemChecked(bSameAsCurrent, oMenuItem);
		menuItems.push(oMenuItem);
		menuItems.push({separator: true});
		menuItems = menuItems.concat(versions);
	}

	buttonSpec.open = bOpen;
	buttonSpec.items = menuItems;

	var updateItems = [];
	updateItems.push(buttonSpec);
	widget.fireEvent("com.ibm.bux.widgetchrome.toolbar.update", null, updateItems);
};

SnapshotsAction.prototype.resetMenu = function(bOpen) {
	var oCV = this.getCognosViewer();
	var widget = oCV.getViewerWidget();

	var root = oCV.findBlueDotMenu();
	var buttonSpec = oCV.findToolbarItem("Snapshots", root);
	if (buttonSpec) {
		buttonSpec.open = false;

		var menuItems = [{ name: "loadng", label: RV_RES.GOTO_LOADING, iconClass: "loading"}];
		buttonSpec.items = menuItems;

		var updateItems = [buttonSpec];
		widget.fireEvent("com.ibm.bux.widgetchrome.toolbar.update", null, updateItems);
	}
};
