/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

function ViewSavedOutputAction()
{
	this.m_obj = "";
	this.creationTime = "";
	this.m_mostRecent = false;
}
ViewSavedOutputAction.prototype = new CognosViewerAction();

ViewSavedOutputAction.prototype.setRequestParms = function(payload) {
	this.m_obj = payload.obj;
	this.creationTime = payload.creationTime;
	this.m_mostRecent = payload.mostRecent;
};

ViewSavedOutputAction.prototype.updateMenu = function() {
	// update the menu so the saved output that's going to be viewed is shown as selected
	var snapshotsAction = new SnapshotsAction();
	snapshotsAction.setCognosViewer(this.getCognosViewer());
	snapshotsAction.populateMenu(false);
};

ViewSavedOutputAction.prototype.execute = function() {
	var cognosViewer = this.getCognosViewer();

	if( cognosViewer.getStatus() === 'fault')
	{
		var widget = this.getCognosViewer().getViewerWidget();
		widget.clearErrorDlg();
	}

	// clear the global prompt information
	cognosViewer.getViewerWidget().setPromptParametersRetrieved(false);
	cognosViewer.envParams["reportPrompts"] = "";

	var sAction = cognosViewer.envParams["ui.action"];
	var formWarpRequest = document.getElementById("formWarpRequest" + cognosViewer.getId());
	if (sAction == "view" && formWarpRequest && formWarpRequest.reRunObj && formWarpRequest.reRunObj.value ) {
		cognosViewer.envParams["ui.reRunObj"] = formWarpRequest["reRunObj"].value;
	} else if (sAction != "view") {
		cognosViewer.envParams["ui.reRunObj"] = cognosViewer.envParams["ui.object"];
	}

	var searchPath = "storeID('" + this.m_obj + "')";
	cognosViewer.envParams["ui.action"] = "buxView";
	cognosViewer.envParams["ui.object"] = cognosViewer.envParams["ui.reRunObj"];
	cognosViewer.envParams["creationTime"] = this.creationTime;

	if (this.m_mostRecent == true) {
		cognosViewer.getViewerWidget().setSavedOutputSearchPath(null);
	}
	else {
		cognosViewer.getViewerWidget().setSavedOutputSearchPath(searchPath);
	}

	// update the menu so the saved output that's going to be viewed is shown as selected
	this.updateMenu();

	// clear the undo queue
	this.getUndoRedoQueue().clearQueue();

	// Clear the properties dialog to it'll get rebuilt. This is needed for the 'View report specification' link
	cognosViewer.getViewerWidget().clearPropertiesDialog();

	this.sendRequest(searchPath);

	this.fireModifiedReportEvent();
};

ViewSavedOutputAction.prototype.sendRequest = function(searchPath) {
	var cognosViewerRequest = new CCognosViewerRequest("buxView");
	// we need to include the report name or we'll end up doing 2 CM queries.
	cognosViewerRequest.addOption("ui.name", this.m_oCV.envParams["ui.name"]);
	cognosViewerRequest.addOption("widget.reloadToolbar", "true");
	cognosViewerRequest.addOption("cv.objectPermissions", this.m_oCV.envParams["cv.objectPermissions"]);
	cognosViewerRequest.addOption("ui.savedOutputSearchPath", searchPath);

	this.getCognosViewer().sendRequest(cognosViewerRequest);
};
