/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */
 function EditContentAction()
{
	this.m_animationDuration = 600;
	this.m_iId = null;
}

EditContentAction.prototype = new CognosViewerAction();
EditContentAction.superclass = CognosViewerAction.prototype;

EditContentAction.prototype.execute = function()
{
	if (typeof this.preferencesChanged != "undefined" && this.preferencesChanged != null && this.preferencesChanged === true )
	{
		this.removeBUAIFrame();
		return;
	}

	this.showBlocker();
	this.buildBUAObjects();
	this.addWindowEventListeners();

	this.m_iId = this.m_oCV.getId();
	var formWarpRequest = document.getElementById("formWarpRequest" + this.m_oCV.getId());
	var containerDiv = this.buildContainerDiv("buaAnimationDiv");

	containerDiv.style.top = window["CVEditContentActionInstance"].cachedDivProperties.top + "px";
	containerDiv.style.left = window["CVEditContentActionInstance"].cachedDivProperties.left + "px";
	containerDiv.style.width = window["CVEditContentActionInstance"].cachedDivProperties.width + "px";
	containerDiv.style.height = window["CVEditContentActionInstance"].cachedDivProperties.height + "px";
	containerDiv.style.zIndex = 6002;

	var sParentClass = formWarpRequest.parentNode.className;
	formWarpRequest.parentNode.removeChild(formWarpRequest);
	document.body.appendChild(containerDiv);
	var oContentPlaceholder = document.getElementById("buaAnimationDivContentPlaceholder");
	oContentPlaceholder.appendChild(formWarpRequest);
	oContentPlaceholder.className = sParentClass;

	var selectionController = getCognosViewerSCObjectRef(this.m_iId);
	selectionController.clearSelectedObjects();

	var slideLeftAnimation = dojo.fx.slideTo({node: "buaAnimationDiv",duration: this.m_animationDuration, left: 4, top: 4});

	var buaBlocker = document.getElementById("buaBlocker");

	var explodeAnimation = dojo.animateProperty( {
		node : "buaAnimationDiv",
		duration: this.m_animationDuration,
		properties : {
			height : {start: window["CVEditContentActionInstance"].cachedDivProperties.height, end : buaBlocker.clientHeight - 8, units:"px"}, //fill in whatever height/width you need...
			width  : {start: window["CVEditContentActionInstance"].cachedDivProperties.width, end : buaBlocker.clientWidth - 8, units:"px"}
		},
		onEnd: function(){
			window["CVEditContentActionInstance"].loadBUAIframe();
		}
	});

	dojo.fx.combine([slideLeftAnimation,explodeAnimation]).play();
};

EditContentAction.prototype.addWindowEventListeners = function()
{
	if (document.all)
	{
		window.attachEvent("onresize", window["CVEditContentActionInstance"].onWindowResize);
	}
	else
	{
		window.addEventListener("resize", window["CVEditContentActionInstance"].onWindowResize, false);
	}
};

EditContentAction.prototype.removeWindowEventListeners = function()
{
	if (document.all)
	{
		window.detachEvent("onresize", window["CVEditContentActionInstance"].onWindowResize);
	}
	else
	{
		window.removeEventListener("resize", window["CVEditContentActionInstance"].onWindowResize, false);
	}
};

EditContentAction.prototype.onWindowResize = function()
{
	var buaBlocker = document.getElementById("buaBlocker");
	if(buaBlocker != null)
	{
		var buaApplicationDiv = document.getElementById("buaApplicationDiv");
		buaApplicationDiv.style.width = (buaBlocker.clientWidth - 8) + "px";
		buaApplicationDiv.style.height = (buaBlocker.clientHeight - 8) + "px";

		var buaAnimationDiv = document.getElementById("buaAnimationDiv");
		buaAnimationDiv.style.width = (buaBlocker.clientWidth - 8) + "px";
		buaAnimationDiv.style.height = (buaBlocker.clientHeight - 8) + "px";
	}
};

EditContentAction.prototype.getViewerId = function()
{
	return this.m_iId;
};

EditContentAction.prototype.buildBUAObjects = function()
{
	window["RSParameters"] = this.buildBUALaunchParameters();
	window["CVEditContentActionInstance"] = this;

	var formWarpRequest = document.getElementById("formWarpRequest" + this.m_oCV.getId());

	var current = formWarpRequest;
	var topCoord = 0, leftCoord = 0;
	while(current != null)
	{
		topCoord += current.offsetTop; leftCoord += current.offsetLeft;
		current = current.offsetParent;
	}

	// pad the coords to allow spacing for the borders
	window["CVEditContentActionInstance"].cachedDivProperties = { parentNode: formWarpRequest.parentNode, left: leftCoord - 6, top: topCoord - 6, width: formWarpRequest.clientWidth + 12, height: formWarpRequest.clientHeight + 12 };
};

EditContentAction.prototype.buildContainerDiv = function(id)
{
	var divElement = document.getElementById(id);
	if(divElement == null)
	{
		divElement = document.createElement("div");
		divElement.setAttribute("id", id);
		divElement.style.position = "absolute";
		divElement.className = "buxWidgetActive buxWidgetTitleNone";
		divElement.style.overflow = "hidden";

		divElement.innerHTML =
			"<table style=\"height: 100%;\" class=\"buxWidgetFrame\" border=\"0\" cellpadding=\"0\" cellspacing=\"6\"><tbody><tr><td height=\"100%\">" +
				"<div role=\"region\" wairole=\"region\" tabindex=\"-1\" class=\"dojoxFloatingPaneContent\" style=\"overflow: auto;\">" +
					"<div style=\"height: 100%\" class=\"mm_iWidget\">" +
						"<div style=\"height: 100%\" class=\"mm_Content view\">" +
							"<div style=\"width:100%; height: 100%; display: block;\" id=\"" + id + "ContentPlaceholder\">" +
							"</div>" +
						"</div>" +
					"</div>" +
				"</div>" +
			"</td></tr></tbody></table>" +

			'<div class="widget-border wb-top"></div>'+
			'<div class="widget-border wb-left"></div>'+
			'<div class="widget-border wb-right"></div>'+
			'<div class="widget-border wb-bottom"></div>'+
			'<div class="widget-corner wc-TL"></div>'+
			'<div class="widget-corner wc-BL"></div>'+
			'<div class="widget-corner wc-TR"></div>'+
			'<div class="widget-corner wc-BR"></div>';
	}

	return divElement;
};

EditContentAction.prototype.removeLoadingToolsMessage = function()
{
	var loadingToolsMessage = document.getElementById("loadingBuaTools");
	if(loadingToolsMessage != null)
	{
		loadingToolsMessage.parentNode.removeChild(loadingToolsMessage);
	}
};

EditContentAction.prototype.showLoadingToolsMessage = function()
{
	var oCV = this.getCognosViewer();
	var divElement = document.createElement("div");
	divElement.setAttribute("id", "loadingBuaTools");
	divElement.style.position = "absolute";
	divElement.style.visibility = "hidden";
	divElement.style.zIndex = 6004;
	divElement.innerHTML =
			'<div class="bux-menu-border" style="padding: 20px; background-color: #FFFFFF;">' +
				'<table border="0" tabindex="-1" style="margin: 10px;" wairole="region" role="region"><tr>' +
				'<td>' +
					'<img alt="" height="32" width="32" src="' + oCV.getWebContentRoot() + '/rv/images/action_busy_32x32.gif"/>' +
				'</td><td style="padding-left:10px;">' +
					RV_RES.IDS_JS_LOADING_EDITING_TOOLS +
					"<br/><a id=\"cancelBUALoading\" href=\"#\" onclick=\"window['CVEditContentActionInstance'].cancelBUALoad();return false;\">" +
						RV_RES.CANCEL +
					"</a></td></tr></table>" +
				'<div class="dlg-border dlg-top"></div>'+
				'<div class="dlg-border dlg-left"></div>'+
				'<div class="dlg-border dlg-right"></div>'+
				'<div class="dlg-border dlg-bottom"></div>'+

				'<div class="dlg-corner dlg-TL"></div>'+
				'<div class="dlg-corner dlg-BL"></div>'+
				'<div class="dlg-corner dlg-TR"></div>'+
				'<div class="dlg-corner dlg-BR"></div>'+
			"</div>";

	document.body.appendChild(divElement);

	var buaBlocker = document.getElementById("buaBlocker");
	if(buaBlocker != null)
	{
		var leftCoord = (buaBlocker.clientWidth / 2) - (divElement.clientWidth / 2);
		var topCoord = (buaBlocker.clientHeight / 2) - (divElement.clientHeight / 2);

		divElement.style.left = leftCoord + "px";
		divElement.style.top = topCoord + "px";
	}

	divElement.style.visibility = "";

	var cancelLink = document.getElementById("cancelBUALoading");
	cancelLink.onkeypress = function(e) { window['CVEditContentActionInstance'].cancel_keypress(e); };
	cancelLink.focus();
};

EditContentAction.prototype.cancel_keypress = function(e)
{
	e = e || window.event;
	if (e.keyCode == 9)
	{
		e.returnValue = false;
		e.cancelBubble = true;
		setTimeout(function() {document.getElementById("cancelBUALoading").focus();}, 0);
	}
};

EditContentAction.prototype.showReportBlocker = function(zIndex, sBackgroundColor)
{
	var buaAnimationDiv = document.getElementById("buaAnimationDiv");
	if(buaAnimationDiv != null)
	{
		var blocker = document.createElement("div");
		blocker.setAttribute("id","reportBlocker");
		blocker.setAttribute("name", "reportBlocker");
		blocker.setAttribute("tabIndex", "1");
		blocker.className = "reportBlocker";
		blocker.style.zIndex = zIndex;
		blocker.style.backgroundColor = sBackgroundColor;

		blocker.style.top = (parseInt(buaAnimationDiv.style.top,10) + 6) + "px";
		blocker.style.left = (parseInt(buaAnimationDiv.style.left,10) + 6) + "px";
		blocker.style.width = (buaAnimationDiv.clientWidth - 12) + "px";
		blocker.style.height = (buaAnimationDiv.clientHeight - 12) + "px";

		document.body.appendChild(blocker);
	}
};

EditContentAction.prototype.loadBUAIframe = function()
{
	this.showReportBlocker(6003, "#FFFFFF");

	this.showLoadingToolsMessage();

	var buaIframe = this.getBUAIframe();
	var buaAnimationDiv = document.getElementById("buaAnimationDiv");

	var buaApplicationDiv = this.buildContainerDiv("buaApplicationDiv");
	buaApplicationDiv.style.top = buaAnimationDiv.style.top;
	buaApplicationDiv.style.left = buaAnimationDiv.style.left;
	buaApplicationDiv.style.width = buaAnimationDiv.style.width;
	buaApplicationDiv.style.height = buaAnimationDiv.style.height;
	buaApplicationDiv.style.display = "";

	if(buaIframe == null)
	{
		var iframeElement = this.buildBUAIFrame();
		document.body.appendChild(buaApplicationDiv);
		var container = document.getElementById("buaApplicationDivContentPlaceholder");
		container.appendChild(iframeElement);
	}
	else
	{
		this.setReportSettings();
	}
};

EditContentAction.prototype.getBUAIframe = function()
{
	return document.getElementById("buaIframe");
};

EditContentAction.prototype.getBUAWindow = function()
{
	var buaWindow = null;
	var buaIframe = this.getBUAIframe();
	if(buaIframe != null)
	{
		buaWindow = buaIframe.contentWindow;
	}

	return buaWindow;
};

EditContentAction.prototype.setReportSettings = function()
{
	var oCV = this.getCognosViewer();
	var widget = oCV.getViewerWidget();

	//Fire an IWidget event to get the title
	widget.fireEvent("com.ibm.bux.widget.getDisplayTitle", null, {callback: function(sTitle)
																			{
																				window["CVEditContentActionInstance"].openReportWithBUA(sTitle);
																			}
																  }
	);
};

EditContentAction.prototype.openReportWithBUA = function(sTitle)
{
	var buaWindow = this.getBUAWindow();

	var subStringIndex = this.m_oCV.envParams["ui.spec"].indexOf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	if (subStringIndex == -1) { subStringIndex = 0; } else { subStringIndex = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>".length; }
	var sReportXML = this.m_oCV.envParams["ui.spec"].substr(subStringIndex, this.m_oCV.envParams["ui.spec"].length);

	if (buaWindow.Application.SetBUAContext)
	{
		var oContext = {};
		oContext["tracking"] = this.m_oCV.getTracking();
		oContext["displayName"] = xml_decode(sTitle); //Chrome HTML encoded this sTtitle 
		oContext["parameterValues"] = this.m_oCV.getExecutionParameters();
		oContext["reportXML"] = sReportXML;
		oContext["showOpenTransition"] = true;

		setTimeout( dojo.hitch(this, this.setBUAContext, oContext), 0);
	}
	else
	{
		//to be deprecated
		buaWindow.Application.SetTracking(this.m_oCV.getTracking());
		buaWindow.Application.SetDisplayName(sTitle);
		buaWindow.Application.SetParameterValues(this.m_oCV.getExecutionParameters());
		buaWindow.Application.OpenReportFromXml(sReportXML, true);
	}
};

EditContentAction.prototype.setBUAContext = function(oContext)
{
	var buaWindow = this.getBUAWindow();
	buaWindow.Application.SetBUAContext(oContext);
};

EditContentAction.prototype.buildBUALaunchParameters = function()
{
	var RSParameters = {};
	RSParameters["rs_UIProfile"] = "BUA";
	RSParameters["ui.action"] = "edit";
	RSParameters["gateway"] = location.protocol + "//" + location.host + this.m_oCV.getGateway();
	RSParameters["theme"] = "bux";
	RSParameters["capabilitiesXML"] = this.m_oCV.capabilitiesXML;

	var cafContextId = "";
	if(typeof this.m_oCV.cafContextId != "undefined")
	{
		cafContextId = this.m_oCV.cafContextId;
	}

	RSParameters["cafcontextid"] = cafContextId;

	RSParameters["paneOnRight"] = this.m_oCV.getViewerWidget().getPaneOnRight();

	var viewerWidget = this.m_oCV.getViewerWidget();
	if(viewerWidget != null)
	{
		var cvGateway = viewerWidget.getAttributeValue("gateway");
		if(cvGateway) {
			RSParameters["cv.gateway"] = cvGateway;
		}
		var cvWebcontent = viewerWidget.getAttributeValue("webcontent");
		if(cvWebcontent) {
			RSParameters["cv.webcontent"] = cvWebcontent;
		}
	}

	return RSParameters;
};


EditContentAction.prototype.showBlocker = function()
{
	var buaBlocker = document.getElementById("buaBlocker");
	if(buaBlocker == null)
	{
		buaBlocker = this.buildBlocker();
		document.body.appendChild(buaBlocker);
	}
	else
	{
		buaBlocker.style.display = "";
	}
};

EditContentAction.prototype.removeBlockers = function()
{
	var buaBlocker = document.getElementById("buaBlocker");
	if(buaBlocker != null)
	{
		buaBlocker.parentNode.removeChild(buaBlocker);
	}

	var reportBlocker = document.getElementById("reportBlocker");
	if(reportBlocker != null)
	{
		reportBlocker.parentNode.removeChild(reportBlocker);
	}
};

EditContentAction.prototype.removeBUAIFrame = function()
{
	if (this.getBUAIframe() != null)
	{
		var iFrameNode = document.getElementById("buaIframe");
		var parentIFrameNode = null;
		if (iFrameNode != null)
		{
			parentIFrameNode = iFrameNode.parentNode;
		}
		if (parentIFrameNode != null)
		{
			parentIFrameNode.removeChild(iFrameNode);
		}
	}
};

EditContentAction.prototype.buildBlocker = function()
{
	var blocker = document.createElement("div");
	blocker.setAttribute("id","buaBlocker");
	blocker.setAttribute("name","buaBlocker");
	blocker.setAttribute("tabIndex", "1");
	blocker.className = "blocker";
	return blocker;
};

EditContentAction.prototype.buildBUAIFrame = function()
{
	var iframeElement = document.createElement("iframe");
	iframeElement.setAttribute("id","buaIframe");
	iframeElement.setAttribute("src", this.m_oCV.sWebContentRoot + "/pat/rsapp.htm");
	iframeElement.setAttribute("name", "buaIframe");
	iframeElement.setAttribute("frameborder",'0');
	iframeElement.className = "buaIframe";

	return iframeElement;
};

EditContentAction.prototype.cleanUp = function()
{
	var reportDiv = document.getElementById("formWarpRequest" + this.m_oCV.getId());
	var animationDivPlaceholder = document.getElementById("buaAnimationDivContentPlaceholder");
	var animatedDiv = document.getElementById("buaAnimationDiv");
	animationDivPlaceholder.removeChild(reportDiv);
	document.body.removeChild(animatedDiv);

	this.removeWindowEventListeners();

	window["CVEditContentActionInstance"].cachedDivProperties.parentNode.appendChild(reportDiv);
	window["CVEditContentActionInstance"].cachedDivProperties = null;
	window["CVEditContentActionInstance"] = null;
};

EditContentAction.prototype.hideBUA = function()
{
	var buaApplicationDiv = document.getElementById("buaApplicationDiv");
	buaApplicationDiv.style.display = "none";
	buaApplicationDiv.style.zIndex = 6000;

	window["CVEditContentActionInstance"].restoreDiv();
};

EditContentAction.prototype.runUpdatedReportFromBUA = function()
{
	var buaWindow = this.getBUAWindow();
	var bIsSpecModified = null;
	var originalSpec = this.m_oCV.envParams["ui.spec"];

	var oContext = buaWindow.Application.GetBUAContext();
	this.m_oCV.envParams["ui.spec"] = oContext["reportXML"];
	this.m_oCV.setTracking(oContext["tracking"]);
	this.m_oCV.setExecutionParameters(oContext["parameterValues"]);
	bIsSpecModified = oContext["isSpecModified"];

	if (bIsSpecModified)
	{
		this.getUndoRedoQueue().setOriginalSpec(originalSpec);
		var redrawAction = new RedrawAction();
		redrawAction.setCognosViewer(this.m_oCV);
		redrawAction.execute();
	}
	else
	{
		this.m_bReadyToHideBUA = true;
	}
};

// deprecated
EditContentAction.prototype.closeBUA = function(bIgnoreBUA)
{
	if (bIgnoreBUA !== true)
	{
		var rsWindow = this.getBUAWindow();
		this.m_oCV.envParams["ui.spec"] = rsWindow.Application.Document.GetReportXml();
		this.m_oCV.setTracking(rsWindow.Application.GetTracking());
		this.m_oCV.setExecutionParameters(rsWindow.Application.GetParameterValues());

		var buaWindow = this.getBUAWindow();
		if (typeof buaWindow.Application.IsSpecModified != "function" || buaWindow.Application.IsSpecModified())
		{
			var redrawAction = new RedrawAction();
			redrawAction.setCognosViewer(this.m_oCV);
			redrawAction.execute();
		}
		else
		{
			this.m_bReadyToHideBUA = true;
		}
	}

	var buaApplicationDiv = document.getElementById("buaApplicationDiv");
	buaApplicationDiv.style.display = "none";
	buaApplicationDiv.style.zIndex = 6000;

    window["CVEditContentActionInstance"].restoreDiv();
};

EditContentAction.prototype.cancelBUALoad = function()
{
	var buaIframe = this.getBUAIframe();
	buaIframe.src="";
	buaIframe.parentNode.removeChild(buaIframe);

	window['CVEditContentActionInstance'].removeLoadingToolsMessage();
	window['CVEditContentActionInstance'].hideBUA();
};

EditContentAction.prototype.restoreDiv = function()
{
	this.removeBlockers();

	var buaAnimationDiv = document.getElementById("buaAnimationDiv");

	var slideRightAnimation = dojo.fx.slideTo(
								{
									node: "buaAnimationDiv",
									duration: this.m_animationDuration,
									left: window["CVEditContentActionInstance"].cachedDivProperties.left,
									top: window["CVEditContentActionInstance"].cachedDivProperties.top
								});

	var restoreAnimation = dojo.animateProperty({
		node : "buaAnimationDiv",
		duration: this.m_animationDuration,
		properties : {
			height : {start: buaAnimationDiv.clientHeight, end : window["CVEditContentActionInstance"].cachedDivProperties.height, units:"px"}, //fill in whatever height/width you need...
			width  : {start: buaAnimationDiv.clientWidth, end : window["CVEditContentActionInstance"].cachedDivProperties.width, units:"px"}
		},
		onEnd: function(){
			window["CVEditContentActionInstance"].cleanUp();
		}
	});

	dojo.fx.combine([slideRightAnimation,restoreAnimation]).play();
};

EditContentAction.prototype.moveBUAToFront = function()
{
	this.removeLoadingToolsMessage();

	var buaApplicationDiv = document.getElementById("buaApplicationDiv");
	if(buaApplicationDiv != null)
	{
		buaApplicationDiv.style.zIndex = 6005;
	}
};

EditContentAction.prototype.setViewerEventReceived = function()
{
	window["CVEditContentActionInstance"].m_bReadyToHideBUA = true;

	if (window["CVEditContentActionInstance"].m_bDoneTransitionComplete)
	{
		window["CVEditContentActionInstance"].transitionFromBUA();
	}
};

EditContentAction.prototype.transitionFromBUA = function()
{
	window["CVEditContentActionInstance"].hideBUA();
	window["CVEditContentActionInstance"].removeBlockers();

	window["CVEditContentActionInstance"].m_bReadyToHideBUA = false;
	window["CVEditContentActionInstance"].m_bDoneTransitionComplete = false;
};

EditContentAction.prototype.setRequestParms = function(params)
{
	EditContentAction.superclass.setRequestParms(params);
	if (params && params["preferencesChanged"])
	{
		this["preferencesChanged"] = params["preferencesChanged"];
	}
};


// Temporary - as this is a binary incompatible API change. Will be removed when both RS and Viewer are dropped
function BUAAdvancedDone() {}

function BUAEvent(eventType)
{
	switch (eventType)
	{
		case "appReady":
			window["CVEditContentActionInstance"].setReportSettings();
			break;
		case "canvasReady":
			window["CVEditContentActionInstance"].moveBUAToFront();
			break;
		case "done": //deprecated
			window["CVEditContentActionInstance"].closeBUA();
			break;
		case "cancelPressed":
			window["CVEditContentActionInstance"].transitionFromBUA();
			break;
		case "donePressed":
			window["CVEditContentActionInstance"].showReportBlocker(6005, "Transparent");
			window["CVEditContentActionInstance"].runUpdatedReportFromBUA();
			break;
		case "doneTransitionComplete":
			window["CVEditContentActionInstance"].m_bDoneTransitionComplete = true;
			if (window["CVEditContentActionInstance"].m_bReadyToHideBUA)
			{
				window["CVEditContentActionInstance"].transitionFromBUA();
			}
			else
			{
				document.getElementById("reportBlocker").style.backgroundColor = "#EEEEEE";
			}
			break;
	}
}

// This action is used to rerender the html output, using the same query
function RedrawAction() {}
RedrawAction.prototype = new ModifyReportAction();
RedrawAction.prototype.reuseQuery = function() { return true; };
RedrawAction.prototype.keepRAPCache = function() { return false; };

RedrawAction.prototype.getUndoHint = function()
{
	return RV_RES.IDS_JS_ADVANCED_EDITING;
};

RedrawAction.prototype.addActionContext = function()
{
	return "<reportActions><reportAction name=\"GetInfo\"/></reportActions>";
};



