/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */
function FilterAction() {
	this.m_sAction = "Filter";
	this.m_sType = "";
	this.m_sItem = "";
	this.m_sFormattedNumber= "";
	this.m_sFormattedEndNumber= "";
	this.m_jsonDetails = "";
}


FilterAction.prototype = new ModifyReportAction();


FilterAction.prototype.genSelectionContextWithUniqueCTXIDs = function() { return true; };

FilterAction.prototype.getUndoHint = function()
{
	if (this.m_sType.indexOf("remove") != -1) {
		return RV_RES.IDS_JS_REMOVE_FILTER;
	} else {
		return RV_RES.IDS_JS_FILTER;
	}
};

FilterAction.prototype.setRequestParms = function( parms )
{
	if (parms.type!=null && typeof parms.type != "undefined") {
		this.m_sType = parms.type;
		if (parms.id!=null && typeof parms.id != "undefined") {
			this.m_sId   = parms.id;
		}
		if (parms.item!=null && typeof parms.item != "undefined") {
			this.m_sItem = parms.item;
		}
		if (parms.details) {
			this.m_jsonDetails = parms.details;
		}
		if (parms.formattedNumber!=null && typeof parms.formattedNumber != "undefined") {
			this.m_sFormattedNumber = parms.formattedNumber;
		}
		if (parms.formattedEndNumber!=null && typeof parms.formattedEndNumber != "undefined") {
			this.m_sFormattedEndNumber = parms.formattedEndNumber;
		}
	} else {
		this.m_sType = parms;
	}
};

FilterAction.prototype.addActionContextAdditionalParms = function()
{
	var parms = "<type>" + this.m_sType + "</type>";
	if (this.m_sId != null && typeof this.m_sId != "undefined") {
		parms+= ("<id>" + xml_encode(this.m_sId) + "</id>");
	}
	if (this.m_sItem != null && typeof this.m_sItem != "undefined" && this.m_sItem!="") {
		parms+= ("<item>" + xml_encode(this.m_sItem) + "</item>");
	}
	if (this.m_jsonDetails && this.m_jsonDetails!="")
	{
		parms+= "<details>" + xml_encode(this.m_jsonDetails) + "</details>";
	}
	if (this.m_sFormattedNumber != null && typeof this.m_sFormattedNumber != "undefined" && this.m_sFormattedNumber!="") {
		parms+= ("<formattedNumber>" + this.m_sFormattedNumber + "</formattedNumber>");
	}
	if (this.m_sFormattedEndNumber != null && typeof this.m_sFormattedEndNumber != "undefined" && this.m_sFormattedEndNumber!="") {
		parms+= ("<formattedEndNumber>" + this.m_sFormattedEndNumber + "</formattedEndNumber>");
	}
	return parms;
};

FilterAction.prototype.buildSelectedItemsString = function(selectedObjects)
{
	var itemsLabel = "";
	var numberOfSelectedItems = selectedObjects.length;
	var numberOfItemsToAdd = numberOfSelectedItems > 5 ? 5 : numberOfSelectedItems;

	for(var index = 0; index < numberOfSelectedItems; ++index)
	{
		var value = this.getItemLabel(selectedObjects[index]);

		if (typeof value == "undefined" || value == "")
		{
			 //a selectedObject has neither displayValue nor useValue.
			return "";
		}
		if((index) < numberOfItemsToAdd)
		{
			itemsLabel += value;
		}
		if((index+1) < numberOfItemsToAdd)
		{
			itemsLabel += ", ";
		}
	}

	if(numberOfSelectedItems > 5)
	{
		itemsLabel += ", ++";
	}

	return itemsLabel;
};

FilterAction.prototype.getItemLabel = function (selectedObject)
{
	var value = selectedObject.getDisplayValues()[0];
	if (typeof value == "undefined")
	{
		value = selectedObject.getUseValues()[0];
	}
	return value;
};

FilterAction.prototype.toggleMenu = function(jsonSpec, enabled)
{
	if (enabled)
	{
		jsonSpec.iconClass = "filter";
		jsonSpec.disabled = false;
	}
	else
	{	jsonSpec.iconClass = "filterDisabled";
		jsonSpec.disabled = true;
	}
	return jsonSpec;
};

FilterAction.prototype.updateMenu = function(jsonSpec)
{
	jsonSpec.visible = (this.ifContainsInteractiveDataContainer() && !this.detailFilteringIsDisabled());
	var result = jsonSpec;
	if (jsonSpec.visible) {
		if (this.m_oCV.getSelectionController().getAllSelectedObjects().length == 0 || !this.isSelectionFilterable()) {
			result = this.toggleMenu(jsonSpec, false);
		}
		else {
			this.buildMenu(jsonSpec);
			if (jsonSpec.disabled == true) {
				result = this.toggleMenu(jsonSpec, false);
			}
			else {
				result = this.buildDynamicMenuItem(jsonSpec, "Filter");
			}
		}
	}
	return result;
};

FilterAction.prototype.detailFilteringIsDisabled = function()
{
	var reportInfoObject =  this.getCognosViewer().envParams["rap.reportInfoObject"];
	if (typeof reportInfoObject != "undefined" &&
		typeof reportInfoObject.reportLevelProperties != "undefined" &&
		reportInfoObject.reportLevelProperties != null &&
		reportInfoObject.reportLevelProperties.detailFilteringDisabled == true) {
		return true;
	}
	return false;
};

FilterAction.prototype.buildMenu = function(jsonSpec)
{

	jsonSpec.visible = (this.ifContainsInteractiveDataContainer() &&
						!this.detailFilteringIsDisabled());
	if (! jsonSpec.visible)
	{
		return jsonSpec;
	}

	if(!this.isSelectionFilterable())
	{
		return this.toggleMenu(jsonSpec, false);
	}
	this.toggleMenu(jsonSpec, true);

	var filterItems = [];
	var filterValueActionsAdded = false;
	var renderedRemoveSeparator = false;

	var selectionController = this.m_oCV.getSelectionController();
	var selectedObjects = selectionController.getAllSelectedObjects();

	if(selectedObjects.length > 0 && selectionController.selectionsInSameDataContainer() && selectionController.selectionsFromSameDataItem())
	{
		var refDataItem = selectedObjects[0].getDataItems()[0][0];
		filterValueActionsAdded = this.addFilterValueActionsToMenu(selectionController, filterItems, refDataItem);

		if (this.isRemoveItemFilterValid(refDataItem)) {
			if (filterValueActionsAdded == true) {
				filterItems.push({separator: true});
				renderedRemoveSeparator=true;
			}

			var label = this.getRefDataItemLabel(refDataItem);
			filterItems.push({ name: "RemoveFilterFor", label: RV_RES.IDS_JS_REMOVE_FILTER_FOR + " " + label, iconClass: "", action: { name: "Filter", payload: "remove" }, items: null });
		}
	}

	if (this.isRemoveAllValid()==true) {
		if (filterValueActionsAdded && !renderedRemoveSeparator) {
			filterItems.push({separator: true});
		}
		filterItems.push({ name: "RemoveAllFiltersForWidget", label: RV_RES.IDS_JS_REMOVE_ALL_FILTERS_FOR_WIDGET, iconClass: "", action: { name: "Filter", payload: "removeAllForWidget" }, items: null });
	}

	if(filterItems.length == 0)
	{
		return this.toggleMenu(jsonSpec, false);

	} else	{
		jsonSpec.items = filterItems;
		this.toggleMenu(jsonSpec, true);
		return jsonSpec;
	}

};

FilterAction.prototype.getRefDataItemLabel = function(refDataItem)
{
	var label = refDataItem;
	var aRapInfo = this.m_oCV.envParams["rap.reportInfo"];
	if (aRapInfo)
	{
		var rapInfoLength = aRapInfo.length;
		for (var rapInfoIndex=0; rapInfoIndex < rapInfoLength; rapInfoIndex++) {
			var rapInfo = aRapInfo[rapInfoIndex];
			if (rapInfo.filter) {
				var filterLength = rapInfo.filter.length;
				for (var iFilterIndex=0; iFilterIndex < filterLength; iFilterIndex++) {
					var filterInfo = rapInfo.filter[iFilterIndex];
					if (filterInfo.item == refDataItem && filterInfo.itemLabel && filterInfo.itemLabel.length > 0) {
						label = filterInfo.itemLabel;
						break;
					}
				}
			}
		}
	}

	return label;
};

FilterAction.prototype.addFilterValueActionsToMenu = function(selectionController, filterItems, refDataItem)
{
	var selectedObjects = selectionController.getAllSelectedObjectsWithUniqueCTXIDs();

	// Don't add the filter actions if the data container is a list and the list
	// column header is selected
	var numberOfSelectedValues = selectedObjects.length;
	var sel = 0;
	if (selectedObjects[0].m_dataContainerType=='list') {
		for (sel=0; sel<selectedObjects.length; ++sel) {
			if (selectedObjects[sel].m_sLayoutType == 'columnTitle') {
				numberOfSelectedValues=0;
				break;
			}
		}
	}

	if (numberOfSelectedValues == 0)
	{
		return false;
	}
	var itemsLabel = this.buildSelectedItemsString(selectedObjects);
	if (itemsLabel == "")
	{
		if (numberOfSelectedValues == 1 && selectedObjects[0].getLayoutType() == "datavalue")
		{
			itemsLabel = RV_RES.IDS_JS_NULL;
			filterItems.push({ name: "InFilter", label: CViewerCommon.getMessage(RV_RES.IDS_JS_FILTER_INCLUDE, itemsLabel), iconClass: "", action: { name: "Filter", payload: "in" }, items: null });
			filterItems.push({ name: "NotInFilter", label: CViewerCommon.getMessage(RV_RES.IDS_JS_FILTER_EXCLUDE, itemsLabel), iconClass: "", action: { name: "Filter", payload: "not" }, items: null });
		}
	}
	else {
		if (selectionController.selectionsAreDateTime() || 
			(selectionController.selectionsHaveCalculationMetadata() && !selectionController.selectionsNonMeasureWithMUN() )) {
			// Don't present numeric filtering options if column titles are
			// selected
			for (sel=0; sel<numberOfSelectedValues; ++sel) {
				if (selectedObjects[sel].m_sLayoutType == 'columnTitle') {
					return false;
				}
			}
			if(numberOfSelectedValues == 1)
			{
				if (selectedObjects[0].getUseValues()[0][0]) {
					filterItems.push({ name: "LessFilter", label: RV_RES.IDS_JS_FILTER_LESS_THAN + " " + itemsLabel, iconClass: "", action: { name: "Filter", payload: { type:"lessThan", formattedNumber: itemsLabel }}, items: null });
					filterItems.push({ name: "LessEqualFilter", label: RV_RES.IDS_JS_FILTER_LESS_THAN_EQUAL + " " + itemsLabel, iconClass: "", action: { name: "Filter", payload: { type:"lessThanEqual", formattedNumber: itemsLabel }}, items: null });
					filterItems.push({ name: "GreaterEqualFilter", label: RV_RES.IDS_JS_FILTER_GREATER_THAN_EQUAL + " " + itemsLabel, iconClass: "", action: { name: "Filter", payload: { type:"greaterThanEqual", formattedNumber: itemsLabel }}, items: null });
					filterItems.push({ name: "GreaterFilter", label: RV_RES.IDS_JS_FILTER_GREATER_THAN + " " + itemsLabel, iconClass: "", action: { name: "Filter", payload: { type:"greaterThan", formattedNumber: itemsLabel }}, items: null });
				}
			}
			else if(numberOfSelectedValues == 2)
			{
				if (selectedObjects[0].getUseValues()[0][0] && selectedObjects[1].getUseValues()[0][0]) {
					var formattedNumber = this.getItemLabel(selectedObjects[0]);
					var formattedEndNumber = this.getItemLabel(selectedObjects[1]);
					filterItems.push({ name: "BetweenFilter", label: CViewerCommon.getMessage(RV_RES.IDS_JS_FILTER_BETWEEN, [formattedNumber, formattedEndNumber]), iconClass: "", action: { name: "Filter", payload: {type: "between", formattedNumber: formattedNumber, formattedEndNumber: formattedEndNumber}}, items: null });
					filterItems.push({ name: "NotBetweenFilter", label: CViewerCommon.getMessage(RV_RES.IDS_JS_FILTER_NOT_BETWEEN, [formattedNumber, formattedEndNumber]), iconClass: "", action: { name: "Filter", payload: {type: "notBetween", formattedNumber: formattedNumber, formattedEndNumber: formattedEndNumber }}, items: null });
				}
			}
			else
			{
				return false;
			}
		}
		else
		{
			var containerType = selectionController.getDataContainerType();
			if (containerType == "crosstab" && selectedObjects[0].getLayoutType() == 'columnTitle')	{
				var reportInfo = this.m_oCV.envParams["rap.reportInfo"];
				if (this.isSelectSingleMember(selectedObjects[0], reportInfo)==true)	{
					return false;
				}
			}
			filterItems.push({ name: "InFilter", label: CViewerCommon.getMessage(RV_RES.IDS_JS_FILTER_INCLUDE, itemsLabel), iconClass: "", action: { name: "Filter", payload: "in" }, items: null });
			filterItems.push({ name: "NotInFilter", label: CViewerCommon.getMessage(RV_RES.IDS_JS_FILTER_EXCLUDE, itemsLabel), iconClass: "", action: { name: "Filter", payload: "not" }, items: null });
		}
	}

	return true;
};

FilterAction.prototype.isRemoveAllValid = function()
{
	// If the reportInfo has any filters for any container, remove all is valid.
	var reportInfo = this.m_oCV.envParams["rap.reportInfo"];
	if(typeof reportInfo != "undefined")
	{
		for(var containerIdx = 0; containerIdx < reportInfo.length; ++containerIdx)
		{
			if(typeof reportInfo[containerIdx].filter != "undefined")
			{
				return true;
			}
		}
	}
	return false;
};

FilterAction.prototype.isRemoveItemFilterValid = function(refDataItem)
{
	var containerId = this.getContainerId(this.m_oCV.getSelectionController());
	var reportInfo = this.m_oCV.envParams["rap.reportInfo"];
	if(containerId!=null && typeof reportInfo != "undefined")
	{
		for(var containerIdx = 0; containerIdx < reportInfo.length; ++containerIdx)
		{
			if(containerId==reportInfo[containerIdx].container && typeof reportInfo[containerIdx].filter != "undefined")
			{
				// This container has filters....does it filter this item?
				for (var item = 0; item < reportInfo[containerIdx].filter.length; ++item)
				{
					if (refDataItem == reportInfo[containerIdx].filter[item].item) {
						return true;
					}
				}
				return false;
			}
		}
	}
	return false;
};

FilterAction.prototype.isSelectionFilterable = function() {

	if (this.isSelectionOnChart()) {
		return false;
	}

	var selectionController = this.m_oCV.getSelectionController();

	var selectedObjects = selectionController.getAllSelectedObjects();
	if (selectedObjects.length > 0 && selectedObjects[0].getCellRef() && selectedObjects[0].getCellRef().getAttribute("no_data_item_column") === "true") {
		return false;
	}
	return true;
};

function GetFilterInfoAction()
{
	this.m_requestParms = null;
}

GetFilterInfoAction.prototype = new ModifyReportAction();

GetFilterInfoAction.prototype.isUndoable = function()
{
	return false;
};

GetFilterInfoAction.prototype.setRequestParms = function( parms )
{
	this.m_requestParms = parms;
};

GetFilterInfoAction.prototype.runReport = function()
{
	return false;
};

GetFilterInfoAction.prototype.updateInfoBar = function()
{
	return false;
};

GetFilterInfoAction.prototype.fireModifiedReportEvent = function()
{
	// do nothing
};

GetFilterInfoAction.prototype.buildActionContextAdditionalParmsXML = function()
{
	var itemDoc = XMLBuilderCreateXMLDocument("item");
	var itemElement = itemDoc.documentElement;
	for( var parm in this.m_requestParms )
	{
		if( this.m_requestParms.hasOwnProperty( parm ) )
		{
			var parameterElement = itemDoc.createElement( parm );
			parameterElement.appendChild( itemDoc.createTextNode( this.m_requestParms[parm]));
			itemElement.appendChild( parameterElement );
		}
	}

	return itemDoc;
};

GetFilterInfoAction.prototype.addActionContextAdditionalParms = function()
{
	if( this.m_requestParms === null )
	{
		return "";
	}
	return XMLBuilderSerializeNode( this.buildActionContextAdditionalParmsXML() );
};

GetFilterInfoAction.prototype.createFilterInfoDispatcherEntry = function(isAsynch)
{
	var filterInfoDispatcherEntry = new ReportInfoDispatcherEntry(this);

	if(isAsynch == true) {
		var dummyArg = 'dummyArg';
		var callbacks = {customArguments:[dummyArg],
							"complete":{"object":filterInfoDispatcherEntry,
										"method":filterInfoDispatcherEntry.onComplete},
							"fault":{"object":filterInfoDispatcherEntry,"method":filterInfoDispatcherEntry.onFault},
							"working":{"object":filterInfoDispatcherEntry, "method":filterInfoDispatcherEntry.onWorking},
							"passportTimeout":{"object":filterInfoDispatcherEntry,"method":filterInfoDispatcherEntry.onFault}
						};
		filterInfoDispatcherEntry.m_asynchRequest = this.m_oCV.createDispatcherEntryJSONServerRequest(callbacks, isAsynch);
	} else {
		filterInfoDispatcherEntry.m_asynchRequest = new XmlHttpObject();
	}
	return filterInfoDispatcherEntry;
};

GetFilterInfoAction.prototype.executeUsingActionDispatcher = function() {
	var filterInfoDispatcherEntry = this.createFilterInfoDispatcherEntry(true);
	filterInfoDispatcherEntry.initialize(true);
	//filterInfoDispatcherEntry.addFormField("cv.responseFormat", "json");

	this.m_oCV["LAST_ACTION_PARAMS"] = {
		retryAction: typeof this.m_sRetryClass != "undefined" ? this.m_sRetryClass : this.m_sAction,
		retryActionRequestParams: this.m_requestParms
	};

	this.m_oCV.getViewerWidget().getActionDispatcher().dispatchRequest(filterInfoDispatcherEntry);
	this.fireModifiedReportEvent();
};

GetFilterInfoAction.prototype.execute = function() {
	this.getCognosViewer().setKeepFocus(this.keepFocusOnWidget());
	if (this.getCognosViewer().getViewerWidget().getActionDispatcher()!=null) {
		this.executeUsingActionDispatcher();
		return;
	}
	var cognosViewerRequest = this.createCognosViewerRequest("modifyReport");
	cognosViewerRequest.addOption("cv.responseFormat", "json");
	cognosViewerRequest.setCallback( this.getCallback() );

	this.getCognosViewer().sendRequest(cognosViewerRequest);

};

function GetFilterValuesAction()
{
	this.m_sAction = 'CollectFilterValues';
	this.m_sRetryClass = 'GetFilterValues';
}

GetFilterValuesAction.prototype = new GetFilterInfoAction();

GetFilterValuesAction.prototype.addActionContextAdditionalParms = function()
{
	if( this.m_requestParms === null )
	{
		return "";
	}
	var itemDoc = this.buildActionContextAdditionalParmsXML();
	var itemElement = itemDoc.documentElement;
	for( var parm in this.m_requestParms ) {
		if (parm == "name") {
			// If we can find a min and max, send it....
			var selectionController = this.m_oCV.getSelectionController();
			if (typeof selectionController != "undefined" &&
				typeof selectionController.getCCDManager() != "undefined") {
				var minMax = selectionController.getCCDManager().GetPageMinMaxForRDI(this.m_requestParms[parm]);
				if (typeof minMax != "undefined") {
					var pageMinElement = itemDoc.createElement( "pageMin" );
					pageMinElement.appendChild( itemDoc.createTextNode( minMax.pageMin ) );
					var pageMaxElement = itemDoc.createElement( "pageMax" );
					pageMaxElement.appendChild( itemDoc.createTextNode( minMax.pageMax ) );
					itemElement.appendChild(pageMinElement);
					itemElement.appendChild(pageMaxElement);
				}
				if (this.m_oCV.isSinglePageReport() == true) {
					var singlePage = itemDoc.createElement("singlePageReport");
					itemElement.appendChild(singlePage);
				}
			}
			break;
		}
	}

	var additionalParms = XMLBuilderSerializeNode( itemDoc );
	return (additionalParms + this.addClientContextData(/*maxValuesPerRDI*/3));
};

GetFilterValuesAction.prototype.getCallback = function()
{
	var viewer = this.getCognosViewer();
	var viewerWidget = viewer.getViewerWidget();

	var callback = function(response) {
		clearLastAction(viewer);
		viewer.hideExecutionCursor();
		viewerWidget.handleGetFilterValuesResponse(response);
	};

	return callback;
};

GetFilterValuesAction.prototype.canBeQueued = function() { return true; };
GetFilterValuesAction.prototype.getActionKey = function()
{
	if (typeof this.m_requestParms != "undefined" &&
		typeof this.m_requestParms.source != "undefined") {
			return this.m_sAction+this.m_requestParms.source;
	}
	return null;
};

function GetFilterableItemsAction()
{
	this.m_sAction = 'CollectFilterableItems';
}

GetFilterableItemsAction.prototype = new GetFilterInfoAction();

GetFilterableItemsAction.prototype.addActionContextAdditionalParms = function()
{
	return this.addClientContextData(/*maxValuesPerRDI*/3);
};

GetFilterableItemsAction.prototype.getCallback = function()
{
	var viewer = this.getCognosViewer();
	var viewerWidget = viewer.getViewerWidget();

	var callback = function(response) {
		clearLastAction(viewer);
		viewer.hideExecutionCursor();
		viewerWidget.handleGetFilterableItemsResponse(response);
	};

	return callback;
};

function UpdateDataFilterAction()
{
	this.m_sAction = "UpdateDataFilter";
}

UpdateDataFilterAction.prototype = new ModifyReportAction();

UpdateDataFilterAction.prototype.getActionKey = function()
{
	if (typeof this.m_requestParams != "undefined") {
		try
		{
			var parms = eval("(" + this.m_requestParams + ")");
			if (parms.clientId != null) {
				return this.m_sAction + parms.clientId;
			}
		}
		catch (e)
		{
			// If eval fails for any reason, return a null actionKey.
		}
	}
	return null;
};

UpdateDataFilterAction.prototype.canBeQueued = function()
{
	return true;
};

UpdateDataFilterAction.prototype.keepFocusOnWidget = function()
{
	return false;
};

UpdateDataFilterAction.prototype.isUndoable = function()
{
	return false;
};

UpdateDataFilterAction.prototype.setRequestParms = function( parms )
{
	this.m_requestParams = parms;
};

UpdateDataFilterAction.prototype.addActionContext = function()
{
	var actionContext = "<reportActions>";
	actionContext += "<reportAction name=\"" + this.m_sAction + "\">";

	// xml encode if it does not start with filter
	var reFilterPayload = new RegExp("^<filter>");
	var actionParams = null;
	if (! this.m_requestParams.match(reFilterPayload) ) {
		actionParams = xml_encode(this.m_requestParams);
	}
	else {
		actionParams = this.m_requestParams;
	}

	actionContext += actionParams;

	actionContext += "</reportAction>";
	actionContext += "<reportAction name=\"GetInfo\"><include><sliders/></include></reportAction>";
	actionContext += "</reportActions>";

	return actionContext;
};



