/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

//NEW DISPATCHER SUPPORT....

//DISPATCHER ENTRY - generic class representing a single dispatched request.
//treat as an abstract class.
//Derived classes must implement onStart(), onComplete(), m_asynchRequest.sendRequest() at minimum.
function DispatcherEntry()
{
	this.m_requestKey    = null;
	this.m_canBeQueued	 = false;
	this.m_asynchRequest = null;
}

DispatcherEntry.prototype.canBeQueued = function() { return this.m_canBeQueued; };
DispatcherEntry.prototype.getKey = function() { return this.m_requestKey; };

//Helpers...
DispatcherEntry.prototype.getFormField = function(name)
{
	if (this.m_asynchRequest!=null) {
		return this.m_asynchRequest.getFormField(name);
	} else {
		return "";
	}
};
DispatcherEntry.prototype.addFormField = function(name, value)
{
	if (this.m_asynchRequest!=null) {
		this.m_asynchRequest.addFormField(name, value);
	}
};
DispatcherEntry.prototype.addDefinedFormField = function(name, value)
{
	if (typeof value != "undefined") {
		this.addFormField(name, value);
	}
};
DispatcherEntry.prototype.addNonNullFormField = function(name, value)
{
	if (value != null) {
		this.addFormField(name, value);
	}
};

//DISPATCHER ENTRY for interactive Actions (modifyReportAction)....
function ActionDispatcherEntry()
{
	this.m_requestKey	 = null;
	this.m_canBeQueued	 = false;
	this.m_oCV			 = null;
	this.m_action		 = null;
	this.m_viewerWidget	 = null;
}
ActionDispatcherEntry.prototype = new DispatcherEntry();

ActionDispatcherEntry.prototype.initializeAction = function(action)
{
	this.m_requestKey	 = action.getActionKey();
	this.m_canBeQueued	 = action.canBeQueued();
	this.m_oCV			 = action.m_oCV;
	this.m_action		 = action;
	this.m_viewerWidget	 = action.m_oCV.getViewerWidget();
};

ActionDispatcherEntry.prototype.onStart = function()
{
	//The request for a dispatcher entry is starting
	//get the latest spec, conversation and tracking and store undo info.
	this.m_oCV.showExecutionCursor();

	if(typeof this.m_oCV.envParams["ui.spec"] != "undefined")
	{
		this.addFormField("ui.spec", this.m_oCV.envParams["ui.spec"]);
	}
	this.addFormField("ui.conversation", encodeURIComponent(this.m_oCV.getConversation()));
	this.addFormField("m_tracking", encodeURIComponent(this.m_oCV.getTracking()));
	
	if (this.m_action.isUndoable())
	{
		this.m_action.getUndoRedoQueue().initUndoObj({"tooltip" : this.m_action.getUndoHint(), "saveSpec" : this.m_action.saveSpecForUndo()});
	}

};

ActionDispatcherEntry.prototype.onComplete = function(asynchDATAResponse, arg1)
{
	//The request for a single entry has completed...
	if (this.m_viewerWidget.getActionDispatcher().queueIsEmpty()==true) {
		this.m_oCV.processResponseText(asynchDATAResponse.getResponseText());
	} else {
		//TODO: may need to define something in the action to limit the "avoid rendering"
		this.m_oCV.processResponseState(asynchDATAResponse.getResponseState());
	}

	if (this.m_viewerWidget.getActionDispatcher()!=null) {
		this.m_oCV.hideExecutionCursor();
		this.m_viewerWidget.getActionDispatcher().requestComplete(this);
	}
};

ActionDispatcherEntry.prototype.onWorking = function(asynchDATAResponse, arg1)
{
	var responseState = this.m_asynchRequest.getResponseState();
	if (responseState) {
		if (typeof responseState.rapReportInfo != "undefined") {
			this.m_oCV.envParams["rap.reportInfoObject"] = eval("(" + responseState.rapReportInfo + ")");
			this.m_oCV.envParams["rap.reportInfo"] = this.m_oCV.envParams["rap.reportInfoObject"].containers;
			this.m_oCV.envParams["json.infoBar"] = responseState.rapReportInfo;
		}
		if (typeof responseState.rapState != "undefined") {
			this.m_oCV.envParams["rap.state"] = responseState.rapState;
		}
		if (typeof responseState.spec != "undefined") {
			if(responseState.spec.indexOf("&lt;") == 0)	{
				responseState.spec = xml_decode(responseState.spec);
			}
			this.m_oCV.envParams["ui.spec"] = responseState.spec;
		}
	}
	this.sendWaitRequest(responseState);
	var cancelCallback = GUtil.generateCallback(this.onCancel, [], this);
	this.m_oCV.showWaitPage(true, cancelCallback);
	this.m_oCV.hideExecutionCursor();
};

ActionDispatcherEntry.prototype.onFault = function(asynchDATAResponse, arg1)
{
	//The executing request has faulted...stop executing this and ALL queued requests.
	this.m_oCV.setSoapFault(XMLBuilderSerializeNode(asynchDATAResponse.getSoapFault()));
	this.m_oCV.hideExecutionCursor();
	this.m_viewerWidget.resetActionDispatcher();
	this.m_viewerWidget.handleFault();
};

ActionDispatcherEntry.prototype.initialize = function(isAsynch)
{
	this.addFormField("ui.action", "modifyReport");

	if(typeof this.m_oCV.envParams["modelPath"] != "undefined" && this.m_oCV.envParams["modelPath"] != "") {
		this.addFormField("modelPath", this.m_oCV.envParams["modelPath"]);
		if(typeof this.m_oCV.envParams["metaDataModelModificationTime"] != "undefined") {
			this.addFormField("metaDataModelModificationTime", this.m_oCV.envParams["metaDataModelModificationTime"]);
		}
	}

	this.addOptionsToRequest();
	this.addOptionsFromViewer();
};


//Adding Options to the ActionDispatcherEntry....to be cleaned up.
ActionDispatcherEntry.prototype.addOptionsToRequest = function()
{
	this.m_action.preProcess();

	if( this.m_action.doAddActionContext() === true )
	{
		var actionContext = this.m_action.addActionContext();
		this.addFormField("cv.actionContext", actionContext);
		if (typeof g_firefoxPluginHelper != "undefined" && g_firefoxPluginHelper != null)
		{
			g_firefoxPluginHelper.debug('Action context', actionContext, "xml");
		}
	}

	this.addFormField("ui.object", this.m_oCV.envParams["ui.object"]);
	this.addDefinedFormField("ui.spec", this.m_oCV.envParams["ui.spec"]);
	this.addDefinedFormField("modelPath", this.m_oCV.envParams["modelPath"]);
	this.addDefinedFormField("packageBase", this.m_oCV.envParams["packageBase"]);
	this.addDefinedFormField("rap.state", this.m_oCV.envParams["rap.state"]);
	this.addDefinedFormField("rap.reportInfo", this.m_oCV.envParams["json.infoBar"]);

	//MARK: Only for runReportAction?
	//this.addAdditionalOptions(cognosViewerRequest);
};

ActionDispatcherEntry.prototype.addOptionsFromViewer = function()
{
	//Adapted from: CCognosViewer.generateRequestParams().

	//MARK: do we need this...its a primary request
	this.addDefinedFormField("ui.primaryAction", this.m_oCV.envParams["ui.primaryAction"]);
	this.addNonNullFormField("cv.header", this.m_oCV.envParams["cv.header"]);
	this.addNonNullFormField("cv.toolbar", this.m_oCV.envParams["cv.toolbar"]);	//MARK: needed?
	this.addNonNullFormField("m_session", this.m_oCV.envParams["m_session"]); //MARK: needed?

	//MARK: looks like a mistake in CCognosViewer version??
	this.addNonNullFormField("cv.showFaultPage", this.m_oCV.envParams["cv.showFaultPage"]);

	this.addNonNullFormField("cv.debugDirectory", this.m_oCV.envParams["cv.debugDirectory"]);
	this.addNonNullFormField("ui.objectClass", this.m_oCV.envParams["ui.objectClass"]);
	this.addNonNullFormField("bux", this.m_oCV.envParams["bux"]);
	this.addNonNullFormField("baseReportModificationTime", this.m_oCV.envParams["baseReportModificationTime"]);
	this.addNonNullFormField("originalReport", this.m_oCV.envParams["originalReport"]);

	//Flash chart option
	var flashChartOptionValue = this.m_oCV.getFlashChartOption();
	if( flashChartOptionValue != null)
	{
		this.addFormField("savedFlashChartOption", flashChartOptionValue);
		if (flashChartOptionValue && this.m_action !=null && typeof(this.m_action.m_requestParams)!= "undefined" && typeof(this.m_action.m_requestParams.targetType)!= "undefined")	{
			var hasAVSChart = false;
			var sTarget = this.m_action.m_requestParams.targetType;
			if (sTarget.match('v2_') != null || sTarget.match('_v2') != null)
			{
				hasAVSChart = true;
			}
			else
			{
				var selectedReportInfo = this.m_action.getSelectedReportInfo();
				if (selectedReportInfo != null)
				{
					var reportInfo = this.m_oCV.envParams["rap.reportInfo"];
					for (var i=0; i <reportInfo.length; ++i)
					{
						if (reportInfo[i].container == selectedReportInfo.container)
						{
							continue;
						}
						if (reportInfo[i].displayTypeId.match('v2_') != null || reportInfo[i].displayTypeId.match('_v2') != null)
						{
							hasAVSChart = true;
							break;
						}
					}
				}
			}
			this.addFormField("hasAVSChart", hasAVSChart);
		}
		else
		{
			this.addFormField("hasAVSChart", this.m_oCV.hasAVSChart());
		}
	}

	var sEP = this.m_oCV.getExecutionParameters();
	if (sEP) {
		this.addFormField("executionParameters", encodeURIComponent(sEP));
	}

	var sSP = this.m_oCV.getSessionPrompts();
	if (sSP) {
		this.addFormField("session_prompts", encodeURIComponent(sSP));
	}

	this.addFormField("ui.conversation", encodeURIComponent(this.m_oCV.getConversation())); //MARK: needed? (its a primary request)
	this.addFormField("m_tracking", encodeURIComponent(this.m_oCV.getTracking())); //MARK: needed? (its a primary request)

	var sCAF = this.m_oCV.getCAFContext();
	if (sCAF) {
		this.addFormField("ui.cafcontextid", sCAF);
	}
	if (this.m_oCV.envParams["ui.routingServerGroup"] != null)
	{
		this.addFormField("ui.routingServerGroup", encodeURIComponent(this.m_oCV.envParams["ui.routingServerGroup"]));
	}
};

ActionDispatcherEntry.prototype.onCancel = function()
{
	this.m_asynchRequest.cancel();
	var asynchRequest = this.m_asynchRequest.newAsynchRequest();
	asynchRequest.addFormField("ui.action", "cancel");
	asynchRequest.addFormField("m_tracking", this.m_oCV.getTracking());
	this.m_oCV.setTracking("");

	if (this.m_oCV.envParams["bux"] != null)
	{
		asynchRequest.addFormField("bux", this.m_oCV.envParams["bux"]);
	}

	asynchRequest.sendRequest();

	this.m_viewerWidget.resetActionDispatcher();
	this.m_oCV.showWaitPage(false);
};

ActionDispatcherEntry.prototype.sendWaitRequest = function(responseState) {

	if (responseState != null && typeof responseState.rapWait != "undefined")
	{
		this.m_oCV.setTracking(responseState.tracking);
		this.m_asynchRequest.addFormField("m_tracking", responseState.tracking);
		this.m_asynchRequest.addFormField("rapWait", "true");
		if (typeof responseState.rapRequestCache != "undefined") {
			this.m_asynchRequest.addFormField("rapRequestCache", responseState.rapRequestCache);
		}
		this.m_asynchRequest = this.m_asynchRequest.resubmitRequest();
	}
	else
	{
		var asynchRequest = this.m_asynchRequest.newAsynchRequest();
		asynchRequest.addFormField("ui.action", "wait");

		var tracking = this.m_asynchRequest.getTracking();
		this.m_oCV.setTracking(tracking);
		asynchRequest.addFormField("m_tracking", tracking);

		asynchRequest.addFormField("ui.primaryAction", this.m_asynchRequest.getPrimaryAction());
		asynchRequest.addFormField("cv.actionState", this.m_asynchRequest.getActionState());
		if (this.m_oCV.envParams["bux"] != null) {
			asynchRequest.addFormField("bux", this.m_oCV.envParams["bux"]);
		}
		asynchRequest.sendRequest();

		this.m_asynchRequest = asynchRequest;
	}
};

//DISPATCHER ENTRY for information Actions (eg: collectFilterableItems, collectFilterValues, getInfo)
function ReportInfoDispatcherEntry(action)
{
	this.initializeAction(action);
}

ReportInfoDispatcherEntry.prototype = new ActionDispatcherEntry();
ReportInfoDispatcherEntry.prototype.onStart = function()
{
	//Nothing at start-of-action needed...
};

ReportInfoDispatcherEntry.prototype.onComplete = function(asynchJSONResponse, arg1)
{
	//The request for a single entry has completed...
	if (this.m_viewerWidget.getActionDispatcher().queueIsEmpty()==true) {
		var callbackFunction = this.m_action.getCallback();
		callbackFunction(asynchJSONResponse);
	}

	if (this.m_viewerWidget.getActionDispatcher()!=null) {
		this.m_oCV.hideExecutionCursor();
		this.m_viewerWidget.getActionDispatcher().requestComplete(this);
	}
};

ReportInfoDispatcherEntry.prototype.onWorking = function(asynchDATAResponse, arg1)
{
	var jsonResponseObject = this.m_asynchRequest.getJSONResponseObject();
	this.sendWaitRequest(jsonResponseObject);
};


